unit
  FRM_IB_Base;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Registry,

  IB_Components,
  IB_ConnectionSource,
  IB_TransactionSource;

type
  TfrmBase = class(TForm)
    StatusBar: TStatusBar;
    IB_TransactionSource: TIB_TransactionSource;
    IB_ConnectionSource: TIB_ConnectionSource;
    pnClose: TPanel;
    pnOKCancel: TPanel;
    btOK: TButton;
    btCancel: TButton;
    pnApply: TPanel;
    btApply: TButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btOKClick(Sender: TObject);
    procedure btCancelClick(Sender: TObject);
    procedure btApplyClick(Sender: TObject);
  private
    { Private declarations }
    FRegDB: TRegINIFile;
    FBaseKey: string;
    FSubKey: string;
    FLayout: string;
    FDefaultTop: word;
    FDefaultLeft: word;
    FDefaultHeight: word;
    FDefaultWidth: word;
    LastLayout: string;
    FLayoutNames: TIB_StringList;
    FSaveSettings: boolean;
    FSaveLayout: boolean;
    FSettingsSaved: boolean;
    FLayoutSaved: boolean;
  protected
    { Not intended for use but made protected and vitual just in case. }
    procedure SetBaseKey( AValue: string ); virtual;
    procedure SetSubKey( AValue: string ); virtual;
    procedure SetLayout( AValue: string ); virtual;
    function GetFullKey: string; virtual;
    function GetIB_Connection: TIB_Connection; virtual;
    procedure SetIB_Connection( AValue: TIB_Connection ); virtual;
    function GetIB_Transaction: TIB_Transaction; virtual;
    procedure SetIB_Transaction( AValue: TIB_Transaction ); virtual;
    procedure ReadInSettings; virtual;
    procedure WriteOutSettings; virtual;
    procedure ReadInLayout; virtual;
    procedure WriteOutLayout; virtual;
    { Use these in later sub-class forms }
    procedure DoReadSettings; virtual;
    procedure DoWriteSettings; virtual;
    procedure DoReadLayout; virtual;
    procedure DoWriteLayout; virtual;
    procedure AddLayout( NewLayout: string ); virtual;
    procedure RemoveLayout( OldLayout: string ); virtual;
    procedure WriteStrings( Key: string; Dest: TStrings );
    procedure ReadStrings( Key: string; Dest: TStrings );
    procedure SetDefaultBounds(ALeft, ATop, AWidth, AHeight: Integer);
  public
    { Inherited Methods }
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    { Public declarations }
    property RegDB: TRegINIFile read FRegDB;
    property BaseKey: string read FBaseKey write SetBaseKey;
    property SubKey: string read FSubKey write SetSubKey;
    property FullKey: string read GetFullKey;
    property Layout: string read FLayout write SetLayout;
    property LayoutNames: TIB_StringList read FLayoutNames;
    property SaveSettings: boolean read FSaveSettings write FSaveSettings;
    property SaveLayout: boolean read FSaveLayout write FSaveLayout;
    property SettingsSaved: boolean read FSettingsSaved;
    property LayoutSaved: boolean read FLayoutSaved;
    property IB_Connection: TIB_Connection read GetIB_Connection
                                           write SetIB_Connection;
    property IB_Transaction: TIB_Transaction read GetIB_Transaction
                                             write SetIB_Transaction;
  end;

implementation

{$R *.DFM}

constructor TfrmBase.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FLayout := '';
  FLayoutNames := TIB_StringList.Create;
  FSaveSettings := true;
  FSaveLayout := true;
end;

destructor TfrmBase.Destroy;
begin
  BaseKey := '';
  FLayoutNames.Free;
  FLayoutNames := nil;
  inherited Destroy;
end;

procedure TfrmBase.SetBaseKey( AValue: string );
begin
  if FBaseKey <> AValue then begin
    SubKey := '';
    WriteOutSettings;
    FBaseKey := AValue;
    ReadInSettings;
  end;
end;

procedure TfrmBase.SetSubKey( AValue: string );
begin
  if SubKey <> AValue then begin
    // Determine if the registry is busy somehwere else.
    if ( SubKey = '' ) and ( RegDB <> nil ) then begin
      raise Exception.Create( 'Cannot change sub-key' );
    end;
    // Close any existing INI session.
    if RegDB <> nil then begin
      FRegDB.Free;
      FRegDB := nil;
    end;
    // Update the new value for SubKey.
    FSubKey := AValue;
    // Open the new INI session.
    if SubKey <> '' then begin
      if CompareText( SubKey, 'Current' ) = 0 then begin
        FRegDB := TRegINIFile.Create( FullKey );
      end else begin
        FRegDB := TRegINIFile.Create( FullKey + '\' + SubKey );
      end;
    end;
  end;
end;

procedure TfrmBase.SetLayout( AValue: string );
begin
  AValue := Trim( AValue );
  if FLayout <> AValue then begin
    SubKey := '';
    WriteOutLayout;
    FLayout := AValue;
    if Layout <> '' then begin
      with LayoutNames do if IndexOf( Layout ) = -1 then begin
        AddLayout( Layout );
      end;
    end;
    ReadInLayout;
  end;
end;

function TfrmBase.GetIB_Connection: TIB_Connection;
begin
  Result := IB_ConnectionSource.IB_Connection;
end;

procedure TfrmBase.SetIB_Connection( AValue: TIB_Connection );
begin
  IB_ConnectionSource.IB_Connection := AValue;
end;

function TfrmBase.GetIB_Transaction: TIB_Transaction;
begin
  Result := IB_TransactionSource.IB_Transaction;
end;

procedure TfrmBase.SetIB_Transaction( AValue: TIB_Transaction );
begin
  IB_TransactionSource.IB_Transaction := AValue;
end;

function TfrmBase.GetFullKey: string;
begin
  if Layout <> '' then begin
    Result := BaseKey + '\LAYOUTS\' + Layout;
  end else begin
    Result := BaseKey;
  end
end;

procedure TfrmBase.WriteStrings( Key: string; Dest: TStrings );
var
  ii: integer;
begin
  with FRegDB do begin
    FRegDB.EraseSection( Key );
    if Dest <> nil then for ii := 0 to Dest.Count - 1 do begin
      FRegDB.WriteString( Key, IntToStr( ii ), Dest[ ii ] );
    end;
  end;
end;

procedure TfrmBase.ReadStrings( Key: string; Dest: TStrings );
var
  ii: integer;
begin
  with FRegDB do begin
    ii := 0;
    Dest.BeginUpdate;
    try
      Dest.Clear;
      while ReadString( Key, IntToStr( ii ),'<**>' ) <> '<**>' do begin
        Dest.Add( ReadString( Key, IntToStr( ii ), '' ));
        Inc( ii );
      end;
    finally
      Dest.EndUpdate;
    end;
  end;
end;

procedure TfrmBase.ReadInSettings;
var
  LastLayout: string;
  ATop, ALeft, AHeight, AWidth: integer;
  AMaximized: boolean;
begin
  if BaseKey <> '' then begin
    FRegDB := TRegINIFile.Create( BaseKey );
    try
      with FRegDB do begin
        AMaximized := ReadBool( 'Settings', 'Maximized', false );
        ATop    := ReadInteger( 'Settings', 'TOP',    FDefaultTop );
        ALeft   := ReadInteger( 'Settings', 'LEFT',   FDefaultLeft );
        AHeight := ReadInteger( 'Settings', 'HEIGHT', FDefaultHeight );
        AWidth  := ReadInteger( 'Settings', 'WIDTH',  FDefaultWidth );
      end;
      if ( AHeight <> 0 ) and ( AWidth <> 0 ) then begin
        SetBounds( ALeft, ATop, AWidth, AHeight );
      end;
      if AMaximized then begin
        WindowState := wsMaximized;
      end;
      ReadStrings( 'LayoutNames', LayoutNames );
      DoReadSettings;
      with FRegDB do begin
        LastLayout := ReadString( 'Settings', 'LAYOUT', '' );
      end;
    finally
      FRegDB.Free;
      FRegDB := nil;
    end;
    Layout := LastLayout;
  end;
end;

procedure TfrmBase.WriteOutSettings;
var
  LastLayout: string;
begin
  if Layout <> '' then begin
    LastLayout := Layout;
    Layout := '';
  end;
  if SaveSettings and ( BaseKey <> '' ) then begin
    FRegDB := TRegINIFile.Create( BaseKey );
    try
      with FRegDB do begin
        WriteBool( 'Settings', 'Maximized', WindowState = wsMaximized );
        WindowState := wsNormal;
        WriteInteger( 'Settings','TOP', Top );
        WriteInteger( 'Settings','LEFT', Left );
        WriteInteger( 'Settings','HEIGHT', Height );
        WriteInteger( 'Settings','WIDTH', Width );
        WriteString( 'Settings', 'LAYOUT', LastLayout );
      end;
      DoWriteSettings;
      WriteStrings( 'LayoutNames', LayoutNames );
    finally
      FRegDB.Free;
      FRegDB := nil;
    end;
    FSettingsSaved := true;
  end;
end;

procedure TfrmBase.ReadInLayout;
begin
  if BaseKey <> ''  then begin
    FRegDB := TRegINIFile.Create( BaseKey + '\LAYOUTS\' + Layout );
    try
      DoReadLayout;
    finally
      FRegDB.Free;
      FRegDB := nil;
    end;
  end;
end;

procedure TfrmBase.WriteOutLayout;
begin
  if SaveLayout and ( BaseKey <> '' ) and ( Layout <> '' ) then begin
    FRegDB := TRegINIFile.Create( BaseKey + '\LAYOUTS\' + Layout );
    try
      DoWriteLayout;
    finally
      FRegDB.Free;
      FRegDB := nil;
    end;
    FLayoutSaved := true;
  end;
end;

procedure TfrmBase.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmBase.SetDefaultBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  FDefaultLeft := ALeft;
  FDefaultTop := ATop;
  FDefaultWidth := AWidth;
  FDefaultHeight := AHeight;
end;

procedure TfrmBase.AddLayout( NewLayout: string );
begin
  LayoutNames.Insert( 0, NewLayout );
end;

procedure TfrmBase.RemoveLayout( OldLayout: string );
var
  ii: integer;
begin
  if Layout = OldLayout then begin
    Layout := '';
  end;
  with LayoutNames do begin
    ii := IndexOf( OldLayout );
    if ii <> -1 then begin
      with TRegINIFile.Create( BaseKey + '\LAYOUTS' ) do try
        EraseSection( Strings[ii] );
      finally
        Free;
      end;
      Delete( ii );
    end;
  end;
end;

procedure TfrmBase.btOKClick(Sender: TObject);
begin
  SaveSettings := true;
  SaveLayout := true;
  ModalResult := mrOk;
end;

procedure TfrmBase.btCancelClick(Sender: TObject);
begin
  SaveSettings := false;
  SaveLayout := false;
  ModalResult := mrCancel;
end;

procedure TfrmBase.btApplyClick(Sender: TObject);
begin
  SaveSettings := true;
  SaveLayout := true;
  WriteOutLayout;
  WriteOutSettings;
end;

procedure TfrmBase.DoReadSettings;
begin
// Abstract.
end;

procedure TfrmBase.DoWriteSettings;
begin
// Abstract.
end;

procedure TfrmBase.DoReadLayout;
begin
// Abstract.
end;

procedure TfrmBase.DoWriteLayout;
begin
// Abstract.
end;

end.

