unit
  FRM_IB_Export;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Buttons, StdCtrls, ExtCtrls,

  FRM_IB_Base,
  
  IB_Header,
  IB_Components,
  IB_ConnectionSource,
  IB_TransactionSource,
  IB_DataSetBar,
  IB_StatementSource;

type
  TfrmExport = class(TfrmBase)
    meSQL: TMemo;
    Panel27: TPanel;
    gbASCII: TGroupBox;
    lbDelimeter: TLabel;
    lbSeparator: TLabel;
    rbDelimeted: TRadioButton;
    rbFixed: TRadioButton;
    edDelimeter: TEdit;
    edSeparator: TEdit;
    Panel29: TPanel;
    sbExport: TSpeedButton;
    sbImport: TSpeedButton;
    dsbExport: TIB_DataSetBar;
    Panel30: TPanel;
    crExport: TIB_Cursor;
    stlExport: TIB_StatementSource;
    dslExport: TIB_DataSource;
    sdExport: TSaveDialog;
    procedure meSQLChange(Sender: TObject);
    procedure stlExportAfterPrepare( Sender: TIB_StatementLink;
                                     Statement: TIB_Statement);
    procedure stlExportAfterUnprepare( Sender: TIB_StatementLink;
                                       Statement: TIB_Statement);
    procedure stlExportBeforePrepare( Sender: TIB_StatementLink;
                                      Statement: TIB_Statement);
    procedure sbImportClick(Sender: TObject);
    procedure sbExportClick(Sender: TObject);
    procedure IB_ConnectionSourceAfterAssignment(
      Sender: TIB_ConnectionLink; AConnection: TIB_Connection);
    procedure IB_TransactionSourceAfterAssignment(
      Sender: TIB_TransactionLink; ATransaction: TIB_Transaction);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FileStream: TFileStream;
    Delimeter: char;
    Separator: char;
    procedure crExportExportFixed( IB_DataSet: TIB_DataSet );
    procedure crExportExportDelimeted( IB_DataSet: TIB_DataSet );
  protected
    procedure DoReadSettings; override;
    procedure DoWriteSettings; override;
  public
    { Public declarations }
  end;

const
  EndOfRec: array [0..1] of char = #13#10;

implementation

{$R *.DFM}

procedure TfrmExport.FormCreate(Sender: TObject);
begin
  inherited;
  SetDefaultBounds( 0, 0, 440, 330 );
end;

procedure TfrmExport.DoReadSettings;
begin
  with RegDB do begin
    sdExport.Filename := ReadString( 'Settings', 'Filename', '*.TXT' );
    edDelimeter.Text  := ReadString( 'Settings', 'Delimeter', '"');
    edSeparator.Text  := ReadString( 'Settings', 'Separator', ',');
    ReadStrings( 'SQL', meSQL.Lines );
  end;
end;

procedure TfrmExport.DoWriteSettings;
begin
  with RegDB do begin
    WriteString( 'Settings', 'Filename',  sdExport.Filename );
    WriteString( 'Settings', 'Delimeter', edDelimeter.Text[1] );
    WriteString( 'Settings', 'Separator', edSeparator.Text[1] );
    WriteStrings( 'SQL', meSQL.Lines );
  end;
end;

procedure TfrmExport.IB_ConnectionSourceAfterAssignment(
  Sender: TIB_ConnectionLink; AConnection: TIB_Connection);
begin
  crExport.IB_Connection := AConnection;
end;

procedure TfrmExport.IB_TransactionSourceAfterAssignment(
  Sender: TIB_TransactionLink; ATransaction: TIB_Transaction);
begin
  crExport.IB_Transaction := ATransaction;
end;

{------------------------------------------------------------------------------}

procedure TfrmExport.meSQLChange(Sender: TObject);
begin
  crExport.Prepared := false;
  crExport.SQL.Clear;
end;

procedure TfrmExport.stlExportAfterPrepare(
  Sender: TIB_StatementLink; Statement: TIB_Statement);
begin
  sbImport.Enabled := Statement.StatementType = stInsert;
  sbExport.Enabled := Statement.StatementType = stSelect;
end;

procedure TfrmExport.stlExportAfterUnprepare(
  Sender: TIB_StatementLink; Statement: TIB_Statement);
begin
  sbExport.Enabled := false;
  sbImport.Enabled := false;
end;

procedure TfrmExport.stlExportBeforePrepare(
  Sender: TIB_StatementLink; Statement: TIB_Statement);
begin
  with crExport.SQL do begin
    if Count = 0 then begin
      AddStrings( meSQL.Lines );
    end;
  end;
end;

procedure TfrmExport.sbImportClick(Sender: TObject);
begin
// Not ready yet!
end;

procedure TfrmExport.sbExportClick(Sender: TObject);
var
  ii, OldInc: integer;
  FldLen, FldTot: integer;
  FldType: string;
  SCHFileName: string;
  FileStub: string;
  WriteBuf: string;
begin
  OldInc := 0;
  with crExport, Fields, sdExport do begin
    if Execute then begin
      FileStub := ExtractFilename( FileName );
      FileStub := Copy( FileStub, 1, Pos( '.', FileStub ) - 1 );
      SCHFileName := ExtractFileDir( Filename ) + '\' + FileStub + '.SCH';
      if FileExists( FileName ) then begin
        DeleteFile( Filename );
      end;
      if FileExists( SCHFileName ) then begin
        DeleteFile( SCHFilename );
      end;
      FileClose( FileCreate( Filename ));
      FileClose( FileCreate( SCHFilename ));
      FileStream := TFileStream.Create( FileName, fmOpenWrite );
      try
        if NeedToPost then Post;
        Active := false;
        Prepare;
        WriteBuf := '[' + FileStub + ']'#13#10;
        if rbFixed.Checked then begin
          WriteBuf := WriteBuf + 'Filetype=FIXED'#13#10;
        end else begin
          WriteBuf := WriteBuf + 'Filetype=VARYING'#13#10;
        end;
        WriteBuf := WriteBuf + 'Delimeter=' + edDelimeter.Text[ 1 ] + #13#10;
        WriteBuf := WriteBuf + 'Separator=' + edSeparator.Text[ 1 ] + #13#10;
        WriteBuf := WriteBuf + 'CharSet=ASCII'#13#10;
        with Fields do begin
          FldTot := 0;
          for ii := 0 to FieldCount - 1 do with Fields[ ii ] do begin
            WriteBuf := WriteBuf + 'Field' + IntToStr( ii + 1 ) + '=' +
                        FieldName + ',';
            case SQLType of
              SQL_Text,
              SQL_Text_,
              SQL_Varying,
              SQL_Varying_: begin
                FldType := 'CHAR';
                FldLen := SQLLen;
              end;
              SQL_FLOAT,
              SQL_FLOAT_,
              SQL_DOUBLE,
              SQL_DOUBLE_: begin
                FldType := 'FLOAT';
                FldLen := SQLLen;
              end;
              SQL_SHORT,
              SQL_SHORT_: begin
                FldType := 'NUMBER';
                FldLen := SQLLen;
              end;
              SQL_LONG,
              SQL_LONG_: begin
                FldType := 'LONGINT';
                FldLen := SQLLen;
              end;
              SQL_DATE,
              SQL_DATE_: begin
                FldType := 'TIMESTAMP';
                FldLen := 19;
              end;
              SQL_BLOB,
              SQL_BLOB_: begin
                FldType := 'CHAR';
                FldLen := SQLLen;
              end;
              SQL_ARRAY,
              SQL_ARRAY_: begin
                FldType := 'CHAR';
                FldLen := SQLLen;
              end;
              else begin
                FldLen := 0;
              end;
            end;
            WriteBuf := WriteBuf + FldType + ',' + IntToStr( FldLen ) + ',' +
                                                   IntToStr( -SQLScale ) + ',' +
                                                   IntToStr( FldTot ) + #13#10;
            FldTot := FldTot + FldLen;
          end;
        end;
        with TStringList.Create do try
          Text := WriteBuf;
          SaveToFile( SCHFileName );
        finally
          Free;
        end;
        try
          if rbFixed.Checked then begin
            AfterFetchRow := crExportExportFixed;
          end else if rbDelimeted.Checked then begin
            AfterFetchRow := crExportExportDelimeted;
            Delimeter := edDelimeter.Text[ 1 ];
            Separator := edSeparator.Text[ 1 ];
          end else begin
            raise Exception.Create( 'Please select an output type.' );
          end;
          stlExport.Statement := nil;
          dslExport.DataSet := nil;
          OldInc := crExport.FetchCallbackInc;
          crExport.FetchCallbackInc := 256;
          Last;
        finally
          Active := false;
          crExport.FetchCallbackInc := OldInc;
          stlExport.Statement := crExport;
          dslExport.DataSet := crExport;;
          AfterFetchRow := nil;
        end;
      finally
        FileStream.Free;
      end;
    end;
  end;
end;

procedure TfrmExport.crExportExportFixed( IB_DataSet: TIB_DataSet );
begin
// Not ready yet!
end;

procedure TfrmExport.crExportExportDelimeted( IB_DataSet: TIB_DataSet );
var
  ii: integer;
  OutBuf: string;
  ExtBuf: extended;
  jj: smallint;
begin
  with crExport, Fields, FileStream do begin
    for ii := 0 to FieldCount - 1 do with PSQLDA.SQLVAR[ ii ] do begin
      if ii > 0 then begin
        Write( Separator, 1 );
      end;
      case SQLType of
        SQL_Text,
        SQL_Text_: begin
          Write( Delimeter, 1 );
          Write( SQLData^, SQLLen );
          Write( Delimeter, 1 );
        end;
        SQL_Varying,
        SQL_Varying_: begin
          Write( Delimeter, 1 );
          Write(Pchar(ptr(longint(SQLData)+2))^,
                 byte(ptr(longint(SQLData)+1)^) * 256 +
                 byte(ptr(longint(SQLData))^));
          Write( Delimeter, 1 );
        end;
        SQL_FLOAT,
        SQL_FLOAT_: begin
          Outbuf := FloatToStr( single( SQLData^ ));
          Write(Pchar(Outbuf)^, Length( Outbuf ));
        end;
        SQL_DOUBLE,
        SQL_DOUBLE_: begin
          Outbuf := FloatToStr( double( SQLData^ ));
          Write(Pchar(Outbuf)^, Length( Outbuf ));
        end;
        SQL_SHORT,
        SQL_SHORT_: begin
          if SQLScale = 0 then begin
            Outbuf := IntToStr( smallint( SQLData^ ));
            Write(Pchar(Outbuf)^, Length( Outbuf ));
          end else begin
            Extbuf := smallint( SQLData^ );
            jj := 0;
            while jj < -SQLScale do begin
              Extbuf := Extbuf / 10;
              Inc( jj );
            end;
            Outbuf := FloatToStr( Extbuf );
            Write(Pchar(Outbuf)^, Length( Outbuf ));
          end;
        end;
        SQL_LONG,
        SQL_LONG_: begin
          if SQLScale = 0 then begin
            Outbuf := IntToStr( integer( SQLData^ ));
            Write(Pchar(Outbuf)^, Length( Outbuf ));
          end else begin
            Extbuf := integer( SQLData^ );
            jj := 0;
            while jj < -SQLScale do begin
              Extbuf := Extbuf / 10;
              Inc( jj );
            end;
            Outbuf := FloatToStr( Extbuf );
            Write(Pchar(Outbuf)^, Length( Outbuf ));
          end;
        end;
        SQL_DATE,
        SQL_DATE_: begin
          Outbuf := FormatDateTime('mm/dd/yyyy hh:nn:ss',
                                   isc_decode_TDateTime( pisc_quad( SQLData )));
          Write(Pchar(Outbuf)^, Length( Outbuf ));
        end;
        SQL_BLOB,
        SQL_BLOB_: begin
          if sqlsubtype = 1 then begin
            Outbuf := '(MEMO)';
            Write( Delimeter, 1 );
            Write(Pchar(Outbuf)^, Length( Outbuf ));
            Write( Delimeter, 1 );
          end else begin
            Outbuf := '(BLOB)';
            Write( Delimeter, 1 );
            Write(Pchar(Outbuf)^, Length( Outbuf ));
            Write( Delimeter, 1 );
          end;
        end;
        SQL_ARRAY,
        SQL_ARRAY_: begin
          Outbuf := '(ARRAY)';
          Write( Delimeter, 1 );
          Write(Pchar(Outbuf)^, Length( Outbuf ));
          Write( Delimeter, 1 );
        end;
      end;
    end;
    Write( EndOfRec, 2 );
  end;
end;

end.

