unit
  FRM_IB_Lookup;

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Grids, ComCtrls, Mask,

  FRM_IB_Base,

  IB_Components,
  IB_UpdateBar,
  IB_NavigationBar,
  IB_DataSetBar,
  IB_SearchBar,
  IB_Grid,
  IB_SearchPanel,
  IB_SearchEdit,
  IB_ConnectionSource,
  IB_TransactionSource,
  IB_ParamEdit;

type
  TfrmLookup = class(TfrmBase)
    IB_Grid: TIB_Grid;
    DataSource: TIB_DataSource;
    SpeedBar: TPanel;
    sbCharities: TIB_SearchBar;
    IB_DataSetBar1: TIB_DataSetBar;
    nbCharities: TIB_NavigationBar;
    IB_UpdateBar1: TIB_UpdateBar;
    IB_SearchPanel: TIB_SearchPanel;
    pnSearch: TPanel;
    edSearch: TIB_SearchEdit;
    procedure DataSourceStateChanged(Sender: TIB_DataSource;
      ADataSet: TIB_DataSet);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure pnSearchResize(Sender: TObject);
    procedure DataSourceDataChange(Sender: TIB_StatementLink;
      Statement: TIB_Statement; Field: TIB_Column);
    procedure IB_GridDblClick(Sender: TObject);
  private
    { Private declarations }
    FTarget: TIB_Statement;
    FSearchField: string;
    FLookupLinks: TIB_StringList;
    FPostOnClose: boolean;
    function GetLookupDataSet: TIB_DataSet;
    procedure SetLookupDataSet( AValue: TIB_DataSet );
    procedure SetLookupLinks( AValue: TIB_StringList );
    procedure UpdateLookupLinks;
  protected
    procedure DoReadSettings; override;
    procedure DoWriteSettings; override;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    { Public declarations }
    property Target: TIB_Statement read FTarget write FTarget;
    property LookupDataSet: TIB_DataSet read GetLookupDataSet
                                        write SetLookupDataSet;
    property LookupLinks: TIB_StringList read FLookupLinks
                                         write SetLookupLinks;
    property PostOnClose: boolean read FPostOnClose write FPostOnClose;
  end;

implementation

{$R *.DFM}

constructor TfrmLookup.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FLookupLinks := TIB_StringList.Create;
end;

destructor TfrmLookup.Destroy;
begin
  FLookupLinks.Free;
  inherited Destroy;
end;

procedure TfrmLookup.FormCreate(Sender: TObject);
begin
  IB_Grid.Align := alClient;
end;

procedure TfrmLookup.DoReadSettings;
begin
//  with Reg_INI do begin
// Retrieve persistent settings here.
//  end;
end;

procedure TfrmLookup.DoWriteSettings;
begin
//  with Reg_INI do begin
// Store persistent settings here.
//  end;
end;

procedure TfrmLookup.DataSourceDataChange(Sender: TIB_StatementLink;
  Statement: TIB_Statement; Field: TIB_Column);
begin
  DataSourceStateChanged( Sender as TIB_DataSource, Statement as TIB_DataSet );
end;

procedure TfrmLookup.DataSourceStateChanged(Sender: TIB_DataSource;
  ADataSet: TIB_DataSet);
begin
  if ADataSet <> nil then with ADataSet do begin
    btOK.Enabled := not EOF and not BOF and not NeedToPost;
    IB_Grid.Visible := not ( State in [ dsSearch, dsEdit, dsInsert ] );
    IB_SearchPanel.Visible := State in [ dsSearch, dsEdit, dsInsert ];
  end else begin
    btOK.Enabled := false;
    IB_SearchPanel.Visible := false;
    IB_Grid.Visible := true;
  end;
end;

procedure TfrmLookup.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if LookupDataSet <> nil then with LookupDataSet do begin
    if NeedToPost then begin
      Cancel;
    end;
    if ModalResult = mrOk then begin
      UpdateLookupLinks;
    end;
  end;
  if PostOnClose and ( Target is TIB_DataSet ) then begin
    with Target as TIB_DataSet do begin
      if NeedToPost then begin 
        if ModalResult = mrOk then begin
          Post;
        end else begin
          Cancel;
        end;
      end;
    end;
  end;
  inherited;
end;

procedure TfrmLookup.UpdateLookupLinks;
var
  ii: integer;
begin
  if ( LookupDataSet = nil ) or ( Target = nil ) then Exit;
  with LookupDataSet do begin
    for ii := 0 to SQLKeyFields.Count - 1 do begin
      if LookupLinks.Values[ SQLKeyFields[ ii ]] <> '' then begin
        Target.FieldByname( LookupLinks.Values[ SQLKeyFields[ii]] ).AsString :=
          FieldByname( SQLKeyFields[ii] ).AsString;
      end;
    end;
  end;
end;

function TfrmLookup.GetLookupDataSet: TIB_DataSet;
begin
  Result := DataSource.DataSet;
end;

procedure TfrmLookup.SetLookupDataSet(AValue: TIB_DataSet);
begin
  DataSource.DataSet := AValue;
end;

procedure TfrmLookup.SetLookupLinks(AValue: TIB_StringList);
begin
  FLookupLinks.Text := AValue.Text;
end;

procedure TfrmLookup.pnSearchResize(Sender: TObject);
begin
  if pnSearch.ClientWidth - edSearch.Left * 2 > 300 then begin
    edSearch.Width := 300;
  end else begin
    edSearch.Width := pnSearch.ClientWidth - edSearch.Left * 2;
  end;
end;

procedure TfrmLookup.IB_GridDblClick(Sender: TObject);
begin
  if DataSource.DataSet <> nil then begin
    with IB_Grid, DataSource.DataSet do begin
      if ( State = dsBrowse ) and CanEdit then Edit;
      if ( State = dsEdit ) and ( GridField <> nil ) then begin
        GridField.FocusControl;
      end;
    end;
  end;
end;

end.

