unit
  FRM_IB_Monitor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Registry, ComCtrls,

  FRM_IB_Base,

  IB_Components,
  IB_Monitor,
  IB_ConnectionSource,
  IB_TransactionSource;

type
  TfrmMonitor = class(TfrmBase)
    meMonitor: TMemo;
    Panel15: TPanel;
    btSQLMonClear: TButton;
    cbConnection: TCheckBox;
    cbTransaction: TCheckBox;
    cbStatement: TCheckBox;
    cbRow: TCheckBox;
    btSQLMonSave: TButton;
    cbSQL: TCheckBox;
    cbEnabled: TCheckBox;
    Panel23: TPanel;
    IB_Monitor: TIB_Monitor;
    sdMonitor: TSaveDialog;
    procedure IB_MonitorMonitorOutputItem(Sender: TObject;
      NewString: string);
    procedure btSQLMonClearClick(Sender: TObject);
    procedure btSQLMonSaveClick(Sender: TObject);
    procedure cbEnabledClick(Sender: TObject);
    procedure cbClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  protected
    procedure DoReadSettings; override;
    procedure DoWriteSettings; override;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

procedure TfrmMonitor.FormCreate(Sender: TObject);
begin
  inherited;
  SetDefaultBounds( 0, 0, 500, 300 );
end;

procedure TfrmMonitor.DoReadSettings;
begin
  with RegDB do begin
    cbEnabled.Checked := ReadBool( 'Settings', 'Enabled', false );
    cbConnection.Checked := ReadBool( 'Settings', 'Connection', true );
    cbTransaction.Checked := ReadBool( 'Settings', 'Transaction', true );
    cbStatement.Checked := ReadBool( 'Settings', 'Statement', false );
    cbSQL.Checked := ReadBool( 'Settings', 'SQL', true );
    cbRow.Checked := ReadBool( 'Settings', 'Row', false );
  end;
end;

procedure TfrmMonitor.DoWriteSettings;
begin
  with RegDB do begin
    WriteBool( 'Settings', 'Enabled', cbEnabled.Checked );
    WriteBool( 'Settings', 'Connection', cbConnection.Checked );
    WriteBool( 'Settings', 'Transaction', cbTransaction.Checked );
    WriteBool( 'Settings', 'Statement', cbStatement.Checked );
    WriteBool( 'Settings', 'SQL', cbSQL.Checked );
    WriteBool( 'Settings', 'Row', cbRow.Checked );
  end;
end;

{------------------------------------------------------------------------------}

procedure TfrmMonitor.IB_MonitorMonitorOutputItem(Sender: TObject;
  NewString: string);
begin
  meMonitor.Lines.Add( NewString );
end;

procedure TfrmMonitor.btSQLMonClearClick(Sender: TObject);
begin
  meMonitor.Lines.Clear;
end;

procedure TfrmMonitor.btSQLMonSaveClick(Sender: TObject);
begin
  with sdMonitor do begin
    if Execute then begin
      meMonitor.Lines.SaveToFile( FileName );
    end;
  end;
end;

procedure TfrmMonitor.cbEnabledClick(Sender: TObject);
begin
  IB_Monitor.Enabled := cbEnabled.Checked;
  cbConnection.Enabled := cbEnabled.Checked;
  cbTransaction.Enabled := cbEnabled.Checked;
  cbStatement.Enabled := cbEnabled.Checked; 
  cbSQL.Enabled := cbEnabled.Checked;
  cbRow.Enabled := cbEnabled.Checked;
  if cbEnabled.Checked then begin
    meMonitor.Color := clWindow;
    meMonitor.Lines.Add( '----------  ENABLED  ----------' );
  end else begin
    meMonitor.Lines.Add( '----------  DISABLED ----------' );
    meMonitor.Color := clSilver;
  end;
end;

procedure TfrmMonitor.cbClick(Sender: TObject);
  procedure UpdateStatementGroup;
  begin
    with IB_Monitor do begin
      if cbStatement.Checked then begin
        StatementGroups := StatementGroups + [ sgAllocate ];
        StatementGroups := StatementGroups + [ sgDescribe ];
        StatementGroups := StatementGroups + [ sgStatementInfo ];
        StatementGroups := StatementGroups + [ sgExecute ];
        StatementGroups := StatementGroups + [ sgIBCursor ];
      end else begin
        StatementGroups := StatementGroups - [ sgAllocate ];
        StatementGroups := StatementGroups - [ sgDescribe ];
        StatementGroups := StatementGroups - [ sgStatementInfo ];
        StatementGroups := StatementGroups - [ sgExecute ];
        StatementGroups := StatementGroups - [ sgIBCursor ];
      end;
      if cbStatement.Checked or cbSQL.Checked then begin
        StatementGroups := StatementGroups + [ sgPrepare ];
        StatementGroups := StatementGroups + [ sgExecuteImmediate ];
      end else begin
        StatementGroups := StatementGroups - [ sgPrepare ];
        StatementGroups := StatementGroups - [ sgExecuteImmediate ];
      end;
    end;
  end;
begin
  with Sender as TCheckBox, IB_Monitor do begin
    if Sender = cbConnection  then begin
      if Checked then begin
        MonitorGroups := MonitorGroups + [ mgConnection ];
      end else begin
        MonitorGroups := MonitorGroups - [ mgConnection ];
      end;
    end else if Sender = cbTransaction then begin
      if Checked then begin
        MonitorGroups := MonitorGroups + [ mgTransaction ];
      end else begin
        MonitorGroups := MonitorGroups - [ mgTransaction ];
      end;
    end else if Sender = cbStatement then begin
      if Checked or cbSQL.Checked then begin
        MonitorGroups := MonitorGroups + [ mgStatement ];
      end else begin
        MonitorGroups := MonitorGroups - [ mgStatement ];
      end;
      UpdateStatementGroup;
    end else if Sender = cbSQL then begin
      if Checked or cbStatement.Checked then begin
        MonitorGroups := MonitorGroups + [ mgStatement ];
      end else begin
        MonitorGroups := MonitorGroups - [ mgStatement ];
      end;
      UpdateStatementGroup;
    end else if Sender = cbRow then begin
      if Checked then begin
        MonitorGroups := MonitorGroups + [ mgRow ];
      end else begin
        MonitorGroups := MonitorGroups - [ mgRow ];
      end;
    end;
  end;
end;

procedure TfrmMonitor.FormDestroy(Sender: TObject);
begin
  try
    WriteOutSettings;
  except
    Application.HandleException( Self );
  end;
end;

end.


