unit
  FRM_IB_Query;

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Buttons,
  ExtCtrls, Grids, Spin, ComCtrls, Registry,

  FRM_IB_Base,

  IB_StatementGrid,
  IB_TransactionBar,
  IB_UpdateBar,
  IB_SearchBar,
  IB_DataSetBar,
  IB_NavigationBar,
  IB_Components,
  IB_Controls,
  IB_BDataSet,
  IB_StatementSource,
  IB_Grid,
  IB_SearchPanel,
  IB_ConnectionSource,
  IB_TransactionSource, Mask, IB_ParamEdit, IB_SearchEdit;

type
  TfrmQuery = class(TfrmBase)
    tsDataSet: TPanel;
    Panel26: TPanel;
    btInvalidateRows: TSpeedButton;
    IB_NavigationBar1: TIB_NavigationBar;
    IB_DataSetBar1: TIB_DataSetBar;
    IB_SearchBar2: TIB_SearchBar;
    IB_UpdateBar1: TIB_UpdateBar;
    IB_DataSetBar2: TIB_DataSetBar;
    IB_TransactionBar: TIB_TransactionBar;
    btNew: TSpeedButton;
    Panel39: TPanel;
    dsQuery: TIB_DataSource;
    qrQuery: TIB_Query;
    slQuery: TIB_StatementSource;
    dsMasterSource: TIB_DataSource;
    pcQuery: TPageControl;
    tsQuerySQLProperties: TTabSheet;
    pcQuerySQL: TPageControl;
    tsQuerySQL: TTabSheet;
    meSQL: TMemo;
    tsQueryLinks: TTabSheet;
    tsQuerySQLKeyFields: TTabSheet;
    meSQLKeyFields: TMemo;
    tsQueryOrderingItems: TTabSheet;
    meOrderingItems: TMemo;
    Panel40: TPanel;
    Label14: TLabel;
    seOrderingItemNo: TSpinEdit;
    tsQueryOrderingLinks: TTabSheet;
    meOrderingLinks: TMemo;
    tsQueryFieldProperties: TTabSheet;
    pcFieldProperties: TPageControl;
    tsQueryAlignment: TTabSheet;
    meFieldsAlignment: TMemo;
    tsQueryDisplayFormat: TTabSheet;
    meFieldsDisplayFormat: TMemo;
    tsQueryDisplayLabel: TTabSheet;
    meFieldsDisplayLabel: TMemo;
    tsQueryDisplayWidth: TTabSheet;
    meFieldsDisplayWidth: TMemo;
    tsQueryEditmask: TTabSheet;
    meFieldsEditMask: TMemo;
    tsQueryIndex: TTabSheet;
    meFieldsIndex: TMemo;
    tsQueryReadOnly: TTabSheet;
    meFieldsReadOnly: TMemo;
    tsQueryVisible: TTabSheet;
    meFieldsVisible: TMemo;
    tsQueryInput: TTabSheet;
    IB_StatementGrid3: TIB_StatementGrid;
    tsQueryFields: TTabSheet;
    IB_StatementGrid4: TIB_StatementGrid;
    tsQueryOutput: TTabSheet;
    grQuery: TIB_Grid;
    dsQuerySource: TIB_DataSource;
    tsQueryHints: TTabSheet;
    meHints: TMemo;
    spQuery: TIB_SearchPanel;
    Panel1: TPanel;
    Label1: TLabel;
    edSQLKeyRelation: TEdit;
    tsColumnAttributes: TTabSheet;
    meColumnAttributes: TMemo;
    Panel2: TPanel;
    mePlan: TMemo;
    Panel3: TPanel;
    meSQLJoinLinks: TMemo;
    Panel4: TPanel;
    meMasterLinks: TMemo;
    Panel5: TPanel;
    meGeneratorLinks: TMemo;
    tsSearchingLinks: TTabSheet;
    meSearchingLinks: TMemo;
    pnSearch: TPanel;
    Label3: TLabel;
    IB_SearchEdit1: TIB_SearchEdit;
    tsFieldList: TTabSheet;
    meFieldList: TMemo;
    procedure meSQLChange(Sender: TObject);
    procedure pcQueryChange(Sender: TObject);
    procedure meQueryAttributesChange(Sender: TObject);
    procedure pcFieldPropertiesChange(Sender: TObject);
    procedure meOrderingLinksExit(Sender: TObject);
    procedure seOrderingItemNoChange(Sender: TObject);
    procedure pcQuerySQLChange(Sender: TObject);
    procedure btInvalidateRowsClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btNewClick(Sender: TObject);
    procedure slQueryBeforePrepare(Sender: TIB_StatementLink;
      Statement: TIB_Statement);
    procedure meQueryMemoEnter(Sender: TObject);
    procedure meQueryMemoExit(Sender: TObject);
    procedure meQueryMemoChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure dsQueryStateChanged(Sender: TIB_DataSource;
      ADataSet: TIB_DataSet);
    procedure IB_DataSetBar1AfterAction(Sender: TObject);
    procedure meHintsEnter(Sender: TObject);
    procedure meHintsExit(Sender: TObject);
    procedure edSQLKeyRelationChange(Sender: TObject);
    procedure slQueryPreparedChanged(Sender: TIB_StatementLink;
      Statement: TIB_Statement);
    procedure IB_ConnectionSourceAfterAssignment(
      Sender: TIB_ConnectionLink; AConnection: TIB_Connection);
    procedure IB_TransactionSourceAfterAssignment(
      Sender: TIB_TransactionLink; ATransaction: TIB_Transaction);
    procedure meSearchingLinksExit(Sender: TObject);
  private
    { Private declarations }
    MemoWasChanged: boolean;
  protected
    procedure DoReadSettings; override;
    procedure DoWriteSettings; override;
  public
    { Public declarations }
    QueryForm: string;
  end;

implementation

{$R *.DFM}

procedure TfrmQuery.FormCreate(Sender: TObject);
begin
  SetDefaultBounds( 0, 0, 600, 400 );
  grQuery.Align := alClient;
end;

procedure TfrmQuery.IB_ConnectionSourceAfterAssignment(
  Sender: TIB_ConnectionLink; AConnection: TIB_Connection);
begin
  qrQuery.IB_Connection := AConnection;
end;

procedure TfrmQuery.IB_TransactionSourceAfterAssignment(
  Sender: TIB_TransactionLink; ATransaction: TIB_Transaction);
begin
  qrQuery.IB_Transaction := ATransaction;
  IB_TransactionBar.IB_Transaction := ATransaction;
end;

procedure TfrmQuery.DoReadSettings;
begin
  with RegDB do begin
    with pcQuery do begin
      ActivePage := Pages[ ReadInteger( 'Settings', 'Query', 0 ) ];
    end;
    with pcQuerySQL do begin
      ActivePage := Pages[ ReadInteger( 'Settings', 'SQL', 0 ) ];
    end;
    with pcFieldProperties do begin
      ActivePage := Pages[ ReadInteger( 'Settings', 'FieldProperties', 0 ) ];
    end;
  end;
  if dsQuerySource.DataSet <> nil then begin
    with dsQuerySource.DataSet do begin
      qrQuery.IB_Connection := IB_Connection;
      qrQuery.IB_Transaction := IB_Transaction;
      qrQuery.SQL.Assign( SQL );
      meGeneratorLinks.Lines.Text := GeneratorLinks.Text;
      meHints.Lines.Text := Hints.Text;
      meMasterLinks.Lines.Text := MasterLinks.Text;
      meSQLJoinLinks.Lines.Text := SQLJoinLinks.Text;
      meSQL.Lines.Text := SQL.Text;
      meSQLKeyFields.Lines.Text := SQLKeyFields.Text;
      edSQLKeyRelation.Text := SQLKeyRelation;
      meOrderingItems.Lines.Text := OrderingItems.Text;
      meOrderingLinks.Lines.Text := OrderingLinks.Text;
      meSearchingLinks.Lines.Text := SearchingLinks.Text;
      seOrderingItemNo.Value := OrderingItemNo;
      edSQLKeyRelation.Text := SQLKeyRelation;
      meColumnAttributes.Lines.Text := ColumnAttributes.Text;
    end;
  end else if RegDB <> nil then begin
    ReadStrings( 'SQL',              meSQL.Lines );
    ReadStrings( 'SQLKeyFields',     meSQLKeyFields.Lines );
    ReadStrings( 'SQLJoinLinks',     meSQLJoinLinks.Lines );
    ReadStrings( 'MasterLinks',      meMasterLinks.Lines );
    ReadStrings( 'GeneratorLinks',   meGeneratorLinks.Lines );
    ReadStrings( 'OrderingItems',    meOrderingItems.Lines );
    ReadStrings( 'OrderingLinks',    meOrderingLinks.Lines );
    ReadStrings( 'SearchingLinks',   meSearchingLinks.Lines );
    ReadStrings( 'ColumnAttributes', meColumnAttributes.Lines );
    ReadStrings( 'Hints',            meHints.Lines );
    with qrQuery do begin
      BeginLayout;
      try
        ReadStrings( 'FieldsAlignment',     FieldsAlignment );
        ReadStrings( 'FieldsDisplayFormat', FieldsDisplayFormat );
        ReadStrings( 'FieldsDisplayLabel',  FieldsDisplayLabel );
        ReadStrings( 'FieldsDisplayWidth',  FieldsDisplayWidth );
        ReadStrings( 'FieldsEditMask',      FieldsEditMask );
        ReadStrings( 'FieldsIndex',         FieldsIndex );
        ReadStrings( 'FieldsReadOnly',      FieldsReadOnly );
        ReadStrings( 'FieldsVisible',       FieldsVisible );
      finally
        EndLayout;
      end;
    end;
    with RegDB do begin
      edSQLKeyRelation.Text := ReadString( 'Settings', 'SQLKeyRelation', '' );
      seOrderingItemNo.Value:= ReadInteger( 'Settings','OrderingItemNo', -1 );
    end;
  end;
end;

procedure TfrmQuery.DoWriteSettings;
begin
  with RegDB do begin
    with pcQuery do begin
      WriteInteger('Settings', 'Query', ActivePage.PageIndex );
    end;
    with pcQuerySQL do begin
      WriteInteger('Settings', 'SQL', ActivePage.PageIndex );
    end;
    with pcFieldProperties do begin
      WriteInteger('Settings', 'FieldProperties', ActivePage.PageIndex );
    end;
  end;
  if dsQuerySource.DataSet <> nil then begin
    with dsQuerySource.DataSet do begin
      SQL.Assign( qrQuery.SQL );
      SQL.Text := meSQL.Lines.Text;
      SQLKeyFields.Text := meSQLKeyFields.Lines.Text;
      SQLKeyRelation := edSQLKeyRelation.Text;
      SQLJoinLinks.Text := meSQLJoinLinks.Lines.Text;
      MasterLinks.Text := meMasterLinks.Lines.Text;
      GeneratorLinks.Text := meGeneratorLinks.Lines.Text;
      OrderingItems.Text := meOrderingItems.Lines.Text;
      OrderingLinks.Text := meOrderingLinks.Lines.Text;
      SearchingLinks.Text := meSearchingLinks.Lines.Text;
      ColumnAttributes.Text := meColumnAttributes.Lines.Text;
      Hints.Text := meHints.Lines.Text;
    end;
  end else if RegDB <> nil then begin
    WriteStrings( 'SQL',              meSQL.Lines );
    WriteStrings( 'SQLKeyFields',     meSQLKeyFields.Lines );
    WriteStrings( 'SQLJoinLinks',     meSQLJoinLinks.Lines );
    WriteStrings( 'MasterLinks',      meMasterLinks.Lines );
    WriteStrings( 'GeneratorLinks',   meGeneratorLinks.Lines );
    WriteStrings( 'OrderingItems',    meOrderingItems.Lines );
    WriteStrings( 'OrderingLinks',    meOrderingLinks.Lines);
    WriteStrings( 'SearchingLinks',   meSearchingLinks.Lines);
    WriteStrings( 'ColumnAttributes', meColumnAttributes.Lines );
    WriteStrings( 'Hints',            meHints.Lines );
    with qrQuery do begin
      WriteStrings( 'FieldsAlignment',     FieldsAlignment );
      WriteStrings( 'FieldsDisplayFormat', FieldsDisplayFormat );
      WriteStrings( 'FieldsDisplayLabel',  FieldsDisplayLabel );
      WriteStrings( 'FieldsDisplayWidth',  FieldsDisplayWidth );
      WriteStrings( 'FieldsEditMask',      FieldsEditMask );
      WriteStrings( 'FieldsIndex',         FieldsIndex );
      WriteStrings( 'FieldsReadOnly',      FieldsReadOnly );
      WriteStrings( 'FieldsVisible',       FieldsVisible );
    end;
    with RegDB do begin
      WriteString( 'Settings', 'SQLKeyRelation', edSQLKeyRelation.Text );
      ReadInteger( 'Settings', 'OrderingItemNo', seOrderingItemNo.Value );
    end;
  end;
end;

procedure TfrmQuery.btInvalidateRowsClick(Sender: TObject);
begin
  qrQuery.InvalidateRows;
end;

procedure TfrmQuery.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with qrQuery do if NeedToPost and ( State <> dsSearch ) then begin
    MessageBeep( 1 );
    Action := caNone;
  end else begin
    inherited;
  end;
end;

procedure TfrmQuery.pcQuerySQLChange(Sender: TObject);
begin
  seOrderingItemNo.Value := qrQuery.OrderingItemNo;
end;

procedure TfrmQuery.pcQueryChange(Sender: TObject);
begin
  if pcQuery.ActivePage = tsQueryFieldProperties then begin
    pcFieldPropertiesChange(Sender);
  end;
end;

procedure TfrmQuery.pcFieldPropertiesChange(Sender: TObject);
begin
  with qrQuery, pcFieldProperties do begin
    if ActivePage = tsQueryAlignment then begin
      meFieldsAlignment.Lines.Text := FieldsAlignment.Text;
    end else if ActivePage = tsQueryDisplayFormat then begin
      meFieldsDisplayFormat.Lines.Text := FieldsDisplayFormat.Text;
    end else if ActivePage = tsQueryDisplayLabel then begin
      meFieldsDisplayLabel.Lines.Text := FieldsDisplayLabel.Text;
    end else if ActivePage = tsQueryDisplayWidth then begin
      meFieldsDisplayWidth.Lines.Text := FieldsDisplayWidth.Text;
    end else if ActivePage = tsQueryEditMask then begin
      meFieldsEditMask.Lines.Text := FieldsEditMask.Text;
    end else if ActivePage = tsQueryIndex then begin
      meFieldsIndex.Lines.Text := FieldsIndex.Text;
    end else if ActivePage = tsQueryReadOnly then begin
      meFieldsReadOnly.Lines.Text := FieldsReadOnly.Text;
    end else if ActivePage = tsQueryVisible then begin
      meFieldsVisible.Lines.Text := FieldsVisible.Text;
    end;
  end;
end;

procedure TfrmQuery.meSQLChange(Sender: TObject);
begin
  with qrQuery do begin
    Prepared := false;
    SQL.Clear;
  end;
  if Sender = meOrderingItems then begin
    seOrderingItemNo.MaxValue :=   meOrderingItems.Lines.Count;
    seOrderingItemNo.MinValue := - meOrderingItems.Lines.Count;
  end;
end;

procedure TfrmQuery.meQueryAttributesChange(Sender: TObject);
begin
  qrQuery.Prepared := false;
end;

procedure TfrmQuery.meOrderingLinksExit(Sender: TObject);
begin
  with qrQuery do begin
    OrderingLinks.Text := meOrderingLinks.Text;
  end;
end;

procedure TfrmQuery.meSearchingLinksExit(Sender: TObject);
begin
  with qrQuery do begin
    SearchingLinks.Text := meSearchingLinks.Text;
  end;
end;

procedure TfrmQuery.seOrderingItemNoChange(Sender: TObject);
begin
  qrQuery.OrderingItemNo := seOrderingItemNo.Value;
end;

procedure TfrmQuery.btNewClick(Sender: TObject);
begin
  qrQuery.Prepared := false;
  meMasterLinks.Lines.Clear;
  meSQL.Lines.Clear;
  meSQLKeyFields.Lines.Clear;
  meOrderingItems.Lines.Clear;
  meOrderingLinks.Lines.Clear;
  meSearchingLinks.Lines.Clear;
  meColumnAttributes.Lines.Clear;
  meGeneratorLinks.Lines.Clear;
  meHints.Lines.Clear;
end;

procedure TfrmQuery.slQueryBeforePrepare(Sender: TIB_StatementLink;
  Statement: TIB_Statement);
begin
  with qrQuery do begin
    MasterLinks.Text := Trim( meMasterLinks.Lines.Text );
    if MasterLinks.Count > 0 then begin
      if MasterSource = nil then begin
        MasterSource := dsMasterSource;
      end;
    end else begin
      MasterSource := nil;
    end;
    SQL.Text := meSQL.Lines.Text;
    SQLKeyFields.Text := Trim( meSQLKeyFields.Lines.Text );
    GeneratorLinks.Text := Trim( meGeneratorLinks.Lines.Text );
    Hints.Text := Trim( meHints.Lines.Text );
    OrderingItems.Text := Trim( meOrderingItems.Lines.Text );
    OrderingLinks.Text := Trim( meOrderingLinks.Lines.Text );
    SearchingLinks.Text := Trim( meSearchingLinks.Lines.Text );
    OrderingItemNo := seOrderingItemNo.Value;
    ColumnAttributes.Text := meColumnAttributes.Lines.Text;
  end;
end;

procedure TfrmQuery.meQueryMemoEnter(Sender: TObject);
begin
  MemoWasChanged := false;
end;

procedure TfrmQuery.meQueryMemoExit(Sender: TObject);
begin
  if MemoWasChanged then begin
    with qrQuery do begin
      if Sender = meFieldsAlignment then begin
        FieldsAlignment := meFieldsAlignment.Lines;
      end else if Sender = meFieldsDisplayFormat then begin
        FieldsDisplayFormat := meFieldsDisplayFormat.Lines;
      end else if Sender = meFieldsDisplaylabel then begin
        FieldsDisplayLabel := meFieldsDisplayLabel.Lines;
      end else if Sender = meFieldsDisplayWidth then begin
        FieldsDisplayWidth := meFieldsDisplayWidth.Lines;
      end else if Sender = meFieldsEditmask then begin
        FieldsEditmask := meFieldsEditmask.Lines;
      end else if Sender = meFieldsIndex then begin
        FieldsIndex := meFieldsIndex.Lines;
      end else if Sender = meFieldsReadOnly then begin
        FieldsReadOnly := meFieldsReadOnly.Lines;
      end else if Sender = meFieldsVisible then begin
        FieldsVisible := meFieldsVisible.Lines;
      end;
    end;
  end;
end;

procedure TfrmQuery.meQueryMemoChange(Sender: TObject);
begin
  MemoWasChanged := true;
end;

procedure TfrmQuery.dsQueryStateChanged(Sender: TIB_DataSource;
  ADataSet: TIB_DataSet);
begin
  if ADataSet <> nil then with ADataSet do begin
    grQuery.Visible := not ( State in [ dsInsert, dsEdit, dsSearch ] );
    spQuery.Visible := State in [ dsInsert, dsEdit, dsSearch ];
    if NeedToPost then begin
      pcQuery.ActivePage := tsQueryOutput;
    end;
  end else begin
    spQuery.Visible := false;
    grQuery.Visible := true;
  end;
  pnSearch.Visible := grQuery.Visible;
  if qrQuery.Prepared then mePlan.Text := qrQuery.StatementPlan
                      else mePlan.Text := '';
end;

procedure TfrmQuery.IB_DataSetBar1AfterAction(Sender: TObject);
begin
  with IB_DataSetbar1 do begin
    if FocusedButton = dbOpen then pcQuery.Activepage := tsQueryOutput;
  end;
end;

procedure TfrmQuery.meHintsEnter(Sender: TObject);
begin
  meHints.Lines.Text := qrQuery.Hints.Text;
end;

procedure TfrmQuery.meHintsExit(Sender: TObject);
begin
  qrQuery.Hints.Text := meHints.Lines.Text;
end;

procedure TfrmQuery.edSQLKeyRelationChange(Sender: TObject);
begin
  qrQuery.SQLKeyRelation := edSQLKeyRelation.Text;
end;

procedure TfrmQuery.slQueryPreparedChanged(Sender: TIB_StatementLink;
  Statement: TIB_Statement);
begin
  with qrQuery do begin
    if Prepared then mePlan.Text := StatementPlan
                else mePlan.Text := '';
    GetFieldList( meFieldList.Lines );
  end;
end;

end.

