unit
  FRM_IB_Script;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls,

  FRM_IB_Base,

  IB_Components,
  IB_ConnectionSource,
  IB_TransactionSource,
  IB_Script;

type
  TfrmScript = class(TfrmBase)
    Panel16: TPanel;
    btClear: TButton;
    btLoad: TButton;
    btSave: TButton;
    btScriptExecute: TButton;
    cbStore: TCheckBox;
    meSQL: TMemo;
    odScript: TOpenDialog;
    sdScript: TSaveDialog;
    IB_Script: TIB_Script;
    procedure btScriptExecuteClick(Sender: TObject);
    procedure btClearClick(Sender: TObject);
    procedure btLoadClick(Sender: TObject);
    procedure btSaveClick(Sender: TObject);
    procedure meSQLChange(Sender: TObject);
    procedure IB_ScriptBeforeExecute(Sender: TIB_Script);
    procedure IB_ConnectionSourceAfterAssignment(
      Sender: TIB_ConnectionLink; AConnection: TIB_Connection);
    procedure IB_TransactionSourceAfterAssignment(
      Sender: TIB_TransactionLink; ATransaction: TIB_Transaction);
    procedure FormCreate(Sender: TObject);
    procedure IB_ScriptAfterExecute(Sender: TIB_Script);
  private
    { Private declarations }
  protected
    procedure DoReadSettings; override;
    procedure DoWriteSettings; override;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

procedure TfrmScript.FormCreate(Sender: TObject);
begin
  inherited;
  SetDefaultBounds( 0, 0, 600, 400 );
end;

procedure TfrmScript.DoReadSettings;
begin
  with RegDB do begin
    cbStore.Checked := ReadBool( 'Settings', 'Store', false );
    sdScript.Filename := ReadString( 'Settings', 'Filename', '*.TXT' );
    ReadStrings( 'SQL', meSQL.Lines );
  end;
end;

procedure TfrmScript.DoWriteSettings;
begin
  with RegDB do begin
    WriteString( 'Settings', 'Filename', sdScript.Filename );
    WriteBool( 'Settings', 'Store', cbStore.Checked );
    if cbStore.Checked then begin
      WriteStrings( 'SQL', meSQL.Lines );
    end else begin
      WriteStrings( 'SQL', nil );
    end;
  end;
end;

procedure TfrmScript.IB_ConnectionSourceAfterAssignment(
  Sender: TIB_ConnectionLink; AConnection: TIB_Connection);
begin
  IB_Script.IB_Connection := AConnection;
end;

procedure TfrmScript.IB_TransactionSourceAfterAssignment(
  Sender: TIB_TransactionLink; ATransaction: TIB_Transaction);
begin
  IB_Script.IB_Transaction := ATransaction;
end;

{------------------------------------------------------------------------------}

procedure TfrmScript.btScriptExecuteClick(Sender: TObject);
begin
  IB_Script.Execute;
end;

procedure TfrmScript.IB_ScriptBeforeExecute(Sender: TIB_Script);
begin
  with IB_Script do begin
    if SQL.Count = 0 then begin
      SQL.Text := meSQL.Lines.Text;
    end;
  end;
end;

procedure TfrmScript.IB_ScriptAfterExecute(Sender: TIB_Script);
begin
  StatusBar.SimpleText := 'Script completed without exceptions';
end;

procedure TfrmScript.btClearClick(Sender: TObject);
begin
  meSQL.Lines.Clear;
end;

procedure TfrmScript.btLoadClick(Sender: TObject);
begin
  if odScript.Execute then begin
    meSQL.Lines.LoadFromFile( odScript.FileName );
  end;
end;

procedure TfrmScript.btSaveClick(Sender: TObject);
begin
  if sdScript.Execute then begin
    meSQL.Lines.SaveToFile( sdScript.FileName );
  end;
end;

procedure TfrmScript.meSQLChange(Sender: TObject);
begin
  if IB_Script.SQL.Count <> 0 then begin
    IB_Script.SQL.Clear;
  end;
end;

end.

