{~~ This unit contains the component IB_DataSetBar. }
unit IB_DataSetBar;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, StdCtrls,
  Forms, Graphics, ExtCtrls, Menus, Buttons, Dialogs,

  IB_Components;


type
{~~ These are the different button types available to this button bar.

dbPrepare prepares the current statement in the SQL property.
dbUnprepare unprepares the current statement in the SQL property.
dbOpen Opens a cursor on the server to process the fetches.
dbClose Closes the cursor opened on the server.
}
TIB_DataSetBar_ButtonType = ( dbPrepare,
                              dbUnprepare,
                              dbOpen,
                              dbClose );

{~~$<!>}
TIB_DataSetBar_ButtonSet = set of TIB_DataSetBar_ButtonType;
{~~$<!>}
TIB_DataSetBar_Button = class;

{~~ Button bar that gives access and control of statements and cursors.}
TIB_DataSetBar = class (TCustomPanel)
  private
    FIB_DataLink: TIB_DataLink;
    ButtonWidth: Integer;
    FVisibleButtons: TIB_DataSetBar_ButtonSet;
    MinBtnSize: TPoint;
    FFocusedButton: TIB_DataSetBar_ButtonType;
    FBeforeAction: TNotifyEvent;
    FAfterAction: TNotifyEvent;
    function GetOnReceiveFocus: TIB_DataLinkEvent;
    procedure SetOnReceiveFocus( AValue: TIB_DataLinkEvent );
    function GeTIB_DataSource: TIB_DataSource;
    procedure SeTIB_DataSource(AValue: TIB_DataSource);
    function GetReceiveFocus: boolean;
    procedure SetReceiveFocus(AValue: boolean);
    procedure InitButtons;
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetVisible(Value: TIB_DataSetBar_ButtonSet);
    procedure AdjustSize (var W: Integer; var H: Integer);
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure IB_DataSetStateChanged( Sender: TIB_DataLink;
                                      DataSource: TIB_DataSource );
    procedure BarClick(Sender: TObject); virtual;
    procedure ActiveChanged;
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
{~~ Storage array for the different buttons on this button bar. }
    Buttons: array[TIB_DataSetBar_ButtonType] of TIB_DataSetBar_Button;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
{~~$<!>}
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
{~~ Method used to pass the click of a single button to the owning parent.}
    procedure BtnClick(Index: TIB_DataSetBar_ButtonType);
{~~$<*>...
The currently selected button of the button set.

Use this property in the xxxxAction events to know which button was clicked.}
    property FocusedButton: TIB_DataSetBar_ButtonType read FFocusedButton;
  published
{~~$<*>...
Reference to the dataset link to perform actions upon.}
    property DataSource: TIB_DataSource
        read GeTIB_DataSource
       write SeTIB_DataSource;
{~~$<*>...
This property is used in conjunction with the AnnounceFocus property of...
the DataSource component. It is a global focusing system that allows...
the SearchBar to link itself to whereever the user's focus goes.

For example, in an MDI application the user could go from one child window...
to another. Each child window would have its own DataSource context....
Thus, if each child window's DataSource component's AnnounceFocus...
property is set to true the SearchBar will receive the focus and align with it.}
    property ReceiveFocus: boolean read GetReceiveFocus write SetReceiveFocus; 
{~~$<*>...
Set which button are visible to the user.}
    property VisibleButtons: TIB_DataSetBar_ButtonSet
        read FVisibleButtons
       write SetVisible
     default [ dbOpen, dbClose ];
{~~$<*>...
Define custom behavior before the action of the button click is executed.}
    property BeforeAction: TNotifyEvent
        read FBeforeAction
       write FBeforeAction;
{~~$<*>...
Define custom behavior after the action of the button click is executed.

This event will not be triggered if an exception was raised.}
    property AfterAction:  TNotifyEvent
        read FAfterAction
       write FAfterAction;
    property OnReceiveFocus: TIB_DataLinkEvent read GetOnReceiveFocus
                                               write SetOnReceiveFocus;
{~~$<!>} property Align;
{~~$<!>} property DragCursor;
{~~$<!>} property DragMode;
{~~$<!>} property Enabled;
{~~$<!>} property Ctl3D;
{~~$<!>} property ParentCtl3D;
{~~$<!>} property ParentShowHint;
{~~$<!>} property PopupMenu;
{~~$<!>} property ShowHint;
{~~$<!>} property TabOrder;
{~~$<!>} property TabStop;
{~~$<!>} property Visible;
{~~$<!>} property OnClick;
{~~$<!>} property OnDblClick;
{~~$<!>} property OnDragDrop;
{~~$<!>} property OnDragOver;
{~~$<!>} property OnEndDrag;
{~~$<!>} property OnEnter;
{~~$<!>} property OnExit;
{~~$<!>} property OnResize;
  end;

  TIB_DataSetBar_Button = class(TSpeedButton)
  private
    FIndex: TIB_DataSetBar_ButtonType;
  protected
    procedure Paint; override;
  public
    property Index: TIB_DataSetBar_ButtonType read FIndex write FIndex;
  end;

implementation

{$IFDEF WIN32}
{$R IB_DataSetBar.RES}
{$ENDIF}

{------------------------------------------------------------------------------}

constructor TIB_DataSetBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] + [csOpaque];
  if not NewStyleControls then ControlStyle := ControlStyle + [csFramed];
  FIB_DataLink := TIB_DataLink.Create( Self );
  with FIB_DataLink do begin
    AfterAssignment := IB_DataSetStateChanged;
    OnStateChanged := IB_DataSetStateChanged;
  end;
  FVisibleButtons := [dbOpen, dbClose];
  InitButtons;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Ctl3D := false;
  ParentCtl3D := false;
  Width := 121;
  Height := 25;
  ButtonWidth := 0;
  FFocusedButton := dbOpen;
end;

destructor TIB_DataSetBar.Destroy;
begin
  with FIB_DataLink do begin
    AfterAssignment := nil;
    OnStateChanged := nil;
  end;
  inherited Destroy;
end;

procedure TIB_DataSetBar.IB_DataSetStateChanged( Sender: TIB_DataLink;
                                       DataSource: TIB_DataSource );
begin
  ActiveChanged;
end;

function TIB_DataSetBar.GeTIB_DataSource: TIB_DataSource;
begin
  if FIB_DataLink <> nil then begin
    Result := FIB_DataLink.DataSource;
  end else begin
    Result := nil;
  end;
end;

procedure TIB_DataSetBar.SeTIB_DataSource(AValue: TIB_DataSource);
begin
  if FIB_DataLink <> nil then begin
    if FIB_DataLink.DataSource <> AValue then begin
      FIB_DataLink.DataSource := AValue;
      ActiveChanged;
    end;
  end;
end;

function TIB_DataSetBar.GetReceiveFocus: boolean;
begin
  if FIB_DataLink <> nil then begin
    Result := FIB_DataLink.ReceiveFocus;
  end else begin
    Result := false;
  end;
end;

procedure TIB_DataSetBar.SetReceiveFocus(AValue: boolean);
begin
  if FIB_DataLink <> nil then begin
    FIB_DataLink.ReceiveFocus := AValue;
  end;
end;

procedure TIB_DataSetBar.InitButtons;
var
  I: TIB_DataSetBar_ButtonType;
  Btn: TIB_DataSetBar_Button;
  X: Integer;
begin
  MinBtnSize := Point(20, 18);
  X := 0;
  for I := Low(Buttons) to High(Buttons) do
  begin
    Btn := TIB_DataSetBar_Button.Create (Self);
    Btn.Index := I;
    Btn.Visible := I in FVisibleButtons;
    Btn.Enabled := True;
    Btn.SetBounds (X, 0, MinBtnSize.X, MinBtnSize.Y);
    case I of
      dbPrepare: begin
        Btn.Hint := 'Prepare DataSet';
        Btn.Glyph.Handle := LoadBitmap(HInstance, 'DSET_BAR_PREPARE'  );
        Btn.NumGlyphs := 2;
      end;
      dbUnprepare: begin
        Btn.Hint := 'Unprepare DataSet';
        Btn.Glyph.Handle := LoadBitmap(HInstance, 'DSET_BAR_UNPREPARE'  );
        Btn.NumGlyphs := 2;
      end;
      dbOpen: begin
        Btn.Hint := 'Open DataSet';
        Btn.Glyph.Handle := LoadBitmap(HInstance, 'DSET_BAR_OPEN'  );
        Btn.NumGlyphs := 2;
      end;
      dbClose: begin
        Btn.Hint := 'Close DataSet';
        Btn.Glyph.Handle := LoadBitmap(HInstance, 'DSET_BAR_CLOSE'  );
        Btn.NumGlyphs := 2;
      end;
    end;
    Btn.Enabled := False;
    Btn.Enabled := True;
    Btn.OnClick := BarClick;
    Btn.OnMouseDown := BtnMouseDown;
    Btn.Parent := Self;
    Buttons[I] := Btn;
    X := X + MinBtnSize.X;
  end;
end;

procedure TIB_DataSetBar.BtnMouseDown (Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  OldFocus: TIB_DataSetBar_ButtonType;
begin
  OldFocus := FocusedButton;
  FFocusedButton := TIB_DataSetBar_Button(Sender).Index;
  if TabStop and (GetFocus <> Handle) and CanFocus then
  begin
    SetFocus;
    if (GetFocus <> Handle) then
      Exit;
  end
  else if TabStop and (GetFocus = Handle) and (OldFocus <> FocusedButton) then
  begin
    Buttons[OldFocus].Invalidate;
    Buttons[FocusedButton].Invalidate;
  end;
end;

procedure TIB_DataSetBar.SetVisible(Value: TIB_DataSetBar_ButtonSet);
var
  I: TIB_DataSetBar_ButtonType;
  W, H: Integer;
begin
  W := Width;
  H := Height;
  FVisibleButtons := Value;
  for I := Low(Buttons) to High(Buttons) do
    Buttons[I].Visible := I in FVisibleButtons;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
  Invalidate;
end;

procedure TIB_DataSetBar.AdjustSize (var W: Integer; var H: Integer);
var
  Count: Integer;
  MinW: Integer;
  I: TIB_DataSetBar_ButtonType;
  Space, Temp, Remain: Integer;
  X: Integer;
begin
  if (csLoading in ComponentState) then Exit;
  if Buttons[dbOpen] = nil then Exit;

  Count := 0;
  for I := Low(Buttons) to High(Buttons) do
  begin
    if Buttons[I].Visible then
    begin
      Inc(Count);
    end;
  end;
  if Count = 0 then Inc(Count);

  MinW := Count * (MinBtnSize.X - 1) + 1;
  if W < MinW then
    W := MinW;
  if H < MinBtnSize.Y then
    H := MinBtnSize.Y;

  ButtonWidth := ((W - 1) div Count) + 1;
  Temp := Count * (ButtonWidth - 1) + 1;
  if Align = alNone then
    W := Temp;

  X := 0;
  Remain := W - Temp;
  Temp := Count div 2;
  for I := Low(Buttons) to High(Buttons) do
  begin
    if Buttons[I].Visible then
    begin
      Space := 0;
      if Remain <> 0 then
      begin
        Dec (Temp, Remain);
        if Temp < 0 then
        begin
          Inc (Temp, Count);
          Space := 1;
        end;
      end;
      Buttons[I].SetBounds (X, 0, ButtonWidth + Space, Height);
      Inc (X, ButtonWidth - 1 + Space);
    end
    else
      Buttons[I].SetBounds (Width + 1, 0, ButtonWidth, Height);
  end;
end;

procedure TIB_DataSetBar.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;

  { check for minimum size }
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure TIB_DataSetBar.WMSetFocus(var Message: TWMSetFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure TIB_DataSetBar.WMKillFocus(var Message: TWMKillFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure TIB_DataSetBar.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TIB_DataSetBar.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not ( csLoading in ComponentState ) then ActiveChanged;
end;

procedure TIB_DataSetBar.ActiveChanged;
begin
  if ( DataSource <> nil ) and
     ( DataSource.DataSet <> nil ) and
     ( Enabled ) then with DataSource.DataSet do begin
    Buttons[ dbPrepare   ].Enabled := not Prepared;
    Buttons[ dbUnprepare ].Enabled :=     Prepared;
    Buttons[ dbOpen      ].Enabled := not Active;
    Buttons[ dbClose     ].Enabled :=     Active;
    if not Prepared or
       ( StatementType = stSelect ) or
       ( StatementType = stSelectForUpdate ) then begin
      Buttons[ dbOpen ].Glyph.Handle := LoadBitmap(HInstance, 'DSET_BAR_OPEN' );
    end else begin
      Buttons[ dbOpen ].Glyph.Handle :=LoadBitmap(HInstance,'DSET_BAR_EXECUTE');
      Buttons[ dbClose ].Enabled := false;
    end;
  end else begin
    Buttons[ dbPrepare   ].Enabled := false;
    Buttons[ dbUnprepare ].Enabled := false;
    Buttons[ dbOpen      ].Enabled := false;
    Buttons[ dbClose     ].Enabled := false;
    Buttons[ dbOpen   ].Glyph.Handle := LoadBitmap(HInstance, 'DSET_BAR_OPEN' );
  end;
end;

procedure TIB_DataSetBar.Loaded;
var
  W, H: Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
  ActiveChanged;
end;

procedure TIB_DataSetBar.KeyDown(var Key: Word; Shift: TShiftState);
var
  NewFocus: TIB_DataSetBar_ButtonType;
  OldFocus: TIB_DataSetBar_ButtonType;
begin
  OldFocus := FocusedButton;
  case Key of
    VK_RIGHT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus < High(Buttons) then
            NewFocus := Succ(NewFocus);
        until (NewFocus = High(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FFocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate;
        end;
      end;
    VK_LEFT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus > Low(Buttons) then
            NewFocus := Pred(NewFocus);
        until (NewFocus = Low(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FFocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate;
        end;
      end;
    VK_SPACE:
      begin
        if Buttons[FocusedButton].Enabled then
          Buttons[FocusedButton].Click;
      end;
  end;
end;

procedure TIB_DataSetBar.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  AdjustSize (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

procedure TIB_DataSetBar.BtnClick(Index: TIB_DataSetBar_ButtonType);
begin
  if not (csDesigning in ComponentState) and
     Assigned( FBeforeAction ) then begin
    FBeforeAction( Self );
  end;

  if (DataSource <> nil) then
  begin
    with DataSource do begin
      if DataSet <> nil then with DataSet do begin
        case Index of
        dbPrepare:   Prepare;
        dbUnprepare: Unprepare;
        dbOpen:      begin
          if ( StatementType = stSelect ) or
             ( StatementType = stSelectForUpdate ) then begin
            Open;
          end else begin
            Execute;
          end;
        end;
        dbClose: Close;
        end;
      end;
    end;
  end;
  if not (csDesigning in ComponentState) and
     Assigned( FAfterAction ) then begin
    FAfterAction( Self );
  end;
end;

procedure TIB_DataSetBar.BarClick( Sender: TObject );
begin
  BtnClick (TIB_DataSetBar_Button(Sender).Index);
end;

procedure TIB_DataSetBar_Button.Paint;
var
  R: TRect;
begin
  inherited Paint;
  if (GetFocus = Parent.Handle) and
     (FIndex = TIB_DataSetBar(Parent).FocusedButton) then
  begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then
      OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

function TIB_DataSetBar.GetOnReceiveFocus: TIB_DataLinkEvent;
begin
  Result := FIB_DataLink.OnReceiveFocus;
end;

procedure TIB_DataSetBar.SetOnReceiveFocus( AValue: TIB_DataLinkEvent );
begin
  FIB_DataLink.OnReceiveFocus := AValue;
end;

end.
