unit
  IB_Editors;

interface

uses
  Classes, Windows, SysUtils, DsgnIntf, TypInfo, Forms, Consts,

  DLG_IB_About,

  FRM_IB_Base,
  FRM_IB_Query,
  FRM_IB_WISQL,
  FRM_IB_Parameters,

  IB_Components,
  IB_BDataSet;

type
  TPasswordProperty = class(TStringProperty)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

  TVersionProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TIB_ConnectionEditor = class(TComponentEditor)
  private
  public
    procedure Edit; override;
  end;

  TIB_TransactionEditor = class(TComponentEditor)
  private
  public
    procedure Edit; override;
  end;

  TIB_StatementEditor = class(TComponentEditor)
  private
  public
    procedure Edit; override;
  end;

  TIB_DataSetEditor = class(TComponentEditor)
  private
  public
    procedure Edit; override;
  end;

  TIB_BDataSetEditor = class(TComponentEditor)
  private
  public
    procedure Edit; override;
  end;
{
  TIB_EventsEditor = class(TComponentEditor)
  private
  public
    procedure Edit; override;
  end;
}
procedure Register;

implementation

function TPasswordProperty.GetValue: string;
var
  ii: integer;
begin
  Result := inherited GetValue;
  for ii := 1 to Length( Result ) do Result[ ii ] := '*';
end;

procedure TPasswordProperty.SetValue(const Value: string);
var
  ii: integer;
begin
  for ii := 1 to Length( Value ) do begin
    if Value[ ii ] <> '*' then begin
      inherited SetValue( Value );
      Break;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TVersionProperty.Edit;
begin
  with TdlgAbout.Create( Application ) do try
    ShowModal;
  finally
    Free;
  end;
end;

function TVersionProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

{------------------------------------------------------------------------------}

procedure TIB_ConnectionEditor.Edit;
begin
  with TfrmWISQL.Create( Application ) do try
    pnClose.Visible := true;
    StatusBar.Visible := false;
    csForEdit.IB_Connection := Component as TIB_Connection;
    BaseKey := 'Software\IB Objects\Editor';
    ShowModal;
    if SaveSettings or SettingsSaved then begin
      Self.Designer.Modified;
    end;
  finally
    Free;
  end;
end;

procedure TIB_TransactionEditor.Edit;
begin
  with TfrmWISQL.Create( Application ) do try
    pnClose.Visible := true;
    StatusBar.Visible := false;
    tsForEdit.IB_Transaction := Component as TIB_Transaction;
    BaseKey := 'Software\IB Objects\Editor';
    ShowModal;
    if SaveSettings or SettingsSaved then begin
      Self.Designer.Modified;
    end;
  finally
    Free;
  end;
end;

procedure TIB_StatementEditor.Edit;
begin
  with TfrmWISQL.Create( Application ) do try
    pnClose.Visible := true;
    StatusBar.Visible := false;
    ssForEdit.Statement := Component as TIB_Statement;
    with Component as TIB_Statement do begin
      csForEdit.IB_Connection := IB_Connection;
      tsForEdit.IB_Transaction := IB_Transaction;
    end;
    BaseKey := 'Software\IB Objects\Editor';
    ShowModal;
    if SaveSettings or SettingsSaved then begin
      Self.Designer.Modified;
    end;
  finally
    Free;
  end;
end;

procedure TIB_DataSetEditor.Edit;
begin
  with TfrmWISQL.Create( Application ) do try
    pnClose.Visible := true;
    StatusBar.Visible := false;
    dsForEdit.DataSet := Component as TIB_DataSet;
    with Component as TIB_Statement do begin
      csForEdit.IB_Connection := IB_Connection;
      tsForEdit.IB_Transaction := IB_Transaction;
    end;
    BaseKey := 'Software\IB Objects\Editor';
    ShowModal;
    if SaveSettings or SettingsSaved then begin
      Self.Designer.Modified;
    end;
  finally
    Free;
  end;
end;

procedure TIB_BDataSetEditor.Edit;
begin
  with TfrmQuery.Create( Application ) do try
    Caption := Component.Owner.Name + '.' +
               Component.Name       + ' - ' +
               Caption;
    pnClose.Visible := true;
    StatusBar.Visible := false;
    dsQuerySource.DataSet := Component as TIB_BDataSet;
    IB_Connection := (Component as TIB_Statement).IB_Connection;
    IB_Transaction := (Component as TIB_Statement).IB_Transaction;
    BaseKey := 'Software\IB Objects\BDataSet Editor';
    ShowModal;
    if SaveSettings or SettingsSaved then begin
      Self.Designer.Modified;
    end;
  finally
    Free;
  end;
end;
{
procedure TIB_EventsEditor.Edit;
begin
  with TfrmQuery.Create( Application ) do try
    Caption := Component.Owner.Name + '.' +
               Component.Name       + ' - ' +
               Caption;
    pnClose.Visible := true;
    StatusBar.Visible := false;
    IB_Connection := (Component as TIB_Statement).IB_Connection;
    BaseKey := 'Software\IB Objects\Events Editor';
    ShowModal;
    if SaveSettings or SettingsSaved then begin
      Self.Designer.Modified;
    end;
  finally
    Free;
  end;
end;
}
{                                                                              }
{   Component Registration                                                     }
{                                                                              }

procedure Register;
begin
  RegisterComponentEditor( TIB_Connection, TIB_ConnectionEditor);
  RegisterComponentEditor( TIB_Transaction, TIB_TransactionEditor);
  RegisterComponentEditor( TIB_Statement, TIB_StatementEditor);
  RegisterComponentEditor( TIB_DataSet, TIB_DataSetEditor);
  RegisterComponentEditor( TIB_BDataSet, TIB_BDataSetEditor);

  RegisterPropertyEditor( TypeInfo( TIB_Row ),
                          TIB_Statement,
                          'Params',
                          TParametersPropertyEditor );
                          
  RegisterPropertyEditor( TypeInfo( string ),
                          TIB_Connection,
                          'Password',
                          TPasswordProperty );

  RegisterPropertyEditor( TypeInfo( string ),
                          TIB_Component,
                          'Version',
                          TVersionProperty );
end;

end.
