
{                                                                              }
{  IB_Grid                                                                     }
{                                                                              }

unit
  IB_Grid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids,

  IB_Header,
  IB_Components,
  IB_BDataSet;

type
  TIB_GridClickEvent = procedure ( Col, Row: longint ) of object;

  TIB_Grid = class;

  TIB_GridLink = class( TIB_DataLink )
  private
    FIB_Grid: TIB_Grid;
  protected
    procedure SysAfterUpdateDescriptors; override;
  end;

  TIB_Grid = class( TCustomGrid )
  private
  { Storage fields }
    FIB_DataLink: TIB_GridLink;
    FMovingRow: boolean;
    FAligningRow: boolean;
    FUpdatingRowCount: boolean;
    FSizingRow: boolean;
    FEditText: string;
    FAcquireFocus: Boolean;
    FReadOnly: boolean;
    FWasDblClick: boolean;
    FMouseIsDown: boolean;
    FMouseWasMoved: boolean;
    FMouseDownPoint: TPoint;
    FGridFields: TList;
    FBuffFields: TList;
    FRowIndicators: TImageList;
    FOrderingIndicators: TImageList;
    FAlwaysShowEditor: boolean;
    FRowSelect: boolean;
    FDrawFocusSelected: boolean;
    FThumbTracking: boolean;
    FRowLines: boolean;
    FColLines: boolean;
    FIndicateRow: boolean;
    FIndicateOrdering: boolean;
    FSelRow: integer;
    FFreezeGrid: boolean;
    FOrderingColor: TColor;
    FOrderingFont: TFont;
  { Event storage fields }
    FInGridlayout: boolean;
    FOnDrawCell: TDrawCellEvent;
    FOnDrawTitle: TDrawCellEvent;
    FOnGetEditMask: TGetEditEvent;
    FOnGetEditText: TGetEditEvent;
    FOnSelectCell: TSelectCellEvent;
    FOnSetEditText: TSetEditEvent;
    FOnTopLeftChanged: TNotifyEvent;
    FOnCellClick: TIB_GridClickEvent;
    FOnTitleClick: TIB_GridClickEvent;
  { Datalink event handlers }
    procedure IB_StateChanged( Sender: TIB_DataLink;
                               IB_DataSource: TIB_DataSource );
    procedure IB_LayoutChanged( Sender: TIB_DataLink;
                                IB_DataSource: TIB_DataSource );
    procedure IB_DataChanged( Sender: TIB_DataLink;
                              IB_DataSource: TIB_DataSource;
                              IB_Field: TIB_Column );
    procedure IB_UpdateData( Sender: TIB_DataLink;
                             IB_DataSource: TIB_DataSource;
                             IB_Field: TIB_Column );
  { Messages }
    procedure CMEnter( var Message: TCMEnter ); message CM_ENTER;
    procedure CMExit( var Message: TMessage ); message CM_EXIT;
    procedure WMVScroll( var Message: TWMVScroll ); message WM_VSCROLL;
  protected
    procedure SetOrderingColor( AValue: TColor ); virtual;
    procedure SetOrderingFont( AValue: TFont ); virtual;
    procedure SetReadOnly( AValue: boolean ); virtual;
  { Inherited Methods }
    procedure Loaded; override;
    procedure KeyDown( var Key: Word; Shift: TShiftState ); override;
    procedure KeyPress( var Key: Char ); override;
    procedure MouseDown( Button: TMouseButton;
                         Shift: TShiftState;
                         X, Y: Integer ); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp( Button: TMouseButton;
                       Shift: TShiftState;
                       X, Y: Integer ); override;
    procedure CreateWnd; override;
    procedure SizeChanged( OldColCount, OldRowCount: Longint ); override;
    procedure ColWidthsChanged; override;
    procedure ColumnMoved(FromIndex, ToIndex: Longint); override;
  { Inherited Methods For Event Dispatch }
    procedure DrawCell( ACol, ARow: Longint;
                        ARect: TRect;
                        AState: TGridDrawState ); override;
    function CanEditAcceptKey(Key: Char): Boolean; override;
    function CanEditModify: boolean; override;
    function CanEditShow: boolean; override;
    function GetEditText( ACol, ARow: Longint ): string; override;
    procedure SetEditText( ACol, ARow: Longint; const Value: string ); override;
    function GetEditMask( ACol, ARow: Longint ): string; override;
    function GetEditLimit: Integer; override;
    function SelectCell( ACol, ARow: Longint ): Boolean; override;
    procedure TopLeftChanged; override;
  { Property Access Methods }
    procedure SetDataSource( AValue: TIB_DataSource ); virtual;
    function GetDataSource: TIB_DataSource; virtual;
  { Utility Methods }
    function AcquireFocus: Boolean;
    procedure UpdateGridFieldsLayout;
  { System Methods }
    procedure SysStateChanged;
    procedure SysUpdateData;
    procedure SysUpdateRowCount;
    procedure SysUpdateScrollBar;
    procedure SysUpdateEditing;
  { Event dispatch Methods }
    procedure TitleClick( ACol, ARow: integer ); dynamic;
    procedure CellClick( ACol, ARow: integer ); dynamic;
    function GetGridFields( Index: integer ): TIB_Column;
    function GetBuffFields( Index: integer ): TIB_Column;
    function GetGridField: TIB_Column;
    function GetGridFieldCount: integer;
    function GetColMoving: boolean;
    function GetColSizing: boolean;
    procedure SetColMoving( AValue: boolean );
    procedure SetColSizing( AValue: boolean );
    function GetColToIndex( ACol: integer ): integer;
    function GetDataRow( ARow: integer ): integer;
    function GetGridRow( ARow: integer ): integer;
    procedure SetRowLines( AValue: boolean );
    procedure SetColLines( AValue: boolean );
    procedure SetIndicateRow( AValue: boolean );
    procedure SetIndicateOrdering( AValue: boolean );
    procedure SetAlwaysShowEditor( AValue: boolean );
    procedure SetRowSelect( AValue: boolean );
    procedure SetDrawFocusSelected( AValue: boolean );
    procedure SetThumbTracking( AValue: boolean );
  { Properties }
    property FixedCols default 0;
    property FixedRows default 1;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  { Methods }
    function GetCellDisplayText( ACol, ARow: Longint): string;
    procedure GetCellColor(    ACol, ARow: Longint;
                               AState: TGridDrawState;
                           var AColor: TColor );
    procedure GetCellFont(     ACol, ARow: Longint;
                               AState: TGridDrawState;
                           var AFont: TFont );
    function GetCellAlignment( ACol, ARow: Longint): TAlignment;
    procedure DefaultDrawCell( ACol, ARow: Longint;
                               ARect: TRect;
                               AState: TGridDrawState;
                               CellDisplayText: string;
                               CellAlignment: TAlignment );
    procedure DefaultDrawTitle( ACol, ARow: Longint;
                                ARect: TRect;
                                AState: TGridDrawState;
                                CellDisplayText: string;
                                CellAlignment: TAlignment );
  { Properties }
    property BuffFields[ Index: integer ]: TIB_Column read GetBuffFields;
    property GridFields[ Index: integer ]: TIB_Column read GetGridFields;
    property GridFieldCount: integer read GetGridFieldCount;
    property GridField: TIB_Column read GetGridField;
    property InGridLayout: boolean read FInGridLayout;
    property ColToIndex[ Index: integer ]: integer read GetColToIndex;
    property DataRow[ Index: integer ]: integer read GetDataRow;
    property GridRow[ Index: integer ]: integer read GetGridRow;
    property MouseDownPoint: TPoint read FMouseDownPoint;
  published
  { Properties }
    property AlwaysShowEditor: boolean read FAlwaysShowEditor
                                       write SetAlwaysShowEditor
                                       default false;
    property RowSelect: boolean read FRowSelect
                                write SetRowSelect
                                default false;
    property DrawFocusSelected: boolean read FDrawFocusSelected
                                        write SetDrawFocusSelected
                                        default false;
    property ThumbTracking: boolean read FThumbTracking
                                    write SetThumbTracking
                                    default false;
    property DefaultRowHeight default 21;
    property DataSource: TIB_DataSource read GetDataSource write SetDataSource;
    property ReadOnly: boolean read FReadOnly write SetReadOnly default false;
    property ColMoving: boolean read GetColMoving
                                write SetColMoving
                                default true;
    property ColSizing: boolean read GetColSizing
                                write SetColSizing
                                default true;
    property RowLines: boolean read FRowLines write SetRowLines default true;
    property ColLines: boolean read FColLines write SetColLines default true;
    property IndicateRow: boolean read FIndicateRow
                                  write SetIndicateRow
                                  default true;
    property IndicateOrdering: boolean read FIndicateOrdering
                                   write SetIndicateOrdering
                                   default true;
    property OrderingColor: TColor read FOrderingColor
                                   write SetOrderingColor
                                   default clWindow;
    property OrderingFont: TFont read FOrderingFont
                                 write SetOrderingFont;
  { Inherited properties }
    property Align;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FixedColor;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
  { Events }
    property OnDrawCell: TDrawCellEvent read FOnDrawCell
                                        write FOnDrawCell;
    property OnDrawTitle: TDrawCellEvent read FOnDrawTitle
                                         write FOnDrawTitle;
    property OnGetEditMask: TGetEditEvent read FOnGetEditMask
                                          write FOnGetEditMask;
    property OnGetEditText: TGetEditEvent read FOnGetEditText
                                          write FOnGetEditText;
    property OnSelectCell: TSelectCellEvent read FOnSelectCell
                                            write FOnSelectCell;
    property OnSetEditText: TSetEditEvent read FOnSetEditText
                                          write FOnSetEditText;
    property OnTopLeftChanged: TNotifyEvent read FOnTopLeftChanged
                                            write FOnTopLeftChanged;
    property OnCellClick: TIB_GridClickEvent read FOnCellClick
                                             write FOnCellClick;
    property OnTitleClick: TIB_GridClickEvent read FOnTitleClick
                                              write FOnTitleClick;
  { Inherited events }
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

implementation

{$R IB_Grid.RES}
uses
  IB_Parse;

type
{ This is used to suppress scrollbar flicker while scrolling. }
  THackCustomGrid = class( TCustomControl )
  private
    FAnchor: TGridCoord;
    FBorderStyle: TBorderStyle;
    FCanEditModify: Boolean;
    FColCount: Longint;
    FColWidths: Pointer;
    FTabStops: Pointer;
    FCurrent: TGridCoord;
    FDefaultColWidth: Integer;
    FDefaultRowHeight: Integer;
    FFixedCols: Integer;
    FFixedRows: Integer;
    FFixedColor: TColor;
    FGridLineWidth: Integer;
    FOptions: TGridOptions;
    FRowCount: Longint;
    FRowHeights: Pointer;
    FScrollBars: TScrollStyle;
end;

{------------------------------------------------------------------------------}

var
  DrawBitmap: TBitmap;
  UserCount: Integer;

procedure UsesBitmap;
begin
  if UserCount = 0 then
    DrawBitmap := TBitmap.Create;
  Inc(UserCount);
end;

procedure ReleaseBitmap;
begin
  Dec(UserCount);
  if UserCount = 0 then DrawBitmap.Free;
end;

function Max(X, Y: Integer): Integer;
begin
  Result := Y;
  if X > Y then Result := X;
end;

procedure WriteText(       ACanvas: TCanvas;
                           ARect: TRect;
                           DX, DY: Integer;
                     const Text: string;
                           Alignment: TAlignment);
const
  AlignFlags : array [TAlignment] of Integer =
    ( DT_LEFT or DT_WORDBREAK or DT_EXPANDTABS or DT_NOPREFIX,
      DT_RIGHT or DT_WORDBREAK or DT_EXPANDTABS or DT_NOPREFIX,
      DT_CENTER or DT_WORDBREAK or DT_EXPANDTABS or DT_NOPREFIX );
var
  B, R: TRect;
  I, Left: Integer;
begin
  I := ColorToRGB(ACanvas.Brush.Color);
  if GetNearestColor(ACanvas.Handle, I) = I then
  begin                       { Use ExtTextOut for solid colors }
    case Alignment of
      taLeftJustify:
        Left := ARect.Left + DX;
      taRightJustify:
        Left := ARect.Right - ACanvas.TextWidth(Text) - 3;
    else { taCenter }
      Left := ARect.Left + (ARect.Right - ARect.Left) shr 1
        - (ACanvas.TextWidth(Text) shr 1);
    end;
    ExtTextOut(ACanvas.Handle, Left, ARect.Top + DY, ETO_OPAQUE or
      ETO_CLIPPED, @ARect, PChar(Text), Length(Text), nil);
  end
  else begin                  { Use FillRect and Drawtext for dithered colors }
    with DrawBitmap, ARect do { Use offscreen bitmap to eliminate flicker and }
    begin                     { brush origin tics in painting / scrolling.    }
      Width := Max(Width, Right - Left);
      Height := Max(Height, Bottom - Top);
      R := Rect(DX, DY, Right - Left - 1, Bottom - Top - 1);
      B := Rect(0, 0, Right - Left, Bottom - Top);
    end;
    with DrawBitmap.Canvas do
    begin
      Font := ACanvas.Font;
      Font.Color := ACanvas.Font.Color;
      Brush := ACanvas.Brush;
      Brush.Style := bsSolid;
      FillRect(B);
      SetBkMode(Handle, TRANSPARENT);
      DrawText(Handle, PChar(Text), Length(Text), R, AlignFlags[Alignment]);
    end;
    ACanvas.CopyRect(ARect, DrawBitmap.Canvas, B);
  end;
end;

{------------------------------------------------------------------------------}

constructor TIB_Grid.Create( AOwner: TComponent );
var
  Bmp: TBitmap;
begin
  inherited Create( AOwner );
//DefaultDrawing := true;
  FAcquireFocus := True;
  FIB_DataLink := TIB_GridLink.Create( Self );
  with FIB_DataLink do begin
    FIB_Grid := Self;
    Control := Self;
    OnStateChanged := IB_StateChanged;
    OnLayoutChanged := IB_LayoutChanged;
    OnDataChange := IB_DataChanged;
    OnUpdateData := IB_UpdateData;
  end;
  FBuffFields := TList.Create;
  FGridFields := TList.Create;
  DefaultRowHeight := 21;
  FixedCols := 0;
  FixedRows := 1;
  Options := [ goFixedHorzLine, goFixedVertLine,
               goHorzLine,      goVertLine,
               goColSizing,     goColMoving,
               goTabs ];
  FSaveCellExtents := false;
  Bmp := TBitmap.Create;
  try
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_ARROW');
    FRowIndicators := TImageList.CreateSize(Bmp.Width, Bmp.Height);
    FRowIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_EDIT');
    FRowIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_INSERT');
    FRowIndicators.AddMasked(Bmp, clYellow);

//    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_UP_SORTABLE');
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_NATURAL_SORTABLE');
    FOrderingIndicators := TImageList.CreateSize(Bmp.Width, Bmp.Height);
    FOrderingIndicators.AddMasked(Bmp, clYellow);
//    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_DOWN_SORTABLE');
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_NATURAL_SORTABLE');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_BI_SORTABLE');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_NATURAL_SORTABLE');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_UP');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_DOWN');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_BI_UP');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_BI_DOWN');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
    Bmp.Handle := LoadBitmap(HInstance, 'IB_GRID_NATURAL');
    FOrderingIndicators.AddMasked(Bmp, clYellow);
  finally
    Bmp.Free;
  end;
  FRowLines := true;
  FColLines := true;
  FIndicateRow := true;
  FIndicateOrdering := true;
  FOrderingColor := clWindow;
  FOrderingFont := TFont.Create;
  UsesBitmap;
end;

destructor TIB_Grid.Destroy;
begin
  DataSource := nil;
  with FIB_DataLink do begin
    OnStateChanged := nil;
    OnLayoutChanged := nil;
    OnDataChange := nil;
    OnUpdateData := nil;
  end;
  FBuffFields.Free;
  FBuffFields := nil;
  FGridFields.Free;
  FGridFields := nil;
  FRowIndicators.Free;
  FRowIndicators := nil;
  FOrderingIndicators.Free;
  FOrderingIndicators := nil;
  FOrderingFont.Free;
  FOrderingFont := nil;;
  ReleaseBitmap;
  inherited Destroy;
end;

procedure TIB_Grid.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
end;

procedure TIB_Grid.CreateWnd;
begin
  inherited CreateWnd;
  SysUpdateRowCount;
  SysUpdateScrollBar;
end;

function TIB_Grid.GetBuffFields( Index: integer ): TIB_Column;
begin
  if ( Index >= 0 ) and ( Index < FBuffFields.Count ) then begin
    Result := FBuffFields.Items[ Index ];
  end else begin
    Result := nil;
  end;
end;

function TIB_Grid.GetGridFields( Index: integer ): TIB_Column;
begin
  if ( Index >= 0 ) and ( Index < GridFieldCount ) then begin
    Result := FGridFields.Items[ Index ];
  end else begin
    Result := nil;
  end;
end;

function TIB_Grid.GetGridField: TIB_Column;
begin
  Result := GridFields[ ColToIndex[ Col ]];
end;

function TIB_Grid.GetGridFieldCount: integer;
begin
  if FGridFields <> nil then begin
    Result := FGridFields.Count;
  end else begin
    Result := 0;
  end;
end;

function TIB_Grid.GetColToIndex( ACol: integer ): integer;
begin
  Result := ACol - FixedCols;
end;

function TIB_Grid.GetDataRow( ARow: integer ): integer;
begin
  Result := ARow - FixedRows + 1;
  if Result < 0 then Result := 0;
end;

function TIB_Grid.GetGridRow( ARow: integer ): integer;
begin
  Result := ARow + FixedRows - 1;
  if Result < 0 then Result := 0;
end;

{------------------------------------------------------------------------------}

function TIB_Grid.GetDataSource: TIB_DataSource;
begin
  if FIB_DataLink <> nil then begin
    Result := FIB_DataLink.DataSource;
  end else begin
    Result := nil;
  end;
end;

procedure TIB_Grid.SetDataSource( AValue: TIB_DataSource );
begin
  if FIB_DataLink <> nil then begin
    if FIB_DataLink.DataSource <> AValue then begin
      FIB_DataLink.DataSource := AValue;
      SysStateChanged;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_Grid.IB_StateChanged( Sender: TIB_DataLink;
                                          IB_DataSource: TIB_DataSource );
begin
  if FFreezeGrid then Exit;
  SysStateChanged;
  if IndicateRow then begin
    InvalidateCell( 0, FSelRow );
  end;
end;

procedure TIB_Grid.IB_LayoutChanged( Sender: TIB_DataLink;
                                     IB_DataSource: TIB_DataSource );
begin
  UpdateGridFieldsLayout;
  Invalidate;
  if not FIB_DataLink.Modifying then InvalidateEditor;
end;

procedure TIB_Grid.IB_DataChanged( Sender: TIB_DataLink;
                                   IB_DataSource: TIB_DataSource;
                                   IB_Field: TIB_Column );
begin
  if FFreezeGrid then Exit;
  if FIB_DataLink.Active then begin
    with FIB_DataLink.DataSet do begin
      if Row <> GridRow[ RowNum ] then begin
        if not BOF then SysUpdateRowCount;
        if not FAligningRow then begin
          try
            FMovingRow := true;
            if BOF then begin
              Row := FixedRows;
            end else if EOF then begin
              Row := RowCount - 1;
            end else begin
              Row := GridRow[ RowNum ];
            end;
          finally
            FMovingRow := false;
          end;
        end;
        if IndicateRow then begin
          InvalidateCell( 0, FSelRow );
          InvalidateCell( 0, GridRow[ RowNum ] );
        end;
      end else begin
        if IB_Field = nil then begin
          Self.InvalidateRow( GridRow[ RowNum] );
        end else if ( FGridFields <> nil ) and
                    ( FGridFields.IndexOf( IB_Field ) >= 0 ) then begin
          InvalidateCell( FGridFields.IndexOf( IB_Field ) + FixedCols, RowNum );
        end;
        InvalidateEditor;
        if InplaceEditor <> nil then begin
          InplaceEditor.Deselect;
        end;
      end;
    end;
  end;
  SysUpdateScrollBar;
end;

procedure TIB_Grid.IB_UpdateData( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}

procedure TIB_Grid.SysStateChanged;
begin
  if not ( csDestroying in ComponentState ) then begin
    UpdateGridFieldsLayout;
    SysUpdateRowCount;
    SysUpdateScrollBar;
    SysUpdateEditing;
    Invalidate;
    if not FIB_DataLink.Active then HideEditor else
    if not FIB_DataLink.Modifying then InvalidateEditor;
  end;
end;

procedure TIB_Grid.SysUpdateData;
begin
  if ( InPlaceEditor <> nil ) and InPlaceEditor.Modified then begin
    if FIB_DataLink.Active then with FIB_DataLink.DataSet do begin
      if GridField <> nil then GridField.AsString := FEditText;
      InvalidateCell( Col, Row );
    end else begin
      HideEditor;
    end;
    InPlaceEditor.Modified := false;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_Grid.SysUpdateRowCount;
var
  ii: integer;
  OldScrollBars: TScrollStyle; 
begin
  if FIB_DataLink.Prepared then begin
    with FIB_DataLink.DataSet do begin
      if Active then begin
        OldScrollBars := ScrollBars;
        try
          FUpdatingRowCount := true;
          if ScrollBars = ssBoth then begin
            THackCustomGrid( Self ).FScrollBars := ssHorizontal;
          end else begin
            THackCustomGrid( Self ).FScrollBars := ssNone;
          end;
          if BufferHasEOF then begin
            if BufferRowCount > 1 then begin
              RowCount := BufferRowCount + FixedRows;
            end else begin
              RowCount := FixedRows + 1;
            end;
          end else begin
            RowCount := BufferRowCount + FixedRows + 1000;
            for ii := 0 to VisibleRowCount - 1 do begin
              if ( RowNum <> TopRow + ii ) or
                 ( State = dsBrowse ) then begin
                BufferRowNum := TopRow + ii;
              end;
              if BufferEOF then Break;
            end;
            if BufferRowCount < 1 then begin
              RowCount := FixedRows + 1;
            end else if BufferHasEOF then begin
              RowCount := BufferRowCount + FixedRows;
            end;
          end;
        finally
          FUpdatingRowCount := false;
          THackCustomGrid( Self ).FScrollBars := OldScrollBars;
        end;
      end else begin
        RowCount := FixedRows + 1;
        Row      := FixedRows;
      end;
    end;
  end;
end;

procedure TIB_Grid.SysUpdateScrollBar;
var
  SIOld, SINew: TScrollInfo;
begin
  if HandleAllocated then begin
    if FIB_Datalink.Prepared then begin
      SIOld.cbSize := sizeof( SIOld );
      SIOld.fMask := SIF_ALL;
      GetScrollInfo( Self.Handle, SB_VERT, SIOld );
      SINew := SIOld;
      with FIB_Datalink.DataSet do begin
        SINew.nMin := 1;
        SINew.nPage := VisibleRowCount;
        SINew.nMax := BufferRowCount + VisibleRowCount - FixedRows - 1;
        if State in [dsInactive, dsBrowse, dsEdit] then
          SINew.nPos := RowNum - FixedRows;
      end;
      if ( SINew.nMin  <> SIOld.nMin  ) or
         ( SINew.nMax  <> SIOld.nMax  ) or
         ( SINew.nPage <> SIOld.nPage ) or
         ( SINew.nPos  <> SIOld.nPos  ) then begin
        SetScrollInfo( Self.Handle, SB_VERT, SINew, True );
      end;
    end;
  end;
end;

procedure TIB_Grid.WMVScroll( var Message: TWMVScroll );
var
  SI: TScrollInfo;
begin
  if not AcquireFocus then Exit;
  if FIB_Datalink.Active then begin
    with Message, FIB_DataLink.DataSet, FIB_Datalink do begin
      if NeedToPost or
        (( ScrollCode = SB_THUMBTRACK ) and
        not ThumbTracking ) then begin
        Exit;
      end else case ScrollCode of
        SB_LINEUP:   if not BOF then MoveBy( -1 );
        SB_LINEDOWN: if not EOF then MoveBy(  1 );
        SB_PAGEUP:   if not BOF then MoveBy( -VisibleRowCount );
        SB_PAGEDOWN: if not EOF then MoveBy(  VisibleRowCount );
        SB_THUMBTRACK,
        SB_THUMBPOSITION:
          begin
            SI.cbSize := SizeOf( SI );
            SI.fMask := SIF_ALL;
            GetScrollInfo( Self.Handle, SB_VERT, SI );
            with SI do begin
              if nTrackPos + nPage > nMax then begin
                Last
              end else begin
                DataSet.RowNum := nTrackPos;
              end;
            end;
          end;
        SB_BOTTOM: Last;
        SB_TOP: First;
        SB_ENDSCROLL: Exit;
      end;
    end;
  end;
end;

procedure TIB_Grid.SysUpdateEditing;
begin
  with FIB_DataLink do
    if not Self.ReadOnly and (( State in [ dsEdit, dsInsert ] ) or
       (( DataSource <> nil ) and DataSource.AutoEdit )) then
      inherited Options := inherited Options + [ goEditing ]
    else
      inherited Options := inherited Options - [ goEditing ];
end;

{------------------------------------------------------------------------------}
{   Grid handling procedures                                                   }
{------------------------------------------------------------------------------}

procedure TIB_Grid.SizeChanged(OldColCount, OldRowCount: Longint);
begin
  inherited SizeChanged( OldColCount, OldRowCount );
  SysUpdateScrollBar;
end;

procedure TIB_Grid.MouseDown( Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer );
var
  Cell: TGridCoord;
  OldCol,OldRow: Integer;
begin
  if not AcquireFocus then Exit;
  FMouseWasMoved := false;
  FMouseIsDown := true;
  FMouseDownPoint.X := X;
  FMouseDownPoint.Y := Y;
  FWasDblClick := ( ssDouble in Shift ) and ( Button = mbLeft );
  if FWasDblClick then begin
    DblClick;
    Exit;
  end;
  Cell := MouseCoord( X, Y );
  if Sizing( X, Y ) or
     ( ColMoving and (( Button  = mbLeft ) and
                      ( Cell.X >= FixedCols ) and
                      ( Cell.Y >= 0 ) and
                      ( Cell.Y < FixedRows ))) then begin
    SysUpdateData;
    inherited MouseDown( Button, Shift, X, Y )
  end else begin
    if ( csDesigning in ComponentState ) and
       ( Cell.Y < FixedRows ) then begin
      FIB_DataLink.UpdateData;
      inherited MouseDown( Button, Shift, X, Y );
    end else if FIB_Datalink.Active then begin
      FIB_DataLink.UpdateData;
      with Cell, FIB_Datalink.Dataset do begin
        InvalidateEditor;
        OldCol := Col;
        OldRow := Row;
        if ( Y >= FixedRows ) and ( X >= FixedCols ) then begin
          MoveColRow( X, Y, true, true );
        end;
        if ( X = OldCol ) and ( Y = OldRow ) then begin
          ShowEditor;
        end else begin
          InvalidateEditor; 
        end;
      end;
    end;
  end;
end;

procedure TIB_Grid.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y );
  if FMouseIsDown then begin
    FMouseWasMoved := ( goColMoving in Options ) and (
                      ( Abs(FMouseDownPoint.X - X ) > 4 ) or
                      ( Abs(FMouseDownPoint.Y - Y ) > 4 ));
  end;
end;

procedure TIB_Grid.MouseUp( Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer );
var
  Cell: TGridCoord;
  SaveState: TGridState;
  SaveShift: TShiftState;
begin
  FMouseIsDown := false;
  FMouseDownPoint.X := 0;
  FMouseDownPoint.Y := 0;
  SaveState := FGridState;
  SaveShift := Shift;
  inherited MouseUp( Button, Shift, X, Y );
  if ( SaveState = gsRowSizing ) or
     ( SaveState = gsColSizing ) or
     (( InplaceEditor <> nil ) and
      ( InplaceEditor.Visible ) and
      ( PtInRect( InplaceEditor.BoundsRect, Point( X, Y )))) then begin
    Exit;
  end else begin
    Cell := MouseCoord( X, Y );
    if ( Button  = mbLeft ) and
       ( Cell.X >= FixedCols ) and
       ( Cell.Y >= 0 ) then begin
      if not (( FMouseWasMoved ) or
              ( FWasDblClick )) then begin
        if Cell.Y < FixedRows then begin
          TitleClick( Cell.X, Cell.Y )
        end else begin
          CellClick ( Cell.X, Cell.Y );
        end;
      end;
    end;
  end;
end;

procedure TIB_Grid.CellClick( ACol, ARow: integer );
begin
  if Assigned( FOnCellClick ) then begin
    FOnCellClick( ACol, ARow );
  end;
end;

procedure TIB_Grid.TitleClick( ACol, ARow: integer );
var
  tmpStr: string;
  tmpNo,
  tmpCode,
  tmpRow: integer;
begin
  if FIB_DataLink.Prepared then begin
    with DataSource.Dataset, GridFields[ ColToIndex[ ACol ]] do begin
      tmpStr := GetColumnAttributeParams( RelName, Fieldname,
                                          OrderingLinks,
                                          '' );
      if tmpStr <> '' then begin
        Val( tmpStr, tmpNo, tmpCode );
        if tmpCode = 0 then begin
          if tmpNo <> 0 then begin
            if Abs( OrderingItemNo ) = tmpNo then begin
              if GetOrderingSQL( -OrderingItemNo ) <> '' then begin
                tmpNo := -OrderingItemNo;
              end else begin
                tmpNo := OrderingItemNo; // Just in case...
              end;
            end else begin
              if GetOrderingSQL( tmpNo ) = '' then begin
                tmpNo := -tmpNo;
                if GetOrderingSQL( tmpNo ) = '' then begin
                  tmpNo := OrderingItemNo; // Just in case...
                end;
              end;
            end;
          end;
          if tmpNo <> OrderingItemNo then begin
            try
              FFreezeGrid := true;
              tmpRow := RowNum;
              OrderingItemNo := tmpNo;
              RowNum := tmpRow;
//              Row    := GridRow[ tmpRow ];
            finally
              FFreezeGrid := false;
              SysUpdateRowCount;
              SysUpdateScrollBar;
              Invalidate;
              InvalidateEditor;
            end;
          end;
        end;
      end;
    end;
  end;
  if Assigned( FOnTitleClick ) then begin
    FOnTitleClick( ACol, ARow );
  end;
end;

procedure TIB_Grid.UpdateGridFieldsLayout;
var
  ii: integer;
begin
  if IndicateRow then begin
    if ColCount < 2 then ColCount := 2;
    FixedCols := 1;
    ColWidths[ 0 ] := 11;
  end else begin
    FixedCols := 0;
  end;
  if FIB_DataLink.Prepared then begin
    with DataSource.DataSet do begin
      FInGridLayout := true;
      try
        FBuffFields.Clear;
        FGridFields.Clear;
        for ii := 0 to FieldCount - 1 do begin
          if Fields[ ii ].Visible then begin
            FGridFields.Add( Fields[ ii ] );
            FBuffFields.Add( BufferFields.ByName( Fields[ ii ].FullFieldName ));
          end;
        end;
        if GridFieldCount + FixedCols >= 1 then begin
          ColCount := GridFieldCount + FixedCols;
        end else begin
          ColCount := 1;
        end;
        for ii := 0 to GridFieldCount - 1 do begin
          if GridFields[ ii ] <> nil then begin
            ColWidths[ ii + FixedCols ] := GridFields[ ii ].DisplayWidth;
          end else begin
            ColWidths[ ii + FixedCols ] := DefaultColWidth;
          end;
        end;
      finally
        FInGridlayout := false;
      end;
    end;
  end else begin
    FBuffFields.Clear;
    FGridFields.Clear;
    ColCount := FixedCols + 1;
    RowCount := FixedRows + 1;
    Invalidate;
  end;
end;

procedure TIB_Grid.CMEnter( var Message: TCMEnter );
begin
  FIB_DataLink.SetFocus;
  inherited;
end;

procedure TIB_Grid.KeyDown( var Key: Word; Shift: TShiftState );
var
  OldScrollBars: TScrollStyle;
begin
  if FIB_DataLink.Active then with DataSource.DataSet do begin
    if ssCtrl in Shift then case Key of
      VK_HOME: begin First; Key := 0; end;
      VK_DOWN: begin Next;  Key := 0; end;
      VK_UP:   begin Prior; Key := 0; end;
      VK_END:  begin Last;  Key := 0; end;
    end;
  end;
  OldScrollBars := ScrollBars;
  try
    if ScrollBars = ssBoth then begin
      THackCustomGrid( Self ).FScrollBars := ssHorizontal;
    end else begin
      THackCustomGrid( Self ).FScrollBars := ssNone;
    end;
    inherited KeyDown( Key, Shift );
  finally
    THackCustomGrid( Self ).FScrollBars := OldScrollBars;
  end;
end;

procedure TIB_Grid.CMExit( var Message: TMessage );
begin
  try
    SysUpdateData;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

function TIB_Grid.AcquireFocus: Boolean;
begin
  Result := True;
  if FAcquireFocus and
     CanFocus and
     not ( csDesigning in ComponentState ) then begin
    SetFocus;
    Result := Focused or ( InplaceEditor <> nil ) and InplaceEditor.Focused;
  end;
end;

{------------------------------------------------------------------------------}

function TIB_Grid.CanEditShow: boolean;
begin
  Result := inherited CanEditShow and
            ( GridField <> nil ) and
            FIB_DataLink.Active and
            ( FIB_DataLink.DataSet.Fields.RowState <> rsNone ) and
            not GridField.ReadOnly and
            FIB_DataLink.CanModify;
end;

function TIB_Grid.CanEditModify: Boolean;
begin
  Result := inherited CanEditModify and
            ( GridField <> nil ) and
            FIB_DataLink.Active and
            ( FIB_DataLink.DataSet.Fields.RowState <> rsNone ) and
            not GridField.ReadOnly and
            FIB_DataLink.CanModify and
            FIB_DataLink.Modify;
end;

function TIB_Grid.CanEditAcceptKey(Key: Char): Boolean;
begin
  Result := ( GridField <> nil ) and GridField.IsValidChar( Key );
end;

function TIB_Grid.GetEditMask( ACol, ARow: Longint ): string;
begin
  if GridFields[ ColToIndex[ ACol ]] <> nil then begin
    Result := GridFields[ ColToIndex[ ACol ]].EditMask;
  end else begin
    Result := '';
  end;
  if Assigned( FOnGetEditMask ) then begin
    FOnGetEditMask( Self, ACol, ARow, Result );
  end;
end;

function TIB_Grid.GetEditLimit: Integer;
begin
  if ( GridField <> nil ) and (
     ( GridField.SQLType = SQL_Varying  ) or
     ( GridField.SQLType = SQL_Varying_ ) or
     ( GridField.SQLType = SQL_Text     ) or
     ( GridField.SQLType = SQL_Text_    )) then begin
    Result := GridField.SQLLen;
  end else begin
    Result := 0;
  end;
end;

{------------------------------------------------------------------------------}

function TIB_Grid.GetEditText( ACol, ARow: Longint ): string;
begin
  if GridFields[ ColToIndex[ ACol ]] <> nil then begin
    Result := GridFields[ ColToIndex[ ACol ]].AsString;
  end else begin
    Result := '';
  end;
  FEditText := Result;
  if Assigned( FOnGetEditText ) then begin
    FOnGetEditText( Self, ACol, ARow, Result );
  end;
end;

procedure TIB_Grid.SetEditText(       ACol, ARow: Longint;
                                      const Value: string );
begin
  if Assigned( FOnSetEditText ) then begin
    FOnSetEditText( Self, ACol, ARow, Value );
  end;
  if ( FEditText <> Value ) and
     ( FIB_DataLink.Modify ) then begin
    FEditText := Value;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_Grid.GetCellColor(     ACol, ARow: Longint;
                                     AState: TGridDrawState;
                                 var AColor: TColor );
begin
  with FIB_DataLink do begin
    if (gdSelected in AState) and ( RowNum = DataRow[ ARow ] ) and 
       (not (gdFocused in AState) or
       ([goDrawFocusSelected, goRowSelect] * Options <> [])) then begin
      AColor := clHighlight;
    end else begin
      if  ColorScheme then begin
        if State = dsSearch then begin
          AColor := Color;
        end else if ( ARow >= FixedRows ) and
                    ( GridFields[ ColToIndex[ ACol ]] <> nil ) then begin
          if NeedToPost and ( ARow = DataSet.RowNum ) then begin
            if GridFields[ ColToIndex[ ACol ]].ReadOnly or
               Self.ReadOnly then begin
              AColor := clSilver;
            end else begin
              AColor := Color;
            end;
          end;
        end;
      end;
      if DataSet <> nil then with DataSet do begin
        if IndicateOrdering and ( State <> dsSearch ) then begin
          if ( GridFields[ ColToIndex[ ACol ]] <> nil ) and
             (( ARow <> DataSet.RowNum ) or not NeedToPost ) and
             ( GridFields[ ColToIndex[ACol]] = OrderingField ) then begin
            AColor := OrderingColor;
          end;
        end;
      end;
    end;
  end;
end;

procedure TIB_Grid.GetCellFont(     ACol, ARow: Longint;
                                    AState: TGridDrawState;
                                var AFont: TFont );
begin
  with FIB_DataLink do begin
    if (gdSelected in AState) and ( RowNum = DataRow[ ARow ] ) and 
       (not (gdFocused in AState) or
       ([goDrawFocusSelected, goRowSelect] * Options <> [])) then begin
      AFont.Color := clHighlightText;
    end else begin
      if DataSet <> nil then with DataSet do begin
        if IndicateOrdering and ( State <> dsSearch ) then begin
          if ( ARow >= FixedRows ) and
             ( GridFields[ ColToIndex[ ACol ]] <> nil ) and
             (( ARow <> DataSet.RowNum ) or not NeedToPost ) and
             ( GridFields[ ColToIndex[ACol]] = OrderingField ) then begin
            AFont.Assign( OrderingFont );
          end;
        end;
      end;
    end;
  end;
end;

function TIB_Grid.GetCellAlignment( ACol, ARow: Longint): TAlignment;
begin
  if ( ARow >= FixedRows ) and
     ( GridFields[ ColToIndex[ ACol ]] <> nil ) then begin
    Result := GridFields[ ColToIndex[ ACol ]].Alignment;
  end else begin
    Result := taLeftJustify;
  end;
end;

function TIB_Grid.GetCellDisplayText( ACol, ARow: Longint): string;
begin
  if FIB_DataLink.Prepared then with FIB_DataLink.DataSet do begin
    if ARow < FixedRows then begin
      if ( ARow = FixedRows - 1 ) and
         ( GridFields[ ColToIndex[ ACol ]] <> nil ) then begin
        Result := GridFields[ ColToIndex[ ACol ]].DisplayName;
      end else begin
        Result := '';
      end;
    end else if ACol < FixedCols then begin
      Result := '';
    end else if Active then begin
      if DataRow[ ARow ] = RowNum then begin
        if Fields.RowState <> rsNone then begin
          Result := GridFields[ ColToIndex[ ACol ]].DisplayText;
        end else begin
          Result := '';
        end;
      end else begin
        BufferRowNum := DataRow[ ARow ];
        if BufferRowNum = DataRow[ ARow ] then begin
          Result := BuffFields[ ColToIndex[ ACol ]].DisplayText;
        end else begin
          if BufferHasEOF then SysUpdateRowCount;
          Result := '';
        end;
      end;
    end else begin
      Result := '';
    end;
  end;
end;

procedure TIB_Grid.DrawCell( ACol, ARow: Longint;
                                   ARect: TRect;
                                   AState: TGridDrawState );
var
  Indicator: integer;
  FrameOffs: byte;
  tmpRect: TRect;
  tmpColor: TColor;
  tmpFont: TFont;
begin
  if csLoading in ComponentState then begin
    Canvas.Brush.Color := Color;
    Canvas.FillRect(ARect);
    Exit;
  end;
  if ( gdFixed in AState ) and ( ACol = FixedCols - 1 ) then begin
    tmpRect := ARect;
    if ColLines or RowLines then begin
      InflateRect( tmpRect, -1, -1 );
      FrameOffs := 1;
    end else begin
      FrameOffs := 2;
    end;
    Canvas.Brush.Color := FixedColor;
    Canvas.FillRect(ARect);
    if Assigned( FIB_DataLink ) and
       FIB_DataLink.Active and
       ( FIB_DataLink.DataSet.Fields.RowState <> rsNone ) and
       ( DataRow[ ARow ] = FIB_DataLink.DataSet.RowNum ) then begin
      Indicator := 0;
      if FIB_DataLink.DataSet <> nil then begin
        case FIB_DataLink.DataSet.State of
          dsEdit: Indicator := 1;
          dsInsert: Indicator := 2;
        end;
      end;
      with FRowIndicators do begin
        BkColor := FixedColor;
        Draw( Canvas,
              tmpRect.Right - Width - FrameOffs,
              ( tmpRect.Top + tmpRect.Bottom - Height ) shr 1,
              Indicator);
      end;
      FSelRow := ARow;
    end;
  end else if ( gdFixed in AState ) and ( ARow = FixedRows - 1 ) then begin
    DefaultDrawTitle( ACol, ARow,
                      ARect,
                      AState,
                      GetCellDisplayText( ACol, ARow ),
                      GetCellAlignment( ACol, ARow ));
    if Assigned( FOnDrawTitle ) then begin
      FOnDrawTitle( Self, ACol, ARow, ARect, AState );
    end;
  end else begin
    if Assigned( FOnDrawCell ) then begin
      FOnDrawCell( Self, ACol, ARow, ARect, AState );
    end else begin
      tmpColor := Color;
      GetCellColor( ACol, ARow, AState, tmpColor );
      Canvas.Brush.Color := tmpColor;
      tmpFont := Canvas.Font;
      tmpFont.Color := Self.Font.Color;
      GetCellFont( ACol, ARow, AState, tmpFont );
      DefaultDrawCell( ACol, ARow,
                       ARect,
                       AState,
                       GetCellDisplayText( ACol, ARow ),
                       GetCellAlignment( ACol, ARow ));
    end;
  end;
end;

procedure TIB_Grid.DefaultDrawCell( ACol, ARow: longint;
                                          ARect: TRect;
                                          AState: TGridDrawState;
                                          CellDisplayText: string;
                                          CellAlignment: TAlignment );
begin
  WriteText( Canvas, ARect, 2, 2, CellDisplayText, CellAlignment );
end;

procedure TIB_Grid.DefaultDrawTitle( ACol, ARow: longint;
                                           ARect: TRect;
                                           AState: TGridDrawState;
                                           CellDisplayText: string;
                                           CellAlignment: TAlignment );
var
  Indicator: integer;
  tmpRect: TRect;
  tmpStr: string;
  tmpNo,
  tmpCode: integer;
begin
  DefaultDrawCell( ACol, ARow, ARect, AState, CellDisplayText, CellAlignment );
  if IndicateOrdering then begin
    Indicator := -1;
    if FIB_DataLink.Prepared then begin
      if GridFields[ ColToIndex[ ACol ]] <> nil then
      with GridFields[ ColToIndex[ ACol ]] do begin
        tmpStr := GetColumnAttributeParams( RelName, Fieldname,
                                            DataSource.Dataset.OrderingLinks,
                                            '' );
      end;
      // 0 IB_GRID_UP_SORTABLE
      // 1 IB_GRID_DOWN_SORTABLE
      // 2 IB_GRID_BI_SORTABLE
      // 3 IB_GRID_NATURAL_SORTABLE
      // 4 IB_GRID_UP
      // 5 IB_GRID_DOWN
      // 6 IB_GRID_BI_UP
      // 7 IB_GRID_BI_DOWN
      // 8 IB_GRID_NATURAL
      if tmpStr <> '' then with DataSource.Dataset do begin
        Val( tmpStr, tmpNo, tmpCode );
        if tmpCode = 0 then begin
          if Abs( OrderingItemNo ) = tmpNo then begin
            if OrderingItemNo > 0 then begin
              if GetOrderingSQL( -tmpNo ) <> '' then begin
                Indicator := 6;
              end else begin
                Indicator := 4;
              end;
            end else if OrderingItemNo < 0 then begin
              if GetOrderingSQL( tmpNo ) <> '' then begin
                Indicator := 7;
              end else begin
                Indicator := 5;
              end;
            end else begin
              Indicator := 8;
            end;
          end else begin
              Indicator := 3;
{
            if tmpNo = 0 then begin
              Indicator := 3;
            end else if GetOrderingSQL( tmpNo ) <> '' then begin
              if GetOrderingSQL( -tmpNo ) <> '' then begin
                Indicator := 2;
              end else begin
                Indicator := 0;
              end;
            end else if GetOrderingSQL( -tmpNo ) <> '' then begin
              Indicator := 1;
            end;
}
          end;
        end;
      end;
    end;
    if Indicator >= 0 then begin
      tmpRect := ARect;
      if ColLines or RowLines then begin
        InflateRect( tmpRect, -1, -1 );
      end;
      if ( tmpRect.Right - tmpRect.Left ) >
         ( Canvas.TextWidth(CellDisplayText) + 3 +
           FOrderingIndicators.Width ) then begin
        tmpRect.Left := tmpRect.Left + Canvas.TextWidth(CellDisplayText) + 3;
        with FOrderingIndicators do begin
          BkColor := FixedColor;
          Draw( Canvas, tmpRect.Left,
                ( tmpRect.Top + tmpRect.Bottom - Height ) shr 1,
                Indicator);
        end;
      end;
    end;
  end;
end;

function TIB_Grid.SelectCell( ACol, ARow: longint ): boolean;
begin
  Result := true;
  SysUpdateData;
  if not FMovingRow and
     not FAligningRow and
     not FUpdatingRowCount then begin
    if Assigned( FOnSelectCell ) then begin
      FOnSelectCell( Self, ACol, ARow, Result );
    end;
    if Result and FIB_DataLink.Active then begin
      with FIB_DataLink.DataSet do begin
        if ( ARow >= FixedRows ) and
           ( DataRow[ ARow ] <> RowNum ) then begin
          try
            FAligningRow := true;
            RowNum := DataRow[ ARow ];
          finally
            FAligningRow := false;
          end;
        end;
      end;
    end;
  end;
  if AlwaysShowEditor then ShowEditor;
end;

procedure TIB_Grid.TopLeftChanged;
begin
  inherited TopLeftChanged;
  if Assigned( FOnTopLeftChanged ) then begin
    FOnTopLeftChanged( Self );
  end;
end;

procedure TIB_Grid.KeyPress( var Key: Char );
begin
  if ( Key = #13 ) then begin
    SysUpdateData;
  end;
  inherited KeyPress( Key );
end;

{------------------------------------------------------------------------------}

procedure TIB_Grid.SetRowLines( AValue: boolean );
begin
  if RowLines <> AValue then begin
    FRowLines := AValue;
    if AValue then begin
      Options := Options + [goFixedHorzLine, goHorzLine];
    end else begin
      Options := Options - [goFixedHorzLine, goHorzLine];
    end;
  end;
end;

procedure TIB_Grid.SetColLines( AValue: boolean );
begin
  if ColLines <> AValue then begin
    FColLines := AValue;
    if AValue then begin
      Options := Options + [goFixedVertLine, goVertLine];
    end else begin
      Options := Options - [goFixedVertLine, goVertLine];
    end;
  end;
end;

procedure TIB_Grid.SetThumbTracking( AValue: boolean );
begin
  if ThumbTracking <> AValue then begin
    FThumbTracking := AValue;
    if AValue then begin
      Options := Options + [goThumbTracking];
    end else begin
      Options := Options - [goThumbTracking];
    end;
  end;
end;

procedure TIB_Grid.SetAlwaysShowEditor( AValue: boolean );
begin
  if AlwaysShowEditor <> AValue then begin
    FAlwaysShowEditor := AValue;
    if AValue then begin
      Options := Options + [goAlwaysShowEditor];
    end else begin
      Options := Options - [goAlwaysShowEditor];
    end;
  end;
end;

procedure TIB_Grid.SetRowSelect( AValue: boolean );
begin
  if RowSelect <> AValue then begin
    FRowSelect := AValue;
    if AValue then begin
      Options := Options + [goRowSelect];
    end else begin
      Options := Options - [goRowSelect];
    end;
  end;
end;

procedure TIB_Grid.SetDrawFocusSelected( AValue: boolean );
begin
  if DrawFocusSelected <> AValue then begin
    FDrawFocusSelected := AValue;
    if AValue then begin
      Options := Options + [goDrawFocusSelected];
    end else begin
      Options := Options - [goDrawFocusSelected];
    end;
  end;
end;

procedure TIB_Grid.SetIndicateRow( AValue: boolean );
begin
  if IndicateRow <> AValue then begin
    FIndicateRow := AValue;
    UpdateGridFieldsLayout;
    SysStateChanged;
  end;
end;

procedure TIB_Grid.SetIndicateOrdering( AValue: boolean );
begin
  if IndicateOrdering <> AValue then begin
    FIndicateOrdering := AValue;
    Invalidate;
  end;
end;

procedure TIB_Grid.SetOrderingColor( AValue: TColor );
begin
  if OrderingColor <> AValue then begin
    FOrderingColor := AValue;
    Invalidate; // Should invalidate just the column.
  end;
end;

procedure TIB_Grid.SetOrderingFont( AValue: TFont );
begin
  FOrderingFont.Assign( AValue );
  Invalidate; // Should invalidate just the column.
end;

procedure TIB_Grid.SetReadOnly( AValue: boolean );
begin
  if ReadOnly <> AValue then begin
    FReadOnly := AValue;
    SysUpdateEditing;
  end;
end;

function TIB_Grid.GetColMoving: boolean;
begin
  Result := goColMoving in Options;
end;

function TIB_Grid.GetColSizing: boolean;
begin
  Result := goColSizing in Options;
end;

procedure TIB_Grid.SetColMoving( AValue: boolean );
begin
  if ColMoving <> AValue then begin
    if AValue then begin
      Options := Options + [ goColMoving ];
    end else begin
      Options := Options - [ goColMoving ];
    end;
  end;
end;

procedure TIB_Grid.SetColSizing( AValue: boolean );
begin
  if ColSizing <> AValue then begin
    if AValue then begin
      Options := Options + [ goColSizing ];
    end else begin
      Options := Options - [ goColSizing ];
    end;
  end;
end;

procedure TIB_Grid.ColWidthsChanged;
var
  ii: integer;
begin
  inherited ColWidthsChanged;
  if not InGridLayout then begin
    for ii := 0 to GridFieldCount - 1 do begin
      GridFields[ ii ].DisplayWidth := ColWidths[ ii + FixedCols ];
    end;
  end;
end;

procedure TIB_Grid.ColumnMoved( FromIndex, ToIndex: Longint );
begin
  Dec( FromIndex, FixedCols );
  Dec( ToIndex,   FixedCols );
  GridFields[ FromIndex ].Index := ToIndex;
  FMouseWasMoved := true;
  SysStateChanged;
end;

procedure TIB_GridLink.SysAfterUpdateDescriptors;
begin
  if FIB_Grid <> nil then FIB_Grid.UpdateGridFieldsLayout;
end;

end.

