unit IB_SearchPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,

  IB_Constants,
  IB_Components,
  IB_Controls;


type
  TIB_SearchPanel = class(TScrollBox)
  private
    { Private declarations }
    FIB_DataLink: TIB_DataLink;
    CursorLabelList: TList;
    CursorEditList: TList;
    FLabelWidth: integer;
    FEditWidth: integer;
    FSpaceWidth: integer;
    FLineHeight: integer;
    FSpaceHeight: integer;
    FTopMargin: integer;
    FLeftMargin: integer;
    FPreventEditing: TIB_StringList;
    FPreventInserting: TIB_StringList;
    FPreventSearching: TIB_StringList;
    procedure SetPreventEdit( AValue: TIB_StringList );
    procedure SetPreventInsert( AValue: TIB_StringList );
    procedure SetPreventSearch( AValue: TIB_StringList );
    procedure PreventStringsChanged( Sender: TObject );
    procedure SetLabelWidth( AValue: integer );
    procedure SetEditWidth( AValue: integer );
    procedure SetSpaceWidth( AValue: integer );
    procedure SetLineHeight( AValue: integer );
    procedure SetSpaceHeight( AValue: integer );
    procedure SetTopMargin( AValue: integer );
    procedure SetLeftMargin( AValue: integer );
    procedure PreparedChanged( Sender: TIB_DataLink;
                               DataSource: TIB_DataSource);
    procedure SetDataSource( AValue: TIB_DataSource );
    function GetDataSource: TIB_DataSource;
  protected
    { Protected declarations }
    procedure SysUpdate;
{$IFNDEF VER100}
    procedure GetChildren(Proc: TGetChildProc ); override;
{$ELSE}
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
{$ENDIF}
  public
    { Public declarations }
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property DataSource: TIB_DataSource read GetDataSource
                                        write SetDataSource;
    property LabelWidth: integer read FLabelWidth
                                 write SetLabelWidth
                                 default 120;
    property EditWidth: integer read FEditWidth
                                write SetEditWidth
                                default 300;
    property SpaceWidth: integer read FSpaceWidth
                                 write SetSpaceWidth
                                 default 6;
    property LineHeight: integer read FLineHeight
                                 write SetLineHeight
                                 default 21;
    property SpaceHeight: integer read FSpaceHeight
                                  write SetSpaceHeight
                                  default 4;
    property TopMargin: integer read FTopMargin
                                write SetTopMargin
                                default 6;
    property LeftMargin: integer read FLeftMargin
                                 write SetLeftMargin
                                 default 8;
    property PreventEditing: TIB_StringList read FPreventEditing
                                            write SetPreventEdit;
    property PreventInserting: TIB_StringList read FPreventInserting
                                              write SetPreventInsert;
    property PreventSearching: TIB_StringList read FPreventSearching
                                              write SetPreventSearch;
  end;

implementation

constructor TIB_SearchPanel.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FIB_DataLink := TIB_DataLink.Create( Self );
  with FIB_DataLink do begin
    Control := Self;
    OnPreparedChanged := PreparedChanged;
    AfterAssignment := PreparedChanged;
  end;
  FPreventEditing := TIB_StringList.Create;
  FPreventInserting := TIB_StringList.Create;
  FPreventSearching := TIB_StringList.Create;
  CursorLabelList := TList.Create;
  CursorEditList := TList.Create;
  FLabelWidth  := 120;
  FEditWidth   := 300;
  FSpaceWidth  := 6;
  FLineHeight  := 21;
  FSpaceHeight := 4;
  FTopMargin   := 6;
  FLeftMargin  := 8;
end;

destructor TIB_SearchPanel.Destroy;
begin
  CursorLabelList.Free;
  CursorEditList.Free;
  FPreventEditing.Free;
  FPreventInserting.Free;
  FPreventSearching.Free;
  inherited Destroy;
end;

function TIB_SearchPanel.GetDataSource: TIB_DataSource;
begin
  Result := FIB_DataLink.DataSource;
end;

procedure TIB_SearchPanel.SetDataSource( AValue: TIB_DataSource );
begin
  FIB_DataLink.DataSource := AValue;
end;

procedure TIB_SearchPanel.PreparedChanged( Sender: TIB_DataLink;
                                           DataSource: TIB_DataSource);
begin
  SysUpdate;
end;

procedure TIB_SearchPanel.SysUpdate;
var
  ii: integer;
  tmpLabel: TLabel;
  tmpEdit: TIB_Edit;
begin
  if FIB_DataLink.Prepared then begin
    with FIB_DataLink.DataSet do begin
      for ii := 0 to FieldCount - 1 do begin
        with Fields[ ii ] do begin
          if ii >= CursorLabelList.Count then begin
            tmpLabel := TLabel.Create( Self );
            tmpEdit := TIB_Edit.Create( Self );
            CursorLabelList.Add( tmpLabel );
            CursorEditList.Add( tmpEdit );
            tmpLabel.Visible := false;
            tmpEdit.Visible := false;
            tmpLabel.Parent := Self;
            tmpEdit.Parent := Self;
          end else begin
            tmpLabel := TLabel( CursorLabelList.Items[ ii ] );
            tmpEdit := TIB_Edit( CursorEditList.Items[ ii ] );
          end;
          tmpEdit.DataField := FieldName;
          tmpEdit.DataSource := DataSource;

          tmpLabel.Caption := DisplayName;

          tmpLabel.Width := LabelWidth;
          tmpEdit.Width := EditWidth;
          tmpLabel.Top := ii * (LineHeight + SpaceHeight) + TopMargin;
          tmpEdit.Top := ii * (LineHeight + SpaceHeight) + TopMargin;
          tmpLabel.Left := LeftMargin;
          tmpEdit.Left := LeftMargin + LabelWidth + SpaceWidth;

          tmpEdit.PreventInserting := IsAttributeSet[ IB_NOINSERT ];
          tmpEdit.PreventEditing   := IsAttributeSet[ IB_NOEDIT   ];
          tmpEdit.PreventSearching := IsAttributeSet[ IB_NOSEARCH ];
          if IsAttributeSet[ IB_UPCASE ] then begin
            tmpEdit.CharCase := ecUpperCase;
          end else if IsAttributeSet[ IB_LOWCASE ] then begin
            tmpEdit.CharCase := ecLowerCase;
          end else begin
            tmpEdit.CharCase := ecNormal;
          end;
        end;
      end;
      for ii := 0 to FieldCount - 1 do with Fields[ ii ] do begin
        TWinControl( CursorLabelList.Items[ ii ] ).Visible := true;
        TWinControl( CursorEditList.Items[ ii ] ).Visible := true;
      end;
    end;
  end else begin
    for ii := CursorLabelList.Count - 1 downto 0 do begin
      TWinControl( CursorLabelList.Items[ ii ] ).Visible := false;
      TWinControl( CursorEditList.Items[ ii ] ).Visible := false;
    end;
  end;
end;

{$IFNDEF VER100}
procedure TIB_SearchPanel.GetChildren(Proc: TGetChildProc );
{$ELSE}
procedure TIB_SearchPanel.GetChildren(Proc: TGetChildProc; Root: TComponent);
{$ENDIF}
begin
end;

procedure TIB_SearchPanel.SetLabelWidth( AValue: integer );
begin
  if LabelWidth <> AValue then begin
    FLabelWidth := AValue;
    SysUpdate;
  end;
end;
procedure TIB_SearchPanel.SetEditWidth( AValue: integer );
begin
  if EditWidth <> AValue then begin
    FEditWidth := AValue;
    SysUpdate;
  end;
end;
procedure TIB_SearchPanel.SetSpaceWidth( AValue: integer );
begin
  if SpaceWidth <> AValue then begin
    FSpaceWidth := AValue;
    SysUpdate;
  end;
end;
procedure TIB_SearchPanel.SetLineHeight( AValue: integer );
begin
  if LineHeight <> AValue then begin
    FLineHeight := AValue;
    SysUpdate;
  end;
end;
procedure TIB_SearchPanel.SetSpaceHeight( AValue: integer );
begin
  if SpaceHeight <> AValue then begin
    FSpaceHeight := AValue;
    SysUpdate;
  end;
end;
procedure TIB_SearchPanel.SetTopMargin( AValue: integer );
begin
  if TopMargin <> AValue then begin
    FTopMargin := AValue;
    SysUpdate;
  end;
end;
procedure TIB_SearchPanel.SetLeftMargin( AValue: integer );
begin
  if LeftMargin <> AValue then begin
    FLeftMargin := AValue;
    SysUpdate;
  end;
end;

procedure TIB_SearchPanel.SetPreventEdit( AValue: TIB_StringList );
begin
  FPreventEditing.Text := AValue.Text;
end;

procedure TIB_SearchPanel.SetPreventInsert( AValue: TIB_StringList );
begin
  FPreventInserting.Text := AValue.Text;
end;

procedure TIB_SearchPanel.SetPreventSearch( AValue: TIB_StringList );
begin
  FPreventSearching.Text := AValue.Text;
end;

procedure TIB_SearchPanel.PreventStringsChanged( Sender: TObject );
begin
  if FIB_DataLink.Prepared then SysUpdate;
end;

end.

