{~~ This unit contains the IB_StatementGrid component.}
unit IB_StatementGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids,

  IB_Header,
  IB_Components,
  IB_StatementSource;

type
{~~...
This component serves as a row viewer and modifier.

It also can display extended attributes of a row's Fields.}
TIB_StatementGrid = class( TStringGrid )
  private
    FIB_StatementSource: TIB_StatementSource;
    FIB_RowType: TIB_RowType;
    FFieldInfo: boolean;
    FDomainInfo: boolean;
    procedure SetStatement( AValue: TIB_Statement ); virtual;
    function GetStatement: TIB_Statement; virtual;
    procedure SetIB_RowType( AValue: TIB_RowType ); virtual;
    procedure SetFieldInfo( AValue: boolean ); virtual;
    procedure SetDomainInfo( AValue: boolean ); virtual;
  protected
    procedure Loaded; override;
    procedure IB_StatementUpdate( Sender: TIB_StatementLink;
                                  Statement: TIB_Statement );
    procedure IB_FieldChanged( Sender: TIB_StatementLink;
                               Statement: TIB_Statement;
                               Field: TIB_Column );
    procedure IB_FieldUpdate( Sender: TIB_StatementLink;
                              Statement: TIB_Statement;
                              Field: TIB_Column );
  public
    constructor Create( AOwner: TComponent ); override;
{~~$<!>}
    procedure UpdateGrid;
{~~$<!>}
    procedure CloseGrid;
{~~$<!>}
    procedure PostChanges;
  published

{~~$<*>...
If true then Field attributes are included in additional Fields in the grid.}
    property FieldInfo: boolean read FFieldInfo
                                write SetFieldInfo
                                default false;
                                 
{~~$<*>...
If true this will include the name of the domain along with the Field type.
FieldInfo must also be true for this property to have any effect.}
    property DomainInfo: boolean read FDomainInfo
                                 write SetDomainInfo
                                 default false;
{~~$<*>...
The statement for which this grid is to display or modify row data.}
    property Statement: TIB_Statement read GetStatement
                                      write SetStatement;
{~~$<*>...
Each statement has two rows. One for Param and one for Field.

This property determines which row the grid will display or modify.}
    property IB_RowType: TIB_RowType read FIB_RowType
                                     write SetIB_RowType;
  end;

implementation

{------------------------------------------------------------------------------}

constructor TIB_StatementGrid.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FIB_StatementSource := TIB_StatementSource.Create( Self );
  with FIB_StatementSource as TIB_StatementSource do begin
    AfterAssignment := IB_StatementUpdate;
    AfterExecute := IB_StatementUpdate;
    OnPreparedChanged := IB_StatementUpdate;
    OnParamsDataChange := IB_FieldChanged;
    OnParamsUpdateData := IB_FieldUpdate;
    OnFieldsDataChange := IB_FieldChanged;
    OnFieldsUpdateData := IB_FieldUpdate;
  end;
  CloseGrid;
end;

procedure TIB_StatementGrid.Loaded;
begin
  inherited Loaded;
  UpdateGrid;
end;

{------------------------------------------------------------------------------}

procedure TIB_StatementGrid.SetIB_RowType( AValue: TIB_RowType );
begin
  if FIB_RowType <> AValue then begin
    FIB_RowType := AValue;
    UpdateGrid;
  end;
end;

procedure TIB_StatementGrid.SetStatement( AValue: TIB_Statement);
begin
  if FIB_StatementSource <> nil then begin
    if FIB_StatementSource.Statement <> AValue then begin
      FIB_StatementSource.Statement := AValue;
      UpdateGrid;
    end;
  end;
end;

function TIB_StatementGrid.GetStatement: TIB_Statement;
begin
  if FIB_StatementSource <> nil then begin
    Result := FIB_StatementSource.Statement;
  end else begin
    Result := nil;
  end;
end;

procedure TIB_StatementGrid.SetFieldInfo( AValue: boolean );
begin
  if FieldInfo <> AValue then begin
    FFieldInfo := AValue;
    UpdateGrid;
  end;
end;

procedure TIB_StatementGrid.SetDomainInfo( AValue: boolean );
begin
  if DomainInfo <> AValue then begin
    FDomainInfo := AValue;
    UpdateGrid;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_StatementGrid.IB_StatementUpdate( Sender: TIB_StatementLink;
                                                Statement: TIB_Statement);
begin
  UpdateGrid;
end;

procedure TIB_StatementGrid.IB_FieldChanged( Sender: TIB_StatementLink;
                                             Statement: TIB_Statement;
                                             Field: TIB_Column );
begin
  UpdateGrid;
end;

procedure TIB_StatementGrid.IB_FieldUpdate( Sender: TIB_StatementLink;
                                            Statement: TIB_Statement;
                                            Field: TIB_Column );
begin
  PostChanges;
end;

{------------------------------------------------------------------------------}

procedure TIB_StatementGrid.CloseGrid;
var
  ii: integer;
begin
  Options := Options - [goEditing];
  if FieldInfo then begin
    ColCount := 4;
    FixedCols := 3;
  end else begin
    ColCount := 3;
    FixedCols := 2;
  end;
  for ii := 0 to RowCount - 1 do begin
    Cells[ 0, ii ] := '';
    Cells[ 1, ii ] := '';
    Cells[ 2, ii ] := '';
    if FieldInfo then begin
      Cells[ 3, ii ] := '';
    end;
  end;
  FixedRows := 0;
  RowCount  := 1;
end;

procedure TIB_StatementGrid.UpdateGrid;
var
  ii: integer;
  tmpRow: TIB_Row;
  ss: string;
begin
  if not ( csLoading    in ComponentState ) and
     not ( csDestroying in ComponentState ) then begin
    if FIB_StatementSource.Prepared then begin
      with Statement do begin
        tmpRow := nil;
        case IB_RowType of
          rtParam: begin
            tmpRow := Params;
            Options := Options + [goEditing];
          end;
          rtField: begin
            tmpRow := Fields;
            Options := Options - [goEditing];
          end;
        end;
        if ( tmpRow <> nil ) and ( tmpRow.ColumnCount > 0 ) then begin
          RowCount := tmpRow.ColumnCount;
          for ii := 0 to tmpRow.ColumnCount - 1 do begin
            with tmpRow[ ii ] do begin
              if IB_RowType = rtField then begin
                if ( RelName <> '' ) and ( Fields.RelationCount > 1 ) then begin
                  Cells[ 0, ii ] := Relname + '.' + SQLName; 
                end else begin
                  Cells[ 0, ii ] := FieldName;
                end;
              end else begin
                Cells[ 0, ii ] := FieldName;
              end;
              ss := Copy( tmpRow[ii].ClassName, 11, 256 );
              case SQLType of
                SQL_TEXT,
                SQL_TEXT_,
                SQL_VARYING,
                SQL_VARYING_: ss := ss + ' ( ' + IntToStr( SQLLen ) + ' )';
                else
                if SQLScale <> 0 then begin
                  ss := ss + ' ( '  + IntToStr( SQLLen   ) + 'b, ' +
                                      IntToStr(-SQLScale ) + ' )';
                end;
              end;
              if ( DomainInfo ) then begin
                if ( DomainName <> '' ) and
                   ( Copy( DomainName, 1, 4 ) <> 'RDB$' ) then begin
                  ss := Trim( DomainName ) + ' - ' + ss;
                end;
              end;
              Cells[ 1, ii ] := ss ;
              if FieldInfo then begin
                ss := '';
                if IsPrimary then begin
                  ss := ss + 'PK ';
                end;
                if IsForeign then begin
                  ss := ss + 'FK ';
                end;
                if IsAlternate then begin
                  ss := ss + 'AK ';
                end;
                if not IsPrimary and
                   not IsForeign and
                   not IsAlternate and
                       IsIndexField then begin
                  ss := ss + 'IN ';
                end;
                if IsNullable then begin
                  ss := ss + 'NULL ';
                end;
                if IsDefaulted then begin
                  ss := ss + 'DF ';
                end;
                Cells[ 2, ii ] := ss;
              end;
              Cells[ FixedCols, ii ] := tmpRow[ ii ].AsString;
            end;
          end;
        end else begin
          CloseGrid;
        end;
      end;
    end else begin
      CloseGrid;
    end;
  end;
  Invalidate;
  HideEditor;
end;

procedure TIB_StatementGrid.PostChanges;
var
  ii: integer;
begin
  if ( Statement <> nil ) and
     ( IB_RowType = rtParam ) then begin
    FIB_StatementSource.OnParamsDataChange := nil;
    try
      if Statement <> nil then with Statement do begin
        for ii := 0 to ParamCount - 1 do begin
          Params[ ii ].AsString := Cells[ FixedCols, ii ];
        end;
      end;
    finally
      FIB_StatementSource.OnParamsDataChange := IB_FieldChanged;
    end;
  end;
end;

end.
