
{                                                                              }
{ TIB_StatementSource                                                          }
{                                                                              }

unit
  IB_StatementSource;

interface

uses
  Classes, IB_Components;

type
  TIB_StatementSource = class( TIB_StatementLink )
  private
  { Event Storage Fields }
    FBeforeAssignment: TIB_StmtLinkEvent;
    FAfterAssignment: TIB_StmtLinkEvent;
    FBeforeAllocate: TIB_StmtLinkEvent;
    FAfterAllocate: TIB_StmtLinkEvent;
    FOnAllocatedChanged: TIB_StmtLinkEvent;
    FBeforePrepare: TIB_StmtLinkEvent;
    FAfterPrepare: TIB_StmtLinkEvent;
    FBeforeUnprepare: TIB_StmtLinkEvent;
    FAfterUnprepare: TIB_StmtLinkEvent;
    FOnPreparedChanged: TIB_StmtLinkEvent;
    FBeforeExecute: TIB_StmtLinkEvent;
    FAfterExecute: TIB_StmtLinkEvent;
    FBeforeExecDDL: TIB_StmtLinkEvent;
    FAfterExecDDL: TIB_StmtLinkEvent;
    FBeforeExecDML: TIB_StmtLinkEvent;
    FAfterExecDML: TIB_StmtLinkEvent;
    FOnActiveChanged: TIB_StmtLinkEvent;
    FOnLayoutChanged: TIB_StmtLinkEvent;
    FOnParamsStateChanged: TIB_StmtLinkEvent;
    FOnFieldsStateChanged: TIB_StmtLinkEvent;
    FOnParamsDataChange: TIB_ColLinkEvent;
    FOnParamsUpdateData: TIB_ColLinkEvent;
    FOnFieldsDataChange: TIB_ColLinkEvent;
    FOnFieldsUpdateData: TIB_ColLinkEvent;
  protected
  { Methods }
    procedure ProcessEvent( AEvent: TIB_StatementEventType;
                            Info: longint ); override;
  published
  { Inherited Properties }
    property Statement;
  { Events }
    property BeforeAssignment: TIB_StmtLinkEvent read FBeforeAssignment
                                                 write FBeforeAssignment;
    property AfterAssignment: TIB_StmtLinkEvent read FAfterAssignment
                                                write FAfterAssignment;
    property BeforeAllocate: TIB_StmtLinkEvent read FBeforeAllocate
                                               write FBeforeAllocate;
    property AfterAllocate: TIB_StmtLinkEvent read FAfterAllocate
                                              write FAfterAllocate;
    property BeforePrepare: TIB_StmtLinkEvent read FBeforePrepare
                                              write FBeforePrepare;
    property AfterPrepare: TIB_StmtLinkEvent read FAfterPrepare
                                             write FAfterPrepare;
    property BeforeUnprepare: TIB_StmtLinkEvent read FBeforeUnprepare
                                                write FBeforeUnprepare;
    property AfterUnprepare: TIB_StmtLinkEvent read FAfterUnprepare
                                               write FAfterUnprepare;
    property BeforeExecute: TIB_StmtLinkEvent read FBeforeExecute
                                              write FBeforeExecute;
    property AfterExecute: TIB_StmtLinkEvent read FAfterExecute
                                             write FAfterExecute;
    property BeforeExecDDL: TIB_StmtLinkEvent read FBeforeExecDDL
                                              write FBeforeExecDDL;
    property AfterExecDDL: TIB_StmtLinkEvent read FAfterExecDDL
                                             write FAfterExecDDL;
    property BeforeExecDML: TIB_StmtLinkEvent read FBeforeExecDML
                                              write FBeforeExecDML;
    property AfterExecDML: TIB_StmtLinkEvent read FAfterExecDML
                                             write FAfterExecDML;
    property OnAllocatedChanged: TIB_StmtLinkEvent read FOnAllocatedChanged
                                                   write FOnAllocatedChanged;
    property OnPreparedChanged: TIB_StmtLinkEvent read FOnPreparedChanged
                                                  write FOnPreparedChanged;
    property OnActiveChanged: TIB_StmtLinkEvent read FOnActiveChanged
                                                write FOnActiveChanged;
    property OnLayoutChanged: TIB_StmtLinkEvent read FOnLayoutChanged
                                                write FOnLayoutChanged;
    property OnParamsDataChange: TIB_ColLinkEvent read FOnParamsDataChange
                                                   write FOnParamsDataChange;
    property OnParamsUpdateData: TIB_ColLinkEvent read FOnParamsUpdateData
                                                  write FOnParamsUpdateData;
    property OnParamsStateChanged: TIB_StmtLinkEvent read FOnParamsStateChanged
                                                    write FOnParamsStateChanged;
    property OnFieldsDataChange: TIB_ColLinkEvent read FOnFieldsDataChange
                                                  write FOnFieldsDataChange;
    property OnFieldsUpdateData: TIB_ColLinkEvent read FOnFieldsUpdateData
                                                  write FOnFieldsUpdateData;
    property OnFieldsStateChanged: TIB_StmtLinkEvent read FOnFieldsStateChanged
                                                    write FOnFieldsStateChanged;
  end; { TIB_StatementSource }

implementation

procedure TIB_StatementSource.ProcessEvent( AEvent: TIB_StatementEventType;
                                            Info: longint );
begin
  if not ( csDestroying in ComponentState ) then case AEvent of
  setBeforeAssignment:
    if Assigned( BeforeAssignment ) then BeforeAssignment( Self, Statement );
  setAfterAssignment: 
    if Assigned( AfterAssignment ) then AfterAssignment( Self, Statement );
  setBeforeAllocate:
    if Assigned( BeforeAllocate ) then BeforeAllocate( Self, Statement );
  setAfterAllocate:
    if Assigned( AfterAllocate ) then AfterAllocate( Self, Statement );
  setBeforePrepare:
    if Assigned( BeforePrepare ) then BeforePrepare( Self, Statement );
  setAfterPrepare: 
    if Assigned( AfterPrepare ) then AfterPrepare( Self, Statement );
  setBeforeUnprepare: 
    if Assigned( BeforeUnprepare ) then BeforeUnprepare( Self, Statement );
  setAfterUnprepare: 
    if Assigned( AfterUnprepare ) then AfterUnprepare( Self, Statement );
  setBeforeExecute: 
    if Assigned( BeforeExecute ) then BeforeExecute( Self, Statement );
  setAfterExecute: 
    if Assigned( AfterExecute ) then AfterExecute( Self, Statement );
  setBeforeExecDDL: 
    if Assigned( BeforeExecDDL ) then BeforeExecDDL( Self, Statement );
  setAfterExecDDL: 
    if Assigned( AfterExecDDL ) then AfterExecDDL( Self, Statement );
  setBeforeExecDML: 
    if Assigned( BeforeExecDML ) then BeforeExecDML( Self, Statement );
  setAfterExecDML: 
    if Assigned( AfterExecDML ) then AfterExecDML( Self, Statement );
  setPreparedChanged: 
    if Assigned( OnPreparedChanged ) then begin
      OnPreparedChanged( Self, Statement );
    end;
  setActiveChanged: 
    if Assigned( OnActiveChanged ) then OnActiveChanged( Self, Statement );
  setLayoutChanged: 
    if Assigned( OnLayoutChanged ) then OnLayoutChanged( Self, Statement );
  setParamsStateChanged: 
    if Assigned( OnParamsStateChanged ) then begin
      OnParamsStateChanged( Self, Statement );
    end;
  setFieldsStateChanged: 
    if Assigned( OnFieldsStateChanged ) then begin
      OnFieldsStateChanged( Self, Statement );
    end;
  setParamsDataChange: 
    if Assigned( FOnParamsDataChange ) then begin
      FOnParamsDataChange( Self, Statement, TIB_Column( Info ));
    end;
  setParamsUpdateData: 
    if Assigned( FOnParamsUpdateData ) then begin
      FOnParamsUpdateData( Self, Statement, TIB_Column( Info ));
    end;
  setFieldsDataChange: 
    if Assigned( FOnFieldsDataChange ) then begin
      FOnFieldsDataChange( Self, Statement, TIB_Column( Info ));
    end;
  setFieldsUpdateData: 
    if Assigned( FOnFieldsUpdateData ) then begin
      FOnFieldsUpdateData( Self, Statement, TIB_Column( Info ));
    end;
  end;
end;

end.
