// INT_IBC_CHECKBOX

{ TIB_CheckBox }

constructor TIB_CheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FPaintControl := TPaintControl.Create(Self, 'BUTTON');
  FPaintControl.Ctl3DButton := True;
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  FIB_FieldDataLink.Control := Self;
  FIB_FieldDataLink.OnStateChanged := StateChanged;
  FIB_FieldDataLink.OnDataChange := DataChange;
  FIB_FieldDataLink.OnUpdateData := Self.UpdateData;
  FValueChecked := 'T';
  FValueUncheck := 'F';
  FBorderStyle := bsSingle;
  State := cbUnchecked;
end;

destructor TIB_CheckBox.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnStateChanged := nil;
    OnDataChange  := nil;
    OnUpdateData   := nil;
  end;
  Destroying;
  FPaintControl.Free;
  FPaintControl := nil;
  inherited Destroy;
end;

procedure TIB_CheckBox.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
  SysDataChange;
end;

{------------------------------------------------------------------------------}

function TIB_CheckBox.GetOnPrepareSQL: TIB_DataLinkEvent;
begin
  Result := FIB_FieldDataLink.OnPrepareSQL;
end;

procedure TIB_CheckBox.SetOnPrepareSQL( AValue: TIB_DataLinkEvent );
begin
  FIB_FieldDataLink.OnPrepareSQL := AValue;
end;

{------------------------------------------------------------------------------}

procedure TIB_CheckBox.StateChanged( Sender: TIB_DataLink;
                                     IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_CheckBox.DataChange( Sender: TIB_DataLink;
                                   IB_DataSource: TIB_DataSource;
                                   IB_Field: TIB_Column );
begin
  SysDataChange;
end;

procedure TIB_CheckBox.UpdateData( Sender: TIB_DataLink;
                                   IB_DataSource: TIB_DataSource;
                                   IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}

function TIB_CheckBox.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;

function TIB_CheckBox.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;

function TIB_CheckBox.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;

function TIB_CheckBox.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;

function TIB_CheckBox.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;

function TIB_CheckBox.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;

function TIB_CheckBox.GetPreventSearching: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsSearching;
end;

{------------------------------------------------------------------------------}

procedure TIB_CheckBox.SetDataSource(AValue: TIB_DataSource);
begin
  FIB_FieldDataLink.DataSource := AValue;
end;

procedure TIB_CheckBox.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;

procedure TIB_CheckBox.SetReadOnly( Value: Boolean );
begin
  if ReadOnly <> Value then begin
    FIB_FieldDataLink.ControlIsReadOnly := Value;
    SysStateChanged;
  end;
end;

procedure TIB_CheckBox.SetPreventEditing( Value: Boolean );
begin
  if PreventEditing <> Value then begin
    FIB_FieldDataLink.ControlPreventsEditing := Value;
    SysStateChanged;
  end;
end;

procedure TIB_CheckBox.SetPreventInserting( Value: Boolean );
begin
  if PreventInserting <> Value then begin
    FIB_FieldDataLink.ControlPreventsInserting := Value;
    SysStateChanged;
  end;
end;

procedure TIB_CheckBox.SetPreventSearching( Value: Boolean );
begin
  if PreventSearching <> Value then begin
    FIB_FieldDataLink.ControlPreventsSearching := Value;
    SysStateChanged;
  end;
end;

function TIB_CheckBox.GetSearchBuffer: string;
begin
  if FIB_FieldDataLink <> nil then begin
    Result := FIB_FieldDataLink.SearchBuffer;
  end else begin
    Result := '';
  end;
end;

procedure TIB_CheckBox.SetSearchBuffer( AValue: string );
begin
  if FIB_FieldDataLink <> nil then begin
    FIB_FieldDataLink.SearchBuffer := AValue;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_CheckBox.SysStateChanged;
begin
  with FIB_FieldDataLink do begin
    if ColorScheme then inherited Color := Color;
  end;
end;

procedure TIB_CheckBox.SysDataChange;
begin
  State := GetFieldState;
end;

procedure TIB_CheckBox.SysUpdateData;
begin
  with FIB_FieldDataLink do begin
    if Self.State = cbGrayed then begin
      FieldText := '';
    end else if Checked then begin
      FieldText := FValueChecked
    end else begin
      FieldText := FValueUncheck;
    end;
  end;
end;

{------------------------------------------------------------------------------}

function TIB_CheckBox.GetFieldState: TCheckBoxState;
begin
  with FIB_FieldDataLink do begin
    if Field = nil then begin
      Result := cbUnchecked;
    end else if FValueChecked = FieldText then begin
      Result := cbChecked;
    end else if FValueUncheck = FieldText then begin
      Result := cbUnchecked;
    end else begin
      Result := cbGrayed;
    end;
  end;
end;

procedure TIB_CheckBox.Toggle;
begin
  if FIB_FieldDataLink.Modify then begin
    inherited Toggle;
    FIB_FieldDataLink.ControlIsModified := true;
  end;
end;

procedure TIB_CheckBox.SetValueCheck( const Value: string );
begin
  if FValueChecked <> Value then begin
    FValueChecked := Value;
    SysDataChange;
  end;
end;

procedure TIB_CheckBox.SetValueUncheck( const Value: string );
begin
  if FValueUncheck <> Value then begin
    FValueUncheck := Value;
    SysDataChange;
  end;
end;

procedure TIB_CheckBox.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  case Key of
    #8, ' ':
      if FIB_FieldDataLink.Modify then begin
        FIB_FieldDataLink.ControlIsModified := true;
      end;
    #27:
      SysDataChange;
  end;
end;

procedure TIB_CheckBox.WndProc(var Message: TMessage);
begin
  if csDestroying in ComponentState then Exit;
  with Message do
    if ( Msg = WM_CREATE           ) or
       ( Msg = WM_WINDOWPOSCHANGED ) or
       ( Msg = CM_TEXTCHANGED      ) or
       ( Msg = CM_FONTCHANGED      ) then FPaintControl.DestroyHandle;
  inherited;
end;

procedure TIB_CheckBox.WMPaint(var Message: TWMPaint);
begin
  if not (csPaintCopy in ControlState) then inherited else begin
    SendMessage(FPaintControl.Handle, BM_SETCHECK, Ord(GetFieldState), 0);
    SendMessage(FPaintControl.Handle, WM_PAINT, Message.DC, 0);
  end;
end;

procedure TIB_CheckBox.CMEnter(var Message: TCMEnter);
begin
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_CheckBox.CMExit(var Message: TCMExit);
begin
  try
    FIB_FieldDataLink.UpdateData;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

{------------------------------------------------------------------------------}

procedure TIB_CheckBox.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of Longint = (0, WS_BORDER);
begin
  inherited CreateParams(Params);
  with Params do begin
    Style := Style or BorderStyles[FBorderStyle];
    if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    end;
  end;
end;

procedure TIB_CheckBox.SetBorderStyle(Value: TBorderStyle);
begin
  if FBorderStyle <> Value then begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

