// INT_IBC_Edit

constructor TIB_Edit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  ControlStyle := ControlStyle + [csReplicatable];
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  with FIB_FieldDataLink do begin
    Control := Self;
    OnStateChanged := IB_StateChanged;
    OnModifyingChanged := IB_ModifyingChanged;
    OnDataChange := IB_DataChange;
    OnUpdateData := IB_UpdateData;
  end;
end;

destructor TIB_Edit.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnStateChanged := nil;
    OnModifyingChanged := nil;
    OnDataChange := nil;
    OnUpdateData := nil;
  end;
  Destroying;
  if FCanvas <> nil then begin
    FCanvas.Free;
    FCanvas := nil;
  end;
  inherited Destroy;
end;

procedure TIB_Edit.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
  SysDataChange;
end;

{------------------------------------------------------------------------------}
{  DataLink event handlers.                                                    } 
{------------------------------------------------------------------------------}

procedure TIB_Edit.IB_ModifyingChanged( Sender: TIB_DataLink;
                                        IB_DataSource: TIB_DataSource );
begin
  inherited ReadOnly := FIB_FieldDataLink.ReadOnly or
                        not ( FIB_FieldDataLink.State in [ dsInsert,
                                                           dsEdit,
                                                           dsSearch ]);
end;

procedure TIB_Edit.IB_StateChanged( Sender: TIB_DataLink;
                                    IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_Edit.IB_DataChange( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  SysDataChange;
end;

procedure TIB_Edit.IB_UpdateData( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}
{ Implementation of data-aware logic.                                          }
{------------------------------------------------------------------------------}

procedure TIB_Edit.SysDataChange;
begin
  if FAlignment <> FIB_FieldDataLink.FieldAlignment then begin
    EditText := '';  {forces update}
    FAlignment := FIB_FieldDataLink.FieldAlignment;
  end;
  EditMask := FIB_FieldDataLink.FieldEditMask;
  MaxLength := FIB_FieldDataLink.FieldMaxLength;
  if EditMask = '' then begin
    Text := FIB_FieldDataLink.FieldText;
  end else begin
    EditText := FIB_FieldDataLink.FieldText;
  end;
  Modified := false;
  Invalidate;
end;

procedure TIB_Edit.SysStateChanged;
begin
  if FIB_FieldDataLink.ColorScheme then begin
    inherited Color := FIB_FieldDataLink.Color;
  end;
end;

procedure TIB_Edit.SysUpdateData;
begin
  if Field <> nil then begin
    if Modified then begin
      ValidateEdit;
      FIB_FieldDataLink.FieldText := Text;
    end;
  end;
end;

{------------------------------------------------------------------------------}
{ DataLink property coordination methods.                                      } 
{------------------------------------------------------------------------------}

function TIB_Edit.GetOnPrepareSQL: TIB_DataLinkEvent;
begin
  Result := FIB_FieldDataLink.OnPrepareSQL;
end;
procedure TIB_Edit.SetOnPrepareSQL( AValue: TIB_DataLinkEvent);
begin
  FIB_FieldDataLink.OnPrepareSQL := AValue;
end;
function TIB_Edit.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;
procedure TIB_Edit.SetDataSource(AValue: TIB_DataSource);
begin
  FIB_FieldDataLink.DataSource := AValue;
end;
function TIB_Edit.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;
procedure TIB_Edit.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;
function TIB_Edit.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;
function TIB_Edit.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;
procedure TIB_Edit.SetReadOnly( Value: Boolean );
begin
  FIB_FieldDataLink.ControlIsReadOnly := Value;
end;
function TIB_Edit.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;
procedure TIB_Edit.SetPreventEditing( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsEditing := Value;
end;
function TIB_Edit.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;
procedure TIB_Edit.SetPreventInserting( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsInserting := Value;
end;
function TIB_Edit.GetPreventSearching: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsSearching;
end;
procedure TIB_Edit.SetPreventSearching( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsSearching := Value;
end;
function TIB_Edit.GetSearchBuffer: string;
begin
  Result := FIB_FieldDataLink.SearchBuffer;
end;
procedure TIB_Edit.SetSearchBuffer( AValue: string );
begin
  FIB_FieldDataLink.SearchBuffer := AValue;
end;

{------------------------------------------------------------------------------}
{  Control specific processing                                                 } 
{------------------------------------------------------------------------------}

function TIB_Edit.EditCanModify: Boolean;
begin
  Result := FIB_FieldDataLink.Modify;
end;

procedure TIB_Edit.Reset;
begin
  SysDataChange;
  SelectAll;
end;

procedure TIB_Edit.SetFocused( Value: Boolean );
begin
  if FFocused <> Value then begin
    FFocused := Value;
    if (FAlignment <> taLeftJustify) and not IsMasked then begin
      FIB_FieldDataLink.Reset;
    end;
  end;
end;

procedure TIB_Edit.Change;
begin
  with FIB_FieldDataLink do begin
    if NeedToPost then begin
      ControlIsModified := true;
    end;
  end;
  inherited Change;
end;

procedure TIB_Edit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then begin
    FIB_FieldDataLink.Modify;
  end;
end;

procedure TIB_Edit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and
     (FIB_FieldDataLink.Field <> nil) and
    not FIB_FieldDataLink.FieldIsValidChar(Key) then
  begin
    MessageBeep(0);
    Key := #0;
  end;
  case Key of
    ^H, ^V, ^X, #32..#255: FIB_FieldDataLink.Modify;
    #27:
    begin
      FIB_FieldDataLink.Reset;
      SelectAll;
      Key := #0;
    end;
  end;
end;

procedure TIB_Edit.WMPaste(var Message: TMessage);
begin
  if FIB_FieldDataLink.Modify then inherited;
end;

procedure TIB_Edit.WMCut(var Message: TMessage);
begin
  if FIB_FieldDataLink.Modify then inherited;
end;

procedure TIB_Edit.CMEnter(var Message: TCMEnter);
begin
  SetFocused( true );
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_Edit.CMExit(var Message: TCMExit);
begin
  try
    FIB_FieldDataLink.UpdateData;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused( false );
  CheckCursor;
  DoExit;
end;

{------------------------------------------------------------------------------}

procedure TIB_Edit.WMPaint(var Message: TWMPaint);
var
  Left: Integer;
  Margins: TPoint;
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
  S: string;
begin
  if ((FAlignment = taLeftJustify) or FFocused) and
    not (csPaintCopy in ControlState) then begin
    inherited;
    Exit;
  end;
  if FCanvas = nil then begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then begin
    DC := BeginPaint( Handle, PS );
  end;
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do begin
      R := ClientRect;
      if not ( NewStyleControls and Ctl3D ) and
             ( BorderStyle = bsSingle ) then begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        InflateRect(R, -1, -1);
      end;
      Brush.Color := Color;
      if (csPaintCopy in ControlState) and
         (FIB_FieldDataLink.Field <> nil) then begin
        S := FIB_FieldDataLink.Field.AsString;
        case CharCase of
          ecUpperCase: S := AnsiUpperCase(S);
          ecLowerCase: S := AnsiLowerCase(S);
        end;
      end else begin
        S := EditText;
      end;
      if PasswordChar <> #0 then begin
        FillChar(S[1], Length(S), PasswordChar);
      end;
      Margins := GetTextMargins;
      case FAlignment of
        taLeftJustify: Left := Margins.X;
        taRightJustify: Left := ClientWidth - TextWidth(S) - Margins.X - 1;
      else
        Left := (ClientWidth - TextWidth(S)) div 2;
      end;
      TextRect( R, Left, Margins.Y, S );
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0 then begin
      EndPaint( Handle, PS );
    end;
  end;
end;

function TIB_Edit.GetTextMargins: TPoint;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  if NewStyleControls then begin
    if BorderStyle = bsNone then begin
      I := 0
    end else if Ctl3D then begin
      I := 1
    end else begin
      I := 2;
    end;
    Result.X := SendMessage(Handle, EM_GETMARGINS, 0, 0) and $0000FFFF + I;
    Result.Y := I;
  end else begin
    if BorderStyle = bsNone then begin
      I := 0;
    end else begin
      DC := GetDC(0);
      GetTextMetrics(DC, SysMetrics);
      SaveFont := SelectObject(DC, Font.Handle);
      GetTextMetrics(DC, Metrics);
      SelectObject(DC, SaveFont);
      ReleaseDC(0, DC);
      I := SysMetrics.tmHeight;
      if I > Metrics.tmHeight then begin
        I := Metrics.tmHeight;
      end;
      I := I div 4;
    end;
    Result.X := I;
    Result.Y := I;
  end;
end;

