// INT_IBC_Image

constructor TIB_Image.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csFramed, csOpaque];
  Width := 105;
  Height := 105;
  TabStop := true;
  ParentColor := false;
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FBorderStyle := bsSingle;
  FAutoDisplay := true;
  FCenter := true;
  FQuickDraw := true;
  FKeepScale := true;
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  with FIB_FieldDataLink do begin
    Control := Self;
    OnStateChanged := IB_StateChanged;
    OnDataChange := IB_DataChange;
    OnUpdateData := IB_UpdateData;
  end;
end;

destructor TIB_Image.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnStateChanged := nil;
    OnDataChange := nil;
    OnUpdateData := nil;
  end;
  Destroying;
  FPicture.Free;
  FPicture := nil;
  inherited Destroy;
end;

procedure TIB_Image.Loaded;
begin
  inherited Loaded;
  SysDataChange;
end;

{------------------------------------------------------------------------------}

procedure TIB_Image.IB_StateChanged( Sender: TIB_DataLink;
                                     DataSource: TIB_DataSource );
begin
  with FIB_FieldDataLink do begin
    if ColorScheme then Self.Color := Color;
  end;
end;

procedure TIB_Image.IB_DataChange( Sender: TIB_DataLink;
                                   DataSource: TIB_DataSource;
                                   Field: TIB_Column );
begin
  if not FIsUpdating and not FIgnorePictureChange then begin
    SysDataChange;
  end;
end;

procedure TIB_Image.IB_UpdateData( Sender: TIB_DataLink;
                                   DataSource: TIB_DataSource;
                                   Field: TIB_Column );
begin
  if FIB_FieldDataLink.State = dsSearch then Exit;
  if Field is TIB_ColumnBlob then begin
    with Field as TIB_ColumnBlob do begin
      FIsUpdating := true;
      try
        if ( Picture.Graphic <> nil ) and
           ( Picture.Graphic is TBitmap ) and
           ( not Picture.Graphic.Empty ) then begin
          Assign( Picture.Graphic );
        end else begin
          Clear;
        end;
        if NeedToStore then begin
          Store;
        end;
        FBlobIDLoaded := Blob_ID; 
      finally
        FPictureLoaded := true;
        FIsUpdating := false;
      end;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_Image.SysDataChange;
begin
  if ( Field <> nil ) and
     ( Field is TIB_ColumnBlob ) then begin
    with Field as TIB_ColumnBlob do begin
      if IsNull or
         (( not isc_quad_equal( Blob_ID, FBlobIDLoaded ) or
                isc_quad_is_zero ( Blob_ID )) and
          ( not isc_quad_equal( Temp_ID, FBlobIDLoaded ) or
                isc_quad_is_zero ( Temp_ID ))) then begin
        FIgnorePictureChange := true;
        try
          Picture.Graphic := nil;
        finally
          FIgnorePictureChange := false;
        end;
        FPictureLoaded := false;
        if FAutoDisplay then begin
          LoadPicture;
        end;
      end;
      if IsLoaded then begin
        LoadPicture;
      end;
    end;
  end;
end;

procedure TIB_Image.LoadPicture;
var
  ABitmap: TBitmap;
begin
  if not FPictureLoaded then begin
    FIgnorePictureChange := true;
    try
      if ( Field <> nil ) and
         ( not Field.IsNull ) and
         ( Field.IsBlob ) then begin
        with Field as TIB_ColumnBlob do begin
          ABitmap := Picture.Bitmap;
          AssignTo( ABitmap );
          FBlobIDLoaded := Blob_ID;
        end;
      end else begin
        Picture.Graphic := nil;
        FBlobIDLoaded.isc_quad_high := 0;
        FBlobIDLoaded.isc_quad_low := 0;
      end;
    finally
      FIgnorePictureChange := false;
    end;
    FPictureLoaded := true;
  end;
end;

procedure TIB_Image.PictureChanged(Sender: TObject);
begin
  if AutoSize then begin
    if (Picture.Width > 0) and (Picture.Height > 0) then begin
      SetBounds(Left, Top, Picture.Width, Picture.Height);
    end else begin
//      SetBounds(Left, Top, Picture.Width, Picture.Height);
    end;
  end;
  if not FIgnorePictureChange then begin
    FIB_FieldDataLink.Modify;
    FIB_FieldDataLink.ControlIsModified := true;
    FPictureLoaded := true;
    FBlobIDLoaded.isc_quad_high := 0;
    FBlobIDLoaded.isc_quad_low := 0;
    if not Focused then begin
      FIB_FieldDataLink.UpdateData;
    end;
  end;
  Invalidate;
end;

{------------------------------------------------------------------------------}

function TIB_Image.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;
function TIB_Image.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;
function TIB_Image.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;
function TIB_Image.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;
function TIB_Image.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;
function TIB_Image.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;
procedure TIB_Image.SetReadOnly( Value: Boolean );
begin
  FIB_FieldDataLink.ControlIsReadOnly := Value;
end;
procedure TIB_Image.SetPreventEditing( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsEditing := Value;
end;
procedure TIB_Image.SetPreventInserting( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsInserting := Value;
end;
function TIB_Image.GetSearchBuffer: string;
begin
  Result := FIB_FieldDataLink.SearchBuffer;
end;
procedure TIB_Image.SetSearchBuffer( AValue: string );
begin
  FIB_FieldDataLink.SearchBuffer := AValue;
end;
procedure TIB_Image.SetDataSource( AValue: TIB_DataSource );
begin
  FIB_FieldDataLink.DataSource := AValue;
end;
procedure TIB_Image.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;

{------------------------------------------------------------------------------}

function TIB_Image.GetPalette: HPALETTE;
begin
  Result := 0;
  if FPicture.Graphic is TBitmap then
    Result := TBitmap(FPicture.Graphic).Palette;
end;

procedure TIB_Image.SetAutoDisplay( Value: Boolean );
begin
  if FAutoDisplay <> Value then begin
    FAutoDisplay := Value;
    if Value then LoadPicture;
  end;
end;

procedure TIB_Image.SetBorderStyle( Value: TBorderStyle );
begin
  if FBorderStyle <> Value then begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TIB_Image.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then begin
    FCenter := Value;
    Invalidate;
  end;
end;

procedure TIB_Image.SetPicture(Value: TPicture);
begin
  FPicture.Assign( Value );
end;

procedure TIB_Image.SetStretch(Value: Boolean);
begin
  if FStretch <> Value then begin
    FStretch := Value;
    Invalidate;
  end;
end;

procedure TIB_Image.SetKeepScale(Value: Boolean);
begin
  if FKeepScale <> Value then begin
    FKeepScale := Value;
    Invalidate;
  end;
end;

procedure TIB_Image.SetAutoSize(Value: Boolean);
begin
  if AutoSize <> Value then begin
    FAutoSize := Value;
    if AutoSize then Align := alNone;
    FIgnorePictureChange := true;
    try
      PictureChanged( Picture );
    finally
      FIgnorePictureChange := false;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_Image.CopyToClipboard;
begin
  if Picture.Graphic <> nil then begin
    Clipboard.Assign(Picture);
  end;
end;

procedure TIB_Image.CutToClipboard;
begin
  if Picture.Graphic <> nil then begin
    if FIB_FieldDataLink.Modify then begin
      CopyToClipboard;
      Picture.Graphic := nil;
    end;
  end;
end;

procedure TIB_Image.PasteFromClipboard;
begin
  if ( Clipboard.HasFormat( CF_BITMAP )) and
     ( FIB_FieldDataLink.Modify ) then begin
    Picture.Bitmap.Assign( Clipboard );
  end;
end;

procedure TIB_Image.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if FBorderStyle = bsSingle then
    Params.Style := Params.Style or WS_BORDER;
end;

procedure TIB_Image.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  case Key of
  VK_INSERT:
    if ssShift in Shift then PasteFromClipBoard else
      if ssCtrl in Shift then CopyToClipBoard;
  VK_DELETE:
    if ssShift in Shift then begin
      CutToClipBoard;
    end else begin
      Picture.Graphic := nil;
    end;
  end;
end;

procedure TIB_Image.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  case Key of
    ^X: CutToClipBoard;
    ^C: CopyToClipBoard;
    ^V: PasteFromClipBoard;
    #13: if not FPictureLoaded then LoadPicture
                               else FIB_FieldDataLink.UpdateData;
    #27: SysDataChange;
  end;
end;

procedure TIB_Image.CMEnter(var Message: TCMEnter);
begin
  Invalidate; { Draw the focus marker }
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_Image.CMExit(var Message: TCMExit);
begin
  FIB_FieldDataLink.UpdateData;
  Invalidate; { Erase the focus marker }
  inherited;
end;

procedure TIB_Image.CMTextChanged(var Message: TMessage);
begin
  inherited;
  if not FPictureLoaded then Invalidate;
end;

procedure TIB_Image.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if TabStop and CanFocus then SetFocus;
  inherited;
end;

procedure TIB_Image.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  LoadPicture;
  inherited;
end;

procedure TIB_Image.WMCut(var Message: TMessage);
begin
  CutToClipboard;
end;

procedure TIB_Image.WMCopy(var Message: TMessage);
begin
  CopyToClipboard;
end;

procedure TIB_Image.WMPaste(var Message: TMessage);
begin
  PasteFromClipboard;
end;

procedure TIB_Image.Paint;
var
  R: TRect;
  S: string;
  DrawPict: TPicture;
  Pal: HPalette;
{$IFNDEF VER100}
  Form: TForm;
{$ELSE}
  Form: TCustomForm;
{$ENDIF}
begin
  with Canvas do begin
    Brush.Style := bsSolid;
    Brush.Color := Color;
    if ( Field <> nil ) and
       ( FPictureLoaded or (csPaintCopy in ControlState)) then begin
      DrawPict := TPicture.Create;
      Pal := 0;
      try
        if (csPaintCopy in ControlState) and
          ( Field <> nil ) and
            Field.IsBlob then begin
          (Field as TIB_ColumnBlob).AssignTo( DrawPict );
          if DrawPict.Graphic is TBitmap then
            DrawPict.Bitmap.IgnorePalette := QuickDraw;
        end else begin
          DrawPict.Assign( Picture );
          if Focused and ( DrawPict.Graphic is TBitmap ) and
            ( DrawPict.Bitmap.Palette <> 0 ) then begin
          { Control has focus, so realize the bitmap palette in foreground }
            Pal := SelectPalette(Handle, DrawPict.Bitmap.Palette, False);
            RealizePalette(Handle);
          end;
        end;
        SetRect( R, 0, 0, DrawPict.Width, DrawPict.Height );
//        if Stretch and KeepScale then begin
//          FillRect(ClientRect);
//          if ( DrawPict.Graphic <> nil ) and
//             not DrawPict.Graphic.Empty then begin
// WIP!!!
//            tmpInt := ( R.Bottom - R.Top ) - DrawPict.Graphic.Height;
//            if tmpInt > ( R.Right - R.Left ) - DrawPict.Width then begin
//            end;
//            InflateRect( R, R.
//            StretchDraw( R, DrawPict.Graphic );
//          end;
{        end else} if Stretch or AutoSize then begin
          if ( DrawPict.Graphic = nil ) or DrawPict.Graphic.Empty then begin
            FillRect(ClientRect);
          end else begin
            StretchDraw( ClientRect, DrawPict.Graphic );
          end;
        end else begin
          if Center then OffsetRect( R,
                                    (ClientWidth  - DrawPict.Width ) div 2,
                                    (ClientHeight - DrawPict.Height) div 2);
          StretchDraw( R, DrawPict.Graphic );
          ExcludeClipRect(Handle, R.Left, R.Top, R.Right, R.Bottom);
          FillRect(ClientRect);
          SelectClipRgn(Handle, 0);
        end;
      finally
        if Pal <> 0 then SelectPalette(Handle, Pal, True);
        DrawPict.Free;
      end;
    end else begin
      Font := Self.Font;
      if ( Field <> nil ) and ( Field.IB_Row.RowState <> rsNone ) then begin
        if Field.IsNull then begin
          S := Field.DisplayLabel + ' <NULL>';
        end else begin
          S := Field.DisplayLabel;
        end;
      end else if csDesigning in ComponentState then begin
        S := Name;
      end else begin
        S := '';
      end;
      if S <> '' then begin
        S := '(' + S + ')';
      end;
      R := ClientRect;
      FillRect( R );
      TextRect( R, (R.Right  - TextWidth (S)) div 2,
                   (R.Bottom - TextHeight(S)) div 2, S );
    end;
    Form := GetParentForm(Self);
    if (Form <> nil) and (Form.ActiveControl = Self) and
      not (csDesigning in ComponentState) and
      not (csPaintCopy in ControlState) then begin
      Brush.Color := clWindowFrame;
      FrameRect(ClientRect);
    end;
  end;
end;


