// INT_IBC_LISTBOX

constructor TIB_ListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  with FIB_FieldDataLink do begin
    Control := Self;
    OnStateChanged := StateChanged;
    OnDataChange := DataChange;
    OnUpdateData := Self.UpdateData;
  end;
end;

destructor TIB_ListBox.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnStateChanged := nil;
    OnDataChange  := nil;
    OnUpdateData   := nil;
  end;
  inherited Destroy;
end;

procedure TIB_ListBox.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
  SysDataChange;
end;

{------------------------------------------------------------------------------}

procedure TIB_ListBox.StateChanged( Sender: TIB_DataLink;
                                    IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_ListBox.DataChange( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  SysDataChange;
end;

procedure TIB_ListBox.UpdateData( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}

function TIB_ListBox.GetOnPrepareSQL: TIB_DataLinkEvent;
begin
  Result := FIB_FieldDataLink.OnPrepareSQL;
end;

procedure TIB_ListBox.SetOnPrepareSQL( AValue: TIB_DataLinkEvent);
begin
  FIB_FieldDataLink.OnPrepareSQL := AValue;
end;

function TIB_ListBox.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;

function TIB_ListBox.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;

function TIB_ListBox.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;

function TIB_ListBox.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;

function TIB_ListBox.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;

function TIB_ListBox.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;

function TIB_ListBox.GetPreventSearching: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsSearching;
end;

{------------------------------------------------------------------------------}

procedure TIB_ListBox.SetReadOnly( Value: Boolean );
begin
  if ReadOnly <> Value then begin
    FIB_FieldDataLink.ControlIsReadOnly := Value;
    SysStateChanged;
  end;
end;

procedure TIB_ListBox.SetPreventEditing( Value: Boolean );
begin
  if PreventEditing <> Value then begin
    FIB_FieldDataLink.ControlPreventsEditing := Value;
    SysStateChanged;
  end;
end;

procedure TIB_ListBox.SetPreventInserting( Value: Boolean );
begin
  if PreventInserting <> Value then begin
    FIB_FieldDataLink.ControlPreventsInserting := Value;
    SysStateChanged;
  end;
end;

procedure TIB_ListBox.SetPreventSearching( Value: Boolean );
begin
  if PreventSearching <> Value then begin
    FIB_FieldDataLink.ControlPreventsSearching := Value;
    SysStateChanged;
  end;
end;

function TIB_ListBox.GetSearchBuffer: string;
begin
  if FIB_FieldDataLink <> nil then begin
    Result := FIB_FieldDataLink.SearchBuffer;
  end else begin
    Result := '';
  end;
end;

procedure TIB_ListBox.SetSearchBuffer( AValue: string );
begin
  if FIB_FieldDataLink <> nil then begin
    FIB_FieldDataLink.SearchBuffer := AValue;
  end;
end;

procedure TIB_ListBox.SetDataSource(AValue: TIB_DataSource);
begin
  FIB_FieldDataLink.DataSource := AValue;
end;

procedure TIB_ListBox.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;

{------------------------------------------------------------------------------}

procedure TIB_ListBox.SysStateChanged;
begin
  with FIB_FieldDataLink do begin
    if ColorScheme then inherited Color := Color;
  end;
end;

procedure TIB_ListBox.SysDataChange;
begin
  if FIB_FieldDataLink.Field <> nil then begin
    ItemIndex := Items.IndexOf(FIB_FieldDataLink.FieldText)
  end else begin
    ItemIndex := -1;
  end;
end;

procedure TIB_ListBox.SysUpdateData;
begin
  if ItemIndex >= 0 then begin
    FIB_FieldDataLink.FieldText := Items[ ItemIndex ]
  end else begin
    FIB_FieldDataLink.FieldText := '';
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_ListBox.Click;
begin
  if FIB_FieldDataLink.Modify then begin
    inherited Click;
    FIB_FieldDataLink.ControlIsModified := true;
  end;
end;

procedure TIB_ListBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if Key in [VK_PRIOR, VK_NEXT, VK_END, VK_HOME, VK_LEFT, VK_UP,
    VK_RIGHT, VK_DOWN] then
    if not FIB_FieldDataLink.Modify then Key := 0;
end;

procedure TIB_ListBox.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  case Key of
    #32..#255:
      if not FIB_FieldDataLink.Modify then Key := #0;
    #27:
      SysDataChange;
  end;
end;

procedure TIB_ListBox.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if FIB_FieldDataLink.Modify then begin
    inherited
  end else begin
    SetFocus;
    with Message do MouseDown(mbLeft, KeysToShiftState(Keys), XPos, YPos);
  end;
end;

procedure TIB_ListBox.CMEnter(var Message: TCMEnter);
begin
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_ListBox.CMExit(var Message: TCMExit);
begin
  try
    SysUpdateData;  
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

procedure TIB_ListBox.SetItems(Value: TStrings);
begin
  Items.Assign(Value);
  SysDataChange;
end;

