// INT_IBC_Memo

{ TIB_Memo }

constructor TIB_Memo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  FAutoDisplay := True;
  FPaintControl := TPaintControl.Create( Self, 'EDIT' );
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  FIB_FieldDataLink.Control := Self;
  FIB_FieldDataLink.OnStateChanged := IB_StateChanged;
  FIB_FieldDataLink.OnModifyingChanged := IB_ModifyingChanged;
  FIB_FieldDataLink.OnDataChange := IB_DataChange;
  FIB_FieldDataLink.OnUpdateData := IB_UpdateData;
end;

destructor TIB_Memo.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnStateChanged := nil;
    OnModifyingChanged := nil;
    OnDataChange := nil;
    OnUpdateData := nil;
  end;
  FPaintControl.Free;
  inherited Destroy;
end;

procedure TIB_Memo.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
  SysDataChange;
end;

{------------------------------------------------------------------------------}
{  DataLink event handlers.                                                    } 
{------------------------------------------------------------------------------}

procedure TIB_Memo.IB_ModifyingChanged( Sender: TIB_DataLink;
                                        IB_DataSource: TIB_DataSource );
begin
  inherited ReadOnly := FIB_FieldDataLink.ReadOnly or
                        not ( FIB_FieldDataLink.State in [ dsInsert,
                                                           dsEdit,
                                                           dsSearch ]);
end;

procedure TIB_Memo.IB_StateChanged( Sender: TIB_DataLink;
                                    IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_Memo.IB_DataChange( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  FIgnoreChange := true;
  try
    SysDataChange;
  finally
    FIgnoreChange := false;
  end;
end;

procedure TIB_Memo.IB_UpdateData( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource;
                                  IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}
{ Implementation of data-aware logic.                                          } 
{------------------------------------------------------------------------------}

procedure TIB_Memo.SysStateChanged;
begin
  if FIB_FieldDataLink.ColorScheme then begin
    inherited Color := FIB_FieldDataLink.Color;
  end;
end;

procedure TIB_Memo.SysDataChange;
begin
  with FIB_FieldDataLink do begin
    MaxLength := FieldMaxLength;
    FMemoLoaded := false;
    if Field <> nil then begin
      case State of
        dsBrowse,
        dsEdit,
        dsInsert,
        dsDelete: begin
          if Field is TIB_ColumnBlob then begin
            if AutoDisplay then begin
              LoadMemo;
            end else begin
              Lines.Text := FieldText;
            end;
          end else begin
            Lines.Text := FieldText;
            FMemoLoaded := True;
          end;
        end else begin
          Lines.Text := FieldText;
          FMemoLoaded := true;
        end;
      end;
    end else begin
      Lines.Text := FieldText;
    end;
  end;
end;

procedure TIB_Memo.SysUpdateData;
begin
  FIB_FieldDataLink.FieldText := Text;
end;

procedure TIB_Memo.LoadMemo;
begin
  if not FMemoLoaded then begin
    try
      if Field is TIB_ColumnBlob then begin
        with Field as TIB_ColumnBlob do begin
          FIgnoreChange := true;
          try
            AsStream.Position := 0;
            Lines.LoadFromStream( AsStream );
          finally
            FIgnoreChange := false;
          end;
        end;
        FMemoLoaded := True;
      end;
    except
      Lines.Text := 'Could not load memo.';
    end;
    SysStateChanged;
  end;
end;

{------------------------------------------------------------------------------}
{ DataLink property coordination methods.                                      } 
{------------------------------------------------------------------------------}

function TIB_Memo.GetOnPrepareSQL: TIB_DataLinkEvent;
begin
  Result := FIB_FieldDataLink.OnPrepareSQL;
end;
procedure TIB_Memo.SetOnPrepareSQL( AValue: TIB_DataLinkEvent);
begin
  FIB_FieldDataLink.OnPrepareSQL := AValue;
end;
function TIB_Memo.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;
procedure TIB_Memo.SetDataSource(AValue: TIB_DataSource);
begin
  FIB_FieldDataLink.DataSource := AValue;
end;
function TIB_Memo.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;
procedure TIB_Memo.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;
function TIB_Memo.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;
function TIB_Memo.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;
procedure TIB_Memo.SetReadOnly( Value: Boolean );
begin
  FIB_FieldDataLink.ControlIsReadOnly := Value;
end;
function TIB_Memo.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;
procedure TIB_Memo.SetPreventEditing( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsEditing := Value;
end;
function TIB_Memo.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;
procedure TIB_Memo.SetPreventInserting( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsInserting := Value;
end;
function TIB_Memo.GetPreventSearching: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsSearching;
end;
procedure TIB_Memo.SetPreventSearching( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsSearching := Value;
end;
function TIB_Memo.GetSearchBuffer: string;
begin
  Result := FIB_FieldDataLink.SearchBuffer;
end;
procedure TIB_Memo.SetSearchBuffer( AValue: string );
begin
  FIB_FieldDataLink.SearchBuffer := AValue;
end;

{------------------------------------------------------------------------------}
{  Control specific processing                                                 } 
{------------------------------------------------------------------------------}

procedure TIB_Memo.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then begin
    FFocused := Value;
    if not ( FIB_FieldDataLink.Field is TIB_ColumnBlob ) then begin
      SysDataChange;
    end;
  end;
end;

procedure TIB_Memo.SetAutoDisplay(Value: Boolean);
begin
  if FAutoDisplay <> Value then begin
    FAutoDisplay := Value;
    if Value then begin
      LoadMemo;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TIB_Memo.WndProc(var Message: TMessage);
begin
  with Message do
    if (Msg = WM_CREATE) or
       (Msg = WM_WINDOWPOSCHANGED) or
       (Msg = CM_FONTCHANGED) then begin
      FPaintControl.DestroyHandle;
    end;
  inherited;
end;

procedure TIB_Memo.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_Memo.CMExit(var Message: TCMExit);
begin
  try
    FIB_FieldDataLink.UpdateData;
  except
    SetFocus;
    raise;
  end;
  SetFocused(False);
  inherited;
end;

procedure TIB_Memo.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if FMemoLoaded then begin
    if (Key = VK_DELETE) or
      ((Key = VK_INSERT) and (ssShift in Shift)) then begin
      FIB_FieldDataLink.Modify;
    end;
  end else begin
    Key := 0;
  end;
end;

procedure TIB_Memo.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if FMemoLoaded then begin
    if (Key in [#32..#255]) and (FIB_FieldDataLink.Field <> nil) and
      not FIB_FieldDataLink.Field.IsValidChar(Key) then
    begin
      MessageBeep(0);
      Key := #0;
    end;
    case Key of
      ^H, ^I, ^J, ^M, ^V, ^X, #32..#255:
        FIB_FieldDataLink.Modify;
      #27:
        SysDataChange;
    end;
  end else begin
    if Key = #13 then begin
      LoadMemo;
    end;
    Key := #0;
  end;
end;

procedure TIB_Memo.Change;
begin
  if not FIgnoreChange then begin
    with FIB_FieldDataLink do begin
      if FMemoLoaded then begin
        Modify;
      end;
      FMemoLoaded := true;
      if State in [ dsInsert, dsEdit, dsSearch ] then begin
        ControlIsModified := true;
      end;
    end;
    inherited Change;
  end;
end;

procedure TIB_Memo.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  if not FMemoLoaded then begin
    LoadMemo;
  end else begin
    inherited;
  end;
end;

procedure TIB_Memo.WMCut(var Message: TMessage);
begin
  if FIB_FieldDataLink.Modify then begin
    inherited;
  end;
end;

procedure TIB_Memo.WMPaste(var Message: TMessage);
begin
  if FIB_FieldDataLink.Modify then begin
    inherited;
  end;
end;

procedure TIB_Memo.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FIB_FieldDataLink);
end;

procedure TIB_Memo.WMPaint(var Message: TWMPaint);
var
  S: string;
begin
  if not (csPaintCopy in ControlState) then begin
    inherited
  end else begin
    if FIB_FieldDataLink.Field <> nil then begin
      if FIB_FieldDataLink.Field is TIB_ColumnBlob then begin
        S := AdjustLineBreaks(FIB_FieldDataLink.Field.AsString)
      end else begin
        S := FIB_FieldDataLink.Field.AsString;
      end;
    end;
    SendMessage(FPaintControl.Handle, WM_SETTEXT, 0, Integer(PChar(S)));
    SendMessage(FPaintControl.Handle, WM_PAINT, Message.DC, 0);
  end;
end;


