// INT_IBC_RadioGroup

constructor TIB_RadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited TabStop := false;
  ControlStyle := [csSetCaption, csDoubleClicks];
  FButtons := TList.Create;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChange;
  FItemIndex := -1;
  FColumns := 1;
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  FIB_FieldDataLink.Control := Self;
  FIB_FieldDataLink.OnStateChanged := StateChanged;
  FIB_FieldDataLink.OnDataChange := DataChange;
  FIB_FieldDataLink.OnUpdateData := UpdateData;
  FValues := TStringList.Create;
  FBorderStyle := bsSingle;
  ArrangeButtons;
end;

destructor TIB_RadioGroup.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnStateChanged := nil;
    OnDataChange := nil;
    OnUpdateData := nil;
  end;
  TStringList(FItems).OnChange := nil;
  Destroying;
  SetButtonCount( 0 );
  FValues.Free;
  FValues := nil;
  FItems.Free;
  FItems := nil;
  FButtons.Free;
  FButtons := nil;
  inherited Destroy;
end;

procedure TIB_RadioGroup.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
  SysDataChange;
end;

procedure TIB_RadioGroup.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of Longint = (0, WS_BORDER);
begin
  inherited CreateParams(Params);
  with Params do begin
    Style := Style or BorderStyles[FBorderStyle];
    if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    end;
  end;
end;

procedure TIB_RadioGroup.SetBorderStyle(Value: TBorderStyle);
begin
  if FBorderStyle <> Value then begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TIB_RadioGroup.Paint;
begin
// Cause no default painting to occur.
// This is to remove the internal caption and box.
end;

procedure TIB_RadioGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
begin
  if (TCustomRadioGroupHack(Self).FButtons.Count <> 0) and
     not TCustomRadioGroupHack(Self).FReading then begin
    DC := GetDC(0);
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
    ReleaseDC(0, DC);
    ButtonsPerCol := (TCustomRadioGroupHack(Self).FButtons.Count + Columns - 1) div Columns;
    ButtonWidth := (Width - 10) div Columns;
    I := Height - 5;
    ButtonHeight := I div ButtonsPerCol;
    TopMargin := (I mod ButtonsPerCol) div 2 + 1;
    for I := 0 to TCustomRadioGroupHack(Self).FButtons.Count - 1 do
      with TRadioButton(TCustomRadioGroupHack(Self).FButtons[I]) do begin
        SetBounds((I div ButtonsPerCol) * ButtonWidth + 8,
          (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
          ButtonWidth, ButtonHeight);
        TabStop := Self.TabStop;  
        Visible := True;
      end;
  end;
end;

{------------------------------------------------------------------------------}
{ TIB_GroupButton                                                              }
{------------------------------------------------------------------------------}

type
  TIB_GroupButton = class( TRadioButton )
  private
    FInClick: Boolean;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure ChangeScale(M, D: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  end;

constructor TIB_GroupButton.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  if AOwner is TIB_RadioGroup then begin
    (AOwner as TIB_RadioGroup).FButtons.Add(Self);
    Enabled := (AOwner as TIB_RadioGroup).Enabled;
    OnClick := (AOwner as TIB_RadioGroup).ButtonClick;
  end;
  Visible := false;
  ParentShowHint := false;
  Parent := AOwner as TIB_RadioGroup;
end;

destructor TIB_GroupButton.Destroy;
begin
  TIB_RadioGroup(Owner).FButtons.Remove(Self);
  inherited Destroy;
end;

procedure TIB_GroupButton.CNCommand(var Message: TWMCommand);
begin
  if not FInClick then begin
    FInClick := True;
    try
      if ((Message.NotifyCode = BN_CLICKED      ) or
          (Message.NotifyCode = BN_DOUBLECLICKED)) and
         TIB_RadioGroup(Parent).CanModify then begin
        inherited;
      end;
    except
      Application.HandleException(Self);
    end;
    FInClick := False;
  end;
end;

procedure TIB_GroupButton.ChangeScale(M, D: Integer);
begin
// Disable ScaleBy for the child controls.
end;

procedure TIB_GroupButton.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TIB_RadioGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then begin
    if not TIB_RadioGroup(Parent).CanModify then Key := #0;
  end;
end;

procedure TIB_GroupButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  TIB_RadioGroup(Parent).KeyDown(Key, Shift);
end;

{------------------------------------------------------------------------------}
{ TIB_RadioGroup                                                               }
{------------------------------------------------------------------------------}

procedure TIB_RadioGroup.ButtonClick(Sender: TObject);
begin
  if not FEditing then begin
    FItemIndex := FButtons.IndexOf(Sender);
    Click;
  end;
end;

procedure TIB_RadioGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then begin
    if FItemIndex >= FItems.Count then begin
      FItemIndex := FItems.Count - 1;
    end;
    UpdateButtons;
  end;
end;

procedure TIB_RadioGroup.ReadState(Reader: TReader);
begin
  FReading := True;
  inherited ReadState(Reader);
  FReading := False;
  UpdateButtons;
end;

procedure TIB_RadioGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do TIB_GroupButton.Create(Self);
  while FButtons.Count > Value do TIB_GroupButton(FButtons.Last).Free;
end;

procedure TIB_RadioGroup.SetItemIndex(Value: Integer);
begin
  if FReading then FItemIndex := Value else begin
    if Value < -1 then Value := -1;
    if Value >= FButtons.Count then Value := FButtons.Count - 1;
    if FItemIndex <> Value then begin
      if FItemIndex >= 0 then begin
        TIB_GroupButton(FButtons[FItemIndex]).Checked := False;
      end;
      FItemIndex := Value;
      if FItemIndex >= 0 then begin
        TIB_GroupButton(FButtons[FItemIndex]).Checked := True;
      end;
    end;
  end;
end;

procedure TIB_RadioGroup.CMEnabledChanged(var Message: TMessage);
var
  I: Integer;
begin
  inherited;
  for I := 0 to FButtons.Count - 1 do
    TIB_GroupButton(FButtons[I]).Enabled := Enabled;
end;

procedure TIB_RadioGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then Value := 1;
  if Value > 16 then Value := 16;
  if FColumns <> Value then
  begin
    FColumns := Value;
    ArrangeButtons;
  end;
end;

procedure TIB_RadioGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I := 0 to FButtons.Count - 1 do
    TIB_GroupButton(FButtons[I]).Caption := FItems[I];
  if FItemIndex >= 0 then begin
    FEditing := True;
    try
      TIB_GroupButton(FButtons[FItemIndex]).Checked := True;
    finally
      FEditing := False;
    end;
  end;
  ArrangeButtons;
end;

procedure TIB_RadioGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ArrangeButtons;
end;

procedure TIB_RadioGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeButtons;
end;

{$IFNDEF VER100}
procedure TIB_RadioGroup.GetChildren(Proc: TGetChildProc );
{$ELSE}
procedure TIB_RadioGroup.GetChildren(Proc: TGetChildProc; Root: TComponent);
{$ENDIF}
begin
end;

{------------------------------------------------------------------------------}

procedure TIB_RadioGroup.StateChanged( Sender: TIB_DataLink;
                                       IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_RadioGroup.DataChange( Sender: TIB_DataLink;
                                     IB_DataSource: TIB_DataSource;
                                     IB_Field: TIB_Column );
begin
  SysDataChange;
end;

procedure TIB_RadioGroup.UpdateData( Sender: TIB_DataLink;
                                     IB_DataSource: TIB_DataSource;
                                     IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}

function TIB_RadioGroup.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;

function TIB_RadioGroup.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;

function TIB_RadioGroup.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;

function TIB_RadioGroup.GetPreventSearching: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsSearching;
end;

procedure TIB_RadioGroup.SetPreventEditing( Value: Boolean );
begin
  if PreventEditing <> Value then begin
    FIB_FieldDataLink.ControlPreventsEditing := Value;
    SysStateChanged;
  end;
end;

procedure TIB_RadioGroup.SetPreventInserting( Value: Boolean );
begin
  if PreventInserting <> Value then begin
    FIB_FieldDataLink.ControlPreventsInserting := Value;
    SysStateChanged;
  end;
end;

procedure TIB_RadioGroup.SetPreventSearching( Value: Boolean );
begin
  if PreventSearching <> Value then begin
    FIB_FieldDataLink.ControlPreventsSearching := Value;
    SysStateChanged;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_RadioGroup.SysDataChange;
begin
  Value := FIB_FieldDataLink.FieldText
end;

procedure TIB_RadioGroup.SysStateChanged;
begin
  with FIB_FieldDataLink do begin
    if ColorScheme then inherited Color := Color;
  end;
end;

procedure TIB_RadioGroup.SysUpdateData;
begin
  FIB_FieldDataLink.FieldText := Value;
end;

{------------------------------------------------------------------------------}

function TIB_RadioGroup.GetOnPrepareSQL: TIB_DataLinkEvent;
begin
  Result := FIB_FieldDataLink.OnPrepareSQL;
end;

procedure TIB_RadioGroup.SetOnPrepareSQL( AValue: TIB_DataLinkEvent);
begin
  FIB_FieldDataLink.OnPrepareSQL := AValue;
end;

function TIB_RadioGroup.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;

procedure TIB_RadioGroup.SetDataSource( AValue: TIB_DataSource );
begin
  FIB_FieldDataLink.DataSource := AValue;
end;

function TIB_RadioGroup.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;

procedure TIB_RadioGroup.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;

procedure TIB_RadioGroup.SetReadOnly( Value: Boolean );
begin
  FIB_FieldDataLink.ControlIsReadOnly := Value;
end;

function TIB_RadioGroup.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;

function TIB_RadioGroup.GetSearchBuffer: string;
begin
  if FIB_FieldDataLink <> nil then begin
    Result := FIB_FieldDataLink.SearchBuffer;
  end else begin
    Result := '';
  end;
end;

procedure TIB_RadioGroup.SetSearchBuffer( AValue: string );
begin
  if FIB_FieldDataLink <> nil then begin
    FIB_FieldDataLink.SearchBuffer := AValue;
  end;
end;

{------------------------------------------------------------------------------}

function TIB_RadioGroup.GetButtonValue( Index: Integer ): string;
begin
  if ( Index < FValues.Count ) and
     ( FValues[Index] <> ''  ) then
    Result := FValues[Index]
  else if Index < Items.Count then
    Result := Items[Index]
  else
    Result := '';
end;

procedure TIB_RadioGroup.SetValue( const Value: string );
var
  I, Index: Integer;
begin
  if ( FValue <> Value ) and not ReadOnly then begin
    FInSetValue := True;
    try
      Index := -1;
      for I := 0 to Items.Count - 1 do
        if Value = GetButtonValue(I) then begin
          Index := I;
          Break;
        end;
      ItemIndex := Index;
    finally
      FInSetValue := False;
    end;
    FValue := Value;
    Change;
  end;
end;

procedure TIB_RadioGroup.CMEnter( var Message: TCMEnter );
begin
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_RadioGroup.CMExit( var Message: TCMExit );
begin
  try
    SysUpdateData;
  except
    if ItemIndex >= 0 then TRadioButton(Controls[ItemIndex]).SetFocus
                      else TRadioButton(Controls[0]).SetFocus;
    raise;
  end;
  inherited;
end;

procedure TIB_RadioGroup.Click;
begin
  if not FInSetValue then begin
    if FIB_FieldDataLink.Modify then begin
      inherited Click;
      if ItemIndex >= 0 then begin
        Value := GetButtonValue(ItemIndex);
      end;
    end;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_RadioGroup.SetItems(Value: TStrings);
begin
  Items.Assign(Value);
  SysDataChange;
end;

procedure TIB_RadioGroup.SetValues(Value: TStrings);
begin
  FValues.Assign(Value);
  SysDataChange;
end;

procedure TIB_RadioGroup.Change;
begin
  with FIB_FieldDataLink do begin
    if Modifying then ControlIsModified := true;
  end;
  if Assigned( FOnChange ) then begin
    FOnChange(Self);
  end;
end;

procedure TIB_RadioGroup.KeyPress(var Key: Char);
begin
  inherited KeyPress( Key );
  if not ReadOnly then case Key of
    #8, ' ': FIB_FieldDataLink.Modify;
    #27:     SysDataChange;
  end;
end;

function TIB_RadioGroup.CanModify: Boolean;
begin
  Result := FIB_FieldDataLink.CanModify;
end;

procedure TIB_RadioGroup.SetTabStop( AValue: boolean );
begin
  if TabStop <> AValue then begin
    FTabStop := AValue;
    ArrangeButtons; 
  end;
end;

