// INT_IBC_RichEdit

{ TIB_RichEdit }

constructor TIB_RichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  FAutoDisplay := True;
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  with FIB_FieldDataLink do begin
    Control := Self;
    OnModifyingChanged := IB_ModifyingChanged;
    OnStateChanged := IB_StateChanged;
    OnDataChange := IB_DataChange;
    OnUpdateData := IB_UpdateData;
  end;
  FPaintControl := TPaintControl.Create(Self, 'EDIT');
end;

destructor TIB_RichEdit.Destroy;
begin
  DataSource := nil;
  with FIB_FieldDataLink do begin
    Control := nil;
    OnModifyingChanged := nil;
    OnStateChanged := nil;
    OnDataChange := nil;
    OnUpdateData := nil;
  end;
  FPaintControl.Free;
  FPaintControl := nil;
  inherited Destroy;
end;

procedure TIB_RichEdit.Loaded;
begin
  inherited Loaded;
  SysStateChanged;
  SysDataChange;
end;

{------------------------------------------------------------------------------}
{  DataLink event handlers.                                                    }
{------------------------------------------------------------------------------}

procedure TIB_RichEdit.IB_ModifyingChanged( Sender: TIB_DataLink;
                                            IB_DataSource: TIB_DataSource );
begin
  with FIB_FieldDataLink do begin
    inherited ReadOnly := ReadOnly or not ( State in [ dsInsert,
                                                       dsEdit,
                                                       dsSearch ]);
  end;
end;

procedure TIB_RichEdit.IB_StateChanged( Sender: TIB_DataLink;
                                        IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_RichEdit.IB_DataChange( Sender: TIB_DataLink;
                                      IB_DataSource: TIB_DataSource;
                                      IB_Field: TIB_Column );
begin
  FIgnoreChange := true;
  try
    SysDataChange;
  finally
    FIgnoreChange := false;
  end;
end;

procedure TIB_RichEdit.IB_UpdateData( Sender: TIB_DataLink;
                                      IB_DataSource: TIB_DataSource;
                                      IB_Field: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}
{ Implementation of data-aware logic.                                          }
{------------------------------------------------------------------------------}

procedure TIB_RichEdit.SysStateChanged;
begin
  if FIB_FieldDataLink.ColorScheme then begin
    inherited Color := FIB_FieldDataLink.Color;
  end;
end;

procedure TIB_RichEdit.SysDataChange;
begin
  with FIB_FieldDataLink do begin
    MaxLength := FieldMaxLength;
    FMemoLoaded := false;
    if Field <> nil then begin
      case State of
        dsBrowse,
        dsEdit,
        dsInsert,
        dsDelete: begin
          if Field is TIB_ColumnBlob then begin
            if AutoDisplay then begin
              LoadMemo;
            end else begin
              Text := FieldText;
            end;
          end else begin
            Text := FieldText;
            FMemoLoaded := true;
          end;
        end else begin
          Text := FieldText;
          FMemoLoaded := true;
        end;
      end;
    end else begin
      Text := FieldText;
    end;
  end;
  Modified := false;
end;

procedure TIB_RichEdit.SysUpdateData;
begin
  if Modified then begin
    FIB_FieldDataLink.FieldText := Text;
  end;
end;

procedure TIB_RichEdit.LoadMemo;
begin
  if not FMemoLoaded and ( Field <> nil ) then begin
    try
      with Field as TIB_ColumnBlob do begin
        FIgnoreChange := true;
        try
          AsStream.Position := 0;
          Lines.LoadFromStream( AsStream );
        finally
          FIgnoreChange := false;
        end;
      end;
      FMemoLoaded := True;
    except
      Lines.Text := 'Could not load memo.';
    end;
    SysStateChanged;
    Modified := false;
  end;
end;

{------------------------------------------------------------------------------}
{ DataLink property coordination methods.                                      } 
{------------------------------------------------------------------------------}

function TIB_RichEdit.GetOnPrepareSQL: TIB_DataLinkEvent;
begin
  Result := FIB_FieldDataLink.OnPrepareSQL;
end;
procedure TIB_RichEdit.SetOnPrepareSQL( AValue: TIB_DataLinkEvent);
begin
  FIB_FieldDataLink.OnPrepareSQL := AValue;
end;
function TIB_RichEdit.GetDataSource: TIB_DataSource;
begin
  Result := FIB_FieldDataLink.DataSource;
end;
procedure TIB_RichEdit.SetDataSource(AValue: TIB_DataSource);
begin
  FIB_FieldDataLink.DataSource := AValue;
end;
function TIB_RichEdit.GetDataField: string;
begin
  Result := FIB_FieldDataLink.FieldName;
end;
procedure TIB_RichEdit.SetDataField( const AValue: string );
begin
  FIB_FieldDataLink.FieldName := AValue;
end;
function TIB_RichEdit.GetField: TIB_Column;
begin
  Result := FIB_FieldDataLink.Field;
end;
function TIB_RichEdit.GetReadOnly: Boolean;
begin
  Result := FIB_FieldDataLink.ControlIsReadOnly;
end;
procedure TIB_RichEdit.SetReadOnly( Value: Boolean );
begin
  FIB_FieldDataLink.ControlIsReadOnly := Value;
end;
function TIB_RichEdit.GetPreventEditing: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsEditing;
end;
procedure TIB_RichEdit.SetPreventEditing( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsEditing := Value;
end;
function TIB_RichEdit.GetPreventInserting: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsInserting;
end;
procedure TIB_RichEdit.SetPreventInserting( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsInserting := Value;
end;
function TIB_RichEdit.GetPreventSearching: Boolean;
begin
  Result := FIB_FieldDataLink.ControlPreventsSearching;
end;
procedure TIB_RichEdit.SetPreventSearching( Value: Boolean );
begin
  FIB_FieldDataLink.ControlPreventsSearching := Value;
end;
function TIB_RichEdit.GetSearchBuffer: string;
begin
  Result := FIB_FieldDataLink.SearchBuffer;
end;
procedure TIB_RichEdit.SetSearchBuffer( AValue: string );
begin
  FIB_FieldDataLink.SearchBuffer := AValue;
end;

{------------------------------------------------------------------------------}

procedure TIB_RichEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then begin
    FFocused := Value;
    if not (FIB_FieldDataLink.Field is TIB_ColumnBlob) then begin
      SysDataChange;
    end;
  end;
end;

procedure TIB_RichEdit.SetAutoDisplay(Value: Boolean);
begin
  if AutoDisplay <> Value then begin
    FAutoDisplay := Value;
    if Value then begin
      LoadMemo;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TIB_RichEdit.WndProc(var Message: TMessage);
begin
  with Message do
    if (Msg = WM_CREATE) or
       (Msg = WM_WINDOWPOSCHANGED) or
       (Msg = CM_FONTCHANGED) then begin
      FPaintControl.DestroyHandle;
    end;
  inherited;
end;

procedure TIB_RichEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  FIB_FieldDataLink.SetFocus;
  inherited;
end;

procedure TIB_RichEdit.CMExit(var Message: TCMExit);
begin
  try
    FIB_FieldDataLink.UpdateData;
  except
    SetFocus;
    raise;
  end;
  SetFocused(False);
  inherited;
end;

procedure TIB_RichEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if FMemoLoaded then begin
    if (Key = VK_DELETE) or
      ((Key = VK_INSERT) and (ssShift in Shift)) then begin
      FIB_FieldDataLink.Modify;
    end;
  end else begin
    Key := 0;
  end;
end;

procedure TIB_RichEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if FMemoLoaded then begin
    if (Key in [#32..#255]) and (Field <> nil) and
      not FIB_FieldDataLink.FieldIsValidChar(Key) then
    begin
      MessageBeep(0);
      Key := #0;
    end;
    case Key of
      ^H, ^I, ^J, ^M, ^V, ^X, #32..#255:
        FIB_FieldDataLink.Modify;
      #27:
        SysDataChange;
    end;
  end else begin
    if Key = #13 then begin
      LoadMemo;
    end;
    Key := #0;
  end;
end;

procedure TIB_RichEdit.Change;
begin
  if not FIgnoreChange then begin
    with FIB_FieldDataLink do begin
      if FMemoLoaded then begin
        Modify;
      end;
      FMemoLoaded := true;
      if State in [ dsInsert, dsEdit, dsSearch ] then begin
        ControlIsModified := true;
      end;
    end;
    inherited Change;
  end;
end;

procedure TIB_RichEdit.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  if not FMemoLoaded then begin
    LoadMemo;
  end else begin
    inherited;
  end;
end;

procedure TIB_RichEdit.WMCut(var Message: TMessage);
begin
  if FIB_FieldDataLink.Modify then begin
    inherited;
  end;
end;

procedure TIB_RichEdit.WMPaste(var Message: TMessage);
begin
  if FIB_FieldDataLink.Modify then begin
    inherited;
  end;
end;

procedure TIB_RichEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FIB_FieldDataLink);
end;

procedure TIB_RichEdit.WMPaint(var Message: TWMPaint);
var
  S: string;
begin
  if not (csPaintCopy in ControlState) then begin
    inherited
  end else begin
    if FIB_FieldDataLink.Field <> nil then begin
      if FIB_FieldDataLink.Field is TIB_ColumnBlob then begin
        S := AdjustLineBreaks(FIB_FieldDataLink.Field.AsString)
      end else begin
        S := FIB_FieldDataLink.Field.AsString;
      end;
    end;
    SendMessage(FPaintControl.Handle, WM_SETTEXT, 0, Integer(PChar(S)));
    SendMessage(FPaintControl.Handle, WM_PAINT, Message.DC, 0);
  end;
end;


