
{                                                                              }
{ TIB_Column                                                                   }
{                                                                              }

  TIB_ColumnEvent = procedure( Sender: TIB_Column ) of object;

  TIB_Column = class(TObject)
  private
    FIB_Session: TIB_Session;
    FIB_Row: TIB_Row;
    FFieldNo: smallint;
    FPXSQLVAR: PXSQLVAR;
    FDataSize: word;
    FOldColumnInd: ^smallint;
    FNewColumnInd: ^smallint;
    FOldColumnBuffer: pointer;
    FNewColumnBuffer: pointer;
    FDomainName: string;
    FReadOnly: boolean;    { User definable storage Column. }
    FSysReadOnly: boolean; { System flag used to lock down Columns. }
    FAlignment: TAlignment;
    FEditMask: string;
    FDisplayFormat: string;
    FDisplayLabel: string;
    FDisplayWidth: integer;
    FVisible: boolean;
    FIsText: boolean;
    FIsBlob: boolean;
    FIsLoaded: boolean;
    FNeedToStore: boolean;
    FOldIsLoaded: boolean;
    FOnBeforeModify: TIB_ColumnEvent;
    FOnAfterModify: TIB_ColumnEvent;
  { Property access methods }
    function GetSQLName: string;
    function GetRelName: string;
    function GetOwnerName: string;
    function GetFieldName: string;
    function GetFullFieldName: string;
    function GetDomainName: string;
    function GetSQLType: smallInt;
    function GetSQLScale: smallInt;
    function GetSQLSubType: smallInt;
    function GetSQLLen: smallInt;
    function GetOldIsNull: boolean;
    procedure SetIsNull( NewValue: boolean ); 
    function GetIsNull: boolean;
    function GetIsNullable: boolean;
    function GetIsPrimary: boolean;
    function GetIsForeign: boolean;
    function GetIsDefaulted: boolean;
    function GetIsIndexField: boolean;
    function GetIsAlternate: boolean;
    function GetStatement: TIB_Statement;
    function GetIndex: integer;
    procedure SetIndex( AValue: integer );
  protected
  { Property access methods }
    function GetReadOnly: boolean; virtual;
    function GetIsModified: boolean; virtual;
    function GetIsCalculated: boolean; virtual;
    function GetRequired: boolean; virtual;
    function GetOldAsString: string; virtual;
    function GetAsString: string; virtual;
    function GetAsDateTime: TDateTime; virtual;
    function GetAsInteger: integer; virtual;
    function GetAsFloat: single; virtual;
    function GetAsSmallInt: smallint; virtual;
    function GetAsDouble: double; virtual;
    function GetAsCurrency: currency; virtual;
    function GetAsComp: comp; virtual;
    function GetAsExtended: extended; virtual;
    function GetAsVariant: Variant; virtual;
    function GetAsStream: TStream; virtual; 
    function GetOldAsStream: TStream; virtual;
  { Property Manipulation Methods }
    procedure SetAsString( NewValue: string ); virtual;
    procedure SetAsDateTime( NewValue: TDateTime ); virtual;
    procedure SetAsInteger( NewValue: integer ); virtual;
    procedure SetAsFloat( NewValue: single ); virtual;
    procedure SetAsSmallInt( NewValue: smallint ); virtual;
    procedure SetAsDouble( NewValue: double ); virtual;
    procedure SetAsCurrency( NewValue: currency ); virtual;
    procedure SetAsComp( NewValue: comp ); virtual;
    procedure SetAsExtended( NewValue: extended); virtual;
    procedure SetAsVariant( NewValue: Variant ); virtual;
    procedure SetAsStream( AStream: TStream ); virtual; 
    function GetSQLTypeSource: string; virtual;
    function GetDisplayName: string; virtual;
    function GetDisplayText: string; virtual;
    function GetDefaultWidth: integer; virtual;
    function GetDisplayWidth: integer; virtual;
  { Attribute Manipulation Methods }
    function GetAttributeParams( AKey: string ): string;
    procedure SetAttributeParams( AKey: string; AValue: string );
    function GetIsAttributeSet( AKey: string ): boolean;
    procedure SetIsAttributeSet( AKey: string; AValue: boolean );
  { Property coordination methods }
    procedure SetAlignment( AValue: TAlignment ); virtual;
    procedure SetDisplayFormat( AValue: string ); virtual;
    procedure SetEditMask( AValue: string ); virtual;
    procedure SetDisplayLabel( AValue: string ); virtual;
    procedure SetDisplayWidth( AValue: integer ); virtual;
    procedure SetVisible( AValue: boolean ); virtual;
  { System methods }
    procedure SysInitBuffer; virtual;
    procedure SysFreeBuffer; virtual;
    procedure SysBeforeModify; virtual;
    procedure SysAfterModify; virtual;
    procedure SysSetIsNull( NewValue: boolean ); virtual;
  { Event dispatch methods }
    procedure DoBeforeModify; virtual;
    procedure DoAfterModify; virtual;
  public
    constructor Create( AIB_Row: TIB_Row; AFieldNo: smallint ); virtual;
    destructor Destroy; override;
  { Methods }
    function IsValidChar( AChar: Char ): boolean; virtual;
    procedure Clear; virtual;
    procedure FocusControl; 
  { Properties }
    property IB_Session: TIB_Session read FIB_Session;
    property Statement: TIB_Statement read GetStatement;
    property IB_Row: TIB_Row read FIB_Row;
    property DataSize: word read FDataSize;
    property FieldNo: smallint read FFieldNo;
    property PSQLVAR: PXSQLVAR read FPXSQLVAR;
    property SQLName:    string read GetSQLName;
    property RelName:    string read GetRelName;
    property OwnerName:  string read GetOwnerName;
    property FieldName:  string read GetFieldName;
    property FullFieldName: string read GetFullFieldName;
    property DomainName: string read GetDomainName;
    property DisplayText: string read GetDisplayText;
    property DisplayName: string read GetDisplayName;
    property SQLTypeSource: string read GetSQLTypeSource;
    property SQLType:    smallInt read GetSQLtype;
    property SQLScale:   smallInt read GetSQLscale;
    property SQLSubType: smallInt read GetSQLsubtype;
    property SQLLen:     smallInt read GetSQLLen;
    property IsModified:   boolean read GetIsModified;
    property IsCalculated: boolean read GetIsCalculated;
    property IsDefaulted:  boolean read GetIsDefaulted;
    property Required:     boolean read GetRequired;
    property IsNullable:   boolean read GetIsNullable;
    property IsPrimary:    boolean read GetIsPrimary;
    property IsForeign:    boolean read GetIsForeign;
    property IsIndexField: boolean read GetIsIndexField;
    property IsAlternate:  boolean read GetIsAlternate;
    property AsString: string read GetAsString write SetAsString;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsInteger: integer read GetAsInteger write SetAsInteger;
    property AsFloat: single read GetAsFloat write SetAsFloat;
    property AsDouble: double read GetAsDouble write SetAsDouble;
    property AsCurrency: currency read GetAsCurrency write SetAsCurrency;
    property AsComp: comp read GetAscomp write SetAscomp;
    property AsExtended: extended read GetAsExtended write SetAsExtended;
    property AsStream: TStream read GetAsStream write SetAsStream;
    property OldAsStream: TStream read GetOldAsStream;
    property IsNull: boolean read GetIsNull write SetIsNull;
    property OldAsString: string read GetOldAsString;
    property OldIsNull: boolean read GetOldIsNull;
    property IsBlob: boolean read FIsBlob;
    property IsText: boolean read FIsText;
    property IsLoaded: boolean read FIsLoaded;
    property OldIsLoaded: boolean read FOldIsLoaded;
    property NeedToStore: boolean read FNeedToStore;
    property ReadOnly: boolean read GetReadOnly;
    property IsAttributeSet[ AKey: string ]: boolean
        read GetIsAttributeSet
       write SetIsAttributeSet;
    property AttributeParams[ AKey: string ]: string
        read GetAttributeParams
       write SetAttributeParams;
  published
  { Properties }
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property DisplayFormat: string read FDisplayFormat write SetDisplayFormat;
    property DisplayLabel: string read FDisplayLabel write SetDisplayLabel;
    property DisplayWidth: integer read GetDisplayWidth write SetDisplayWidth;
    property EditMask: string read FEditMask write SetEditMask;
    property Index: integer read GetIndex write SetIndex;
    property Visible: boolean read FVisible write SetVisible;
  { Events }
    property OnBeforeModify: TIB_ColumnEvent read FOnBeforeModify
                                            write FOnBeforeModify;
    property OnAfterModify: TIB_ColumnEvent read FOnBeforeModify
                                           write FOnBeforeModify;
  end;

  TIB_ColumnNumBase = class( TIB_Column )
  protected
    function GetDisplayText: string; override; 
  end;

  TIB_ColumnNumeric = class( TIB_ColumnNumBase )
  private
    FNoRoundErr: boolean;
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string); override;
    function GetAsExtended: extended; override;
    procedure SetAsExtended( NewValue: extended); override;
    function GetValue: extended; virtual;
    procedure SetValue( NewValue: extended ); virtual;
    function GetSQLTypeSource: string; override;
  public
    property Value: extended read GetValue write SetValue;
    property NoRoundErr: boolean read FNoRoundErr;
  end;

  TIB_ColumnFloat = class( TIB_ColumnNumBase )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string); override;
    function GetAsFloat: single; override;
    procedure SetAsFloat( NewValue: single); override;
    function GetValue: single; virtual;
    procedure SetValue( NewValue: single ); virtual;
  public
    property Value: single read GetValue write SetValue;
  end;

  TIB_ColumnDouble = class( TIB_ColumnNumBase )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string); override;
    function GetAsDouble: Double; override;
    procedure SetAsDouble( NewValue: Double); override;
    function GetValue: double; virtual;
    procedure SetValue( NewValue: double ); virtual;
  public
    property Value: double read GetValue write SetValue;
  end;

  TIB_ColumnSmallInt = class( TIB_ColumnNumBase )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string ); override;
    function GetAsSmallInt: SmallInt; override;
    procedure SetAsSmallInt( NewValue: SmallInt); override;
    function GetValue: smallint; virtual;
    procedure SetValue( NewValue: smallint ); virtual;
  public
    property Value: smallint read GetValue write SetValue;
  end;

  TIB_ColumnInteger = class( TIB_ColumnNumBase )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string ); override;
    function GetAsInteger: integer; override;
    procedure SetAsInteger( NewValue: integer ); override;
    function GetValue: integer; virtual;
    procedure SetValue( NewValue: integer ); virtual;
  public
    property Value: integer read GetValue write SetValue;
  end;

  TIB_ColumnCurr = class( TIB_ColumnNumBase )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string ); override;
    function GetAsCurrency: currency; override;
    procedure SetAsCurrency( NewValue: currency ); override;
    function GetValue: currency; virtual;
    procedure SetValue( NewValue: currency ); virtual;
  public
    property Value: currency read GetValue write SetValue;
  end;

  TIB_ColumnComp = class( TIB_ColumnNumBase )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string ); override;
    function GetAsComp: Comp; override;
    procedure SetAsComp( NewValue: Comp ); override;
    function GetValue: comp; virtual;
    procedure SetValue( NewValue: comp ); virtual;
  public
    property Value: comp read GetValue write SetValue;
  end;

  TIB_ColumnDateTime = class( TIB_Column )
  protected
//    FYear:  word;
//    FMonth: word;
//    FDay:   word;
//    FHour:  word;
//    FMin:   word;
//    FSec:   word;
//    FMSec:  word;
//    FTMSec: word;
//    FCDateTime: TM;
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string ); override;
    function GetAsDateTime: TDateTime; override;
    procedure SetAsDateTime( NewValue: TDateTime ); override;
    function GetDisplayText: string; override;
    function GetValue: TDateTime; virtual;
    procedure SetValue( NewValue: TDateTime ); virtual;
  public
    property Value: TDateTime read GetValue write SetValue;
//    property Year:  word read FYear;
//    property Month: word read FMonth;
//    property Day:   word read FDay;
//    property Hour:  word read FHour;
//    property Min:   word read FMin;
//    property Sec:   word read FSec;
//    property MSec:  word read FMSec;
//    property TMSec: word read FTMsec;
//    property CDateTime: TM read FCDateTime;
  end;

  TIB_ColumnText = class( TIB_Column )
  protected
    function GetAsString: string; override;
    procedure SetAsString( NewValue: string ); override;
    function GetValue: string; virtual;
    procedure SetValue( NewValue: string ); virtual;
  public
    constructor Create( AIB_Row: TIB_Row; AFieldNo: smallint ); override;
    property Value: string read GetValue write SetValue;
  end;

  TIB_ColumnVarText = class( TIB_ColumnText )
  protected
    function GetValue: string; override;
    procedure SetValue( NewValue: string ); override;
  end;

  TIB_ColumnDB_KEY = class( TIB_ColumnText )
  protected
    function GetDefaultWidth: integer; override;
    function GetValue: string; override;
    procedure SetValue( NewValue: string ); override;
  end;

// IMP_IB_Column
