
{                                                                              }
{ TIB_Connection                                                               }
{                                                                              }

  TIB_ConnectionStatus = ( csDisconnected,        csConnected,
                           csDisconnectPending,   csConnectPending,
                           csDropDatabasePending, csCreateDatabasePending,
                           csForcedConnectPending );

  TIB_ConnectionEventType = ( cetBeforeAssignment,
                              cetAfterAssignment,
                              cetBeforeExecDDL,
                              cetAfterExecDDL,
                              cetConnectedChanged,
                              cetBeforeCreateDatabase,
                              cetAfterCreateDatabase,
                              cetBeforeConnect,
                              cetAfterConnect,
                              cetBeforeDisconnect,
                              cetAfterDisconnect,
                              cetBeforeDropDatabase,
                              cetAfterDropDatabase );

  TIB_Protocol = ( cpLocal, cpNetBEUI, cpTCP_IP, cpNovell, cpOther );
  TIB_ForcedWrites = ( fwDefault, fwTrue, fwFalse );
  TIB_ReservePageSpace = ( rpsDefault, rpsTrue, rpsFalse );
  TIB_DBKEYScope = ( ksDefault, ksTransaction, ksConnection );
  EIB_ConnectionError = class( EIB_Error );
  TIB_ConnectionEvent = procedure( Sender: TIB_Connection ) of object;
  TIB_LoginEvent = procedure(     Sender: TIB_Connection;
                              var Abort: boolean ) of object;

  TIB_CustomizeDBPEvent = procedure (     Sender: TIB_Connection;
                                      var BufPtr: integer;
                                      var Buffer: array of char ) of object;

  TIB_ConnectCharacteristics = packed record
    dbAllocation: longint;
    dbLevelPrefix: byte;
    dbBase_Level: byte;
    dbFilePrefix: byte;
    dbFile: string;
    dbSite: string;
    dbImplementationPrefix: byte;
    dbImplementation: byte;
    dbClass: byte;
    dbNo_Reserve: byte;
    dbForced_Writes: byte;
    dbODS_Minor_Version: word;
    dbODS_Version: word;
    dbPage_Size: word;
    dbVersionPrefix: byte;
    dbVersion: string;
  end;

  TIB_Connection = class(TIB_Component)
  private
  { Utility storage }
    flag_connect_after_load: boolean;
    flag_domain_name_info_invalid: boolean;
    flag_generator_name_info_invalid: boolean;
    flag_table_name_info_invalid: boolean;
    flag_view_name_info_invalid: boolean;
    flag_procedure_name_info_invalid: boolean;
    flag_constraint_name_info_invalid: boolean;
    flag_trigger_name_info_invalid: boolean;
    flag_index_name_info_invalid: boolean;
    flag_primary_key_info_invalid: boolean;
    flag_foreign_key_info_invalid: boolean;
    flag_required_info_invalid: boolean;
    flag_defaults_info_invalid: boolean;
    flag_characteristics_info_invalid: boolean;
    flag_junk_value: integer;
  { Property storage fields }
    FdbHandle: isc_db_Handle;
    FIsHandleShared: boolean;
    FParams: TIB_StringList;
    FDomainNames: TIB_StringList;
    FGeneratorNames: TIB_StringList;
    FTableNames: TIB_StringList;
    FViewNames: TIB_StringList;
    FProcedureNames: TIB_StringList;
    FConstraintNames: TIB_StringList;
    FTriggerNames: TIB_StringList;
    FIndexNames: TIB_StringList;
    FPrimaryKeys: TIB_StringList;
    FForeignKeys: TIB_StringList;
    FRequired: TIB_StringList;
    FDefaults: TIB_StringList;
    FStatementList: TList;
    FDataSetList: TList;
    FTransactionList: TList;
    FConnectionLinkList: TList;
    FConnectionStatus: TIB_ConnectionStatus;
    FAnnounceFocus: boolean;
    FLoginPrompt: boolean;
    FLoginAttempts: word;
    FLoginAborted: boolean;
    FLoginDBReadOnly: boolean;
  { Database characteristics }
    FCharacteristics: TIB_ConnectCharacteristics;
  { Event Storage }
    FBeforeExecDDL: TIB_ConnectionEvent;
    FAfterExecDDL: TIB_ConnectionEvent;
    FBeforeConnect: TIB_ConnectionEvent;
    FAfterConnect: TIB_ConnectionEvent;
    FBeforeDisconnect: TIB_ConnectionEvent;
    FAfterDisconnect: TIB_ConnectionEvent;
    FBeforeCreateDatabase: TIB_ConnectionEvent;
    FAfterCreateDatabase: TIB_ConnectionEvent;
    FBeforeDropDatabase: TIB_ConnectionEvent;
    FAfterDropDatabase: TIB_ConnectionEvent;
    FOnConnectedChanged: TIB_ConnectionEvent;
    FOnLogin: TIB_LoginEvent;
    FOnLoginFailure: TIB_ConnectionEvent;
    FOnGainFocus: TIB_ConnectionEvent;
    FOnLoseFocus: TIB_ConnectionEvent;
    FOnCustomizeDPB: TIB_CustomizeDBPEvent;
  { Property Access Methods }
    procedure SetParams( Value: TIB_StringList );
    function GetPrm( Index: integer ): string;
    procedure SetPrm( Index: integer; Value: string );
    function GetParameter( Parameter: string ): string;
    procedure SetParameter( Parameter: string; Value: string );
    function GetForcedWrites: TIB_ForcedWrites;
    procedure SetForcedWrites( AValue: TIB_ForcedWrites );
    function GetDBKeyScope: TIB_DBKEYScope;
    procedure SetDBKeyScope( AValue: TIB_DBKEYScope );
    function GetReservePageSpace: TIB_ReservePageSpace;
    procedure SetReservePageSpace( AValue: TIB_ReservePageSpace );
    function GetBuffers: byte;
    procedure SetBuffers( AValue: byte );
    function GetPageSize: word;
    procedure SetPageSize( AValue: word );
    function GetTransactionCount: integer;
    function GetTransaction( Index: integer ): TIB_Transaction;
    function GetStatementCount: integer;
    function GetStatement( Index: integer ): TIB_Statement;
    function GetDataSetCount: integer;
    function GetDataSet( Index: integer ): TIB_DataSet;
    function GetConnectionLinkCount: integer;
    function GetPdbHandle: pisc_db_handle;
    procedure SetdbHandle( AValue: isc_db_handle );
    procedure SetdbHandleShared( AValue: isc_db_handle );
    function GetDomainNames: TStrings;
    function GetGeneratorNames: TStrings;
    function GetTableNames: TStrings;
    function GetViewNames: TStrings;
    function GetProcedureNames: TStrings;
    function GetConstraintNames: TStrings;
    function GetTriggerNames: TStrings;
    function GetIndexNames: TStrings;
    function GetPrimaryKeys: TStrings;
    function GetForeignKeys: TStrings;
    function GetRequired: TStrings;
    function GetDefaults: TStrings;
    function GetProtocol: TIB_Protocol;
    procedure SetProtocol( AValue: TIB_Protocol );
    function GetCharacteristics: TIB_ConnectCharacteristics;
  protected
  { Inherited Methods }
    procedure Loaded; override;
  { Property Access methods }
    function GetDatabaseName: string; virtual;
    procedure SetDatabaseName( AValue: string ); virtual;
    procedure SetConnected( Value: boolean); virtual;
    function GetConnected: boolean; virtual;
    procedure SetAnnounceFocus( AValue: boolean ); virtual;
  { System Methods }
    procedure SysInvalidateOptimizationFlags; virtual;
    procedure SysUpdateCharacteristics; virtual;
    procedure SysConnectAfterLoad; virtual;
    procedure SysCheckTransactionsAfterLoad; virtual;
    procedure SysCheckStatementsAfterLoad; virtual;
    procedure SysBeforeExecDDL; virtual;
    procedure SysAfterExecDDL; virtual;
    procedure SysBeforeConnect; virtual;
    procedure SysLogin; virtual;
    procedure SysBeforeDisconnect; virtual;
    function SysConnect: boolean; virtual;
    procedure SysDisconnect; virtual;
    procedure SysAfterConnect; virtual;
    procedure SysAfterDisconnect; virtual;
    procedure SysCloseTransactions; virtual;
    procedure SysDeallocateStatements; virtual;
    procedure SysStartTransactions; virtual;
    procedure SysCreateDatabase; virtual;
    procedure SysDropDatabase; virtual;
    procedure SysBeforeCreateDatabase; virtual;
    procedure SysBeforeDropDatabase; virtual;
    procedure SysAfterCreateDatabase; virtual;
    procedure SysAfterDropDatabase; virtual;
  { API level calls }
    procedure API_Connect;
    procedure API_Disconnect;
    procedure API_Database_Info( var Items:  array of Char;
                                 var Buffer: array of Char );
    procedure SQL_CreateDatabase; 
  { Event Dispatch Methods }
    procedure DoBeforeExecDDL; virtual;
    procedure DoAfterExecDDL; virtual;
    procedure DoLogin; virtual;
    procedure DoLoginFailure; virtual;
    procedure DoBeforeConnect; virtual;
    procedure DoAfterConnect; virtual;
    procedure DoBeforeDisconnect; virtual;
    procedure DoAfterDisconnect; virtual;
    procedure DoBeforeCreateDatabase; virtual;
    procedure DoAfterCreateDatabase; virtual;
    procedure DoBeforeDropDatabase; virtual;
    procedure DoAfterDropDatabase; virtual;
    procedure DoConnectedChanged; virtual;
    procedure DoGainFocus; virtual;
    procedure DoLoseFocus; virtual;
    procedure DoCustomizeDPB( var bufptr: integer;
                              var buffer: array of char ); virtual;
  { Link Dispatch Method }
    procedure DoLinkEvent( AEvent: TIB_ConnectionEventType );
  { Protected properties }
    property ConnectionLinkCount: integer read GetConnectionLinkCount;
  { Link handling methods }
    procedure AddConnectionLink( NewLink: TObject );
    procedure RemoveConnectionLink( OldLink: TObject ); 
  public
  { Inherited Methods }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  { New methods }
    procedure Connect; dynamic;
    procedure Disconnect; dynamic;
    procedure Open; dynamic;
    procedure Close; dynamic;
    procedure CloseDataSets; dynamic;
    procedure CreateDatabase; dynamic;
    procedure DropDatabase; dynamic;
    procedure SetFocus; dynamic;
  { properties }
    property dbHandle: isc_db_Handle read FdbHandle write SetdbHandle;
    property dbHandleShared: isc_db_Handle write SetdbHandleShared;
    property PdbHandle: pisc_db_Handle read GetPdbHandle;
    property IsHandleShared: boolean read FIsHandleShared write FIsHandleShared;
    property TransactionCount: integer read GetTransactionCount;
    property Transactions[index: integer]: TIB_Transaction read GetTransaction;
    property StatementCount: integer read GetStatementCount;
    property Statements[index: integer]: TIB_Statement read GetStatement;
    property DataSetCount: integer read GetDataSetCount;
    property DataSets[index: integer]: TIB_DataSet read GetDataSet;
    property Parameter[index: string]: string read GetParameter
                                              write SetParameter;
    property ConnectionStatus: TIB_ConnectionStatus read FConnectionStatus;
    property Characteristics: TIB_ConnectCharacteristics
        read GetCharacteristics;
    property Database: string read GetDatabaseName
                              write SetDatabaseName
                              stored false;
    property Info_DomainNames: TStrings read GetDomainNames;
    property Info_GeneratorNames: TStrings read GetGeneratorNames;
    property Info_TableNames: TStrings read GetTableNames;
    property Info_ViewNames: TStrings read GetViewNames;
    property Info_ProcedureNames: TStrings read GetProcedureNames;
    property Info_ConstraintNames: TStrings read GetConstraintNames;
    property Info_TriggerNames: TStrings read GetTriggerNames;
    property Info_IndexNames: TStrings read GetIndexNames;
    property Info_PrimaryKeys: TStrings read GetPrimaryKeys;
    property Info_ForeignKeys: TStrings read GetForeignKeys;
    property Info_Required: TStrings read GetRequired;
    property Info_Defaults: TStrings read GetDefaults;
  published
  { Properties }
    property AnnounceFocus: boolean read FAnnounceFocus
                                    write SetAnnounceFocus
                                    default false;
    property LoginPrompt: boolean  read FLoginPrompt write FLoginPrompt;
    property LoginAborted: boolean read FLoginAborted;
    property LoginAttempts: word read FLoginAttempts
                                 write FLoginAttempts
                                 default 3;
    property LoginDBReadOnly: boolean read FLoginDBReadOnly
                                      write FLoginDBReadOnly
                                      default false;
    property Username:    string index  1 read GetPrm write SetPrm stored false;
    property Password:    string index  2 read GetPrm write SetPrm stored false;
    property Server:      string index  3 read GetPrm write SetPrm stored false;
    property Path:        string index  5 read GetPrm write SetPrm stored false;
//  property License:     string index  6 read GetPrm write SetPrm stored false;
//  property SysDBA:      string index  7 read GetPrm write SetPrm stored false;
//  property EncryptKey:  string index  8 read GetPrm write SetPrm stored false;
//  property CharSet:     string index  9 read GetPrm write SetPrm stored false;
//  property MessageFile: string index 10 read GetPrm write SetPrm stored false;
//  property ShadowFile:  string index 11 read GetPrm write SetPrm stored false;
    property Protocol: TIB_Protocol read GetProtocol
                                    write SetProtocol
                                    stored false;
    property ForcedWrites: TIB_ForcedWrites read GetForcedWrites
                                            write SetForcedWrites
                                            stored false;
    property DBKeyScope: TIB_DBKEYScope read GetDBKeyScope
                                        write SetDBKeyScope
                                        stored false;
    property ReservePageSpace: TIB_ReservePageSpace read GetReservePageSpace
                                                    write SetReservePageSpace
                                                    stored false;
    property DatabaseName: string read GetDatabaseName
                                  write SetDatabaseName
                                  stored false;
    property Buffers: byte read GetBuffers write SetBuffers stored false;
    property PageSize: word read GetPageSize write SetPageSize stored false;
    property Params: TIB_StringList read FParams write SetParams;
    property Connected: Boolean read GetConnected
                                write SetConnected
                                default false;
  { New Events }
    property BeforeExecDDL: TIB_ConnectionEvent
       read FBeforeExecDDL
      write FBeforeExecDDL;
    property AfterExecDDL: TIB_ConnectionEvent
       read FAfterExecDDL
      write FAfterExecDDL;
    property BeforeConnect: TIB_ConnectionEvent
       read FBeforeConnect
      write FBeforeConnect;
    property AfterConnect: TIB_ConnectionEvent
       read FAfterConnect
      write FAfterConnect;
    property BeforeDisconnect: TIB_ConnectionEvent
       read FBeforeDisconnect
      write FBeforeDisconnect;
    property AfterDisconnect: TIB_ConnectionEvent
       read FAfterDisconnect
      write FAfterDisconnect;
    property BeforeCreateDatabase: TIB_ConnectionEvent
       read FBeforeCreateDatabase
      write FBeforeCreateDatabase;
    property AfterCreateDatabase: TIB_ConnectionEvent
       read FAfterCreateDatabase
      write FAfterCreateDatabase;
    property BeforeDropDatabase: TIB_ConnectionEvent
       read FBeforeDropDatabase
      write FBeforeDropDatabase;
    property AfterDropDatabase: TIB_ConnectionEvent
       read FAfterDropDatabase
      write FAfterDropDatabase;
    property OnConnectedChanged: TIB_ConnectionEvent
       read FOnConnectedChanged
      write FOnConnectedChanged;
    property OnLogin: TIB_LoginEvent
       read FOnLogin
      write FOnLogin;
    property OnLoginFailure: TIB_ConnectionEvent
       read FOnLoginFailure
      write FOnLoginFailure;
    property OnGainFocus: TIB_ConnectionEvent read FOnGainFocus
                                              write FOnGainFocus;     
    property OnLoseFocus: TIB_ConnectionEvent read FOnLoseFocus
                                              write FOnLoseFocus;
    property OnCustomizeDPB: TIB_CustomizeDBPEvent read FOnCustomizeDPB
                                                   write FOnCustomizeDPB;
{------------------------------------------------------------------------------}
    property _ConnectionLinkCount: integer read GetConnectionLinkCount
                                           write flag_junk_value
                                           stored false;
    property _TransactionCount: integer read GetTransactionCount
                                        write flag_junk_value
                                        stored false;
    property _StatementCount: integer read GetStatementCount
                                      write flag_junk_value
                                      stored false;
    property _DataSetCount: integer read GetDataSetCount
                                    write flag_junk_value
                                    stored false;
{------------------------------------------------------------------------------}
  end; { TIB_Connection }

// IMP_IB_CONNECTION
