
{                                                                              }
{ TIB_DataLink                                                                 }
{                                                                              }

  TIB_DataLinkEvent = procedure( Sender: TIB_DataLink;
                                 ADataSource: TIB_DataSource ) of object;

  TIB_ColumnDataLinkEvent = procedure( Sender: TIB_DataLink;
                                       ADataSource: TIB_DataSource;
                                       AField: TIB_Column ) of object;

  TIB_DataLink = class(TIB_Component)
  private
    FDataSource: TIB_DataSource;
    FReceiveFocus: boolean;
    FControl: TWinControl;
    FModifying: boolean;
    FSearchBuffer: string; 
    FSearchSaved: string;
    FSearchLast: string;
  { Control Property Storage }
    FControlIsModified: boolean;
    FControlIsReadOnly: boolean;
    FControlPreventsEditing: boolean;
    FControlPreventsInserting: boolean;
    FControlPreventsSearching: boolean;
  protected
  { Inherited Methods }
    procedure Loaded; override;
  { Property access methods }
    procedure SetDataSource( AValue: TIB_DataSource ); virtual;
    function GetDataSet: TIB_DataSet; virtual;
    function GetPrepared: boolean; virtual;
    function GetActive: boolean; virtual;
    function GetNeedToPost: boolean; virtual;
    function GetState: TIB_DataSetState; virtual;
    function GetRowState: TIB_RowState; virtual;
    function GetCanModify: boolean; virtual;
    function GetReadOnly: boolean; virtual;
    function GetPreventEditing: boolean; virtual;
    function GetPreventInserting: boolean; virtual;
    function GetPreventSearching: boolean; virtual;
    procedure SetSearchBuffer( AValue: string ); virtual;
    function GetSearchBuffer: string; virtual;
    function GetColor: TColor; virtual;
    function GetEditingColor: TColor; virtual;
    function GetInsertingColor: TColor; virtual;
    function GetSearchingColor: TColor; virtual;
    function GetInvalidColor: TColor; virtual;
    function GetActiveColor: TColor; virtual;
    procedure SetReceiveFocus( AValue: boolean ); virtual;
    function GetColorScheme: boolean; virtual;
    procedure SetModifying( AValue: boolean ); virtual;
    function GetModifying: boolean; virtual;
    function GetRowNum: longint; virtual;
  { Control Property Access }
    procedure SetControlIsModified( AValue: boolean ); virtual;
    procedure SetControlIsReadOnly( AValue: boolean ); virtual;
    procedure SetControlPreventsEditing( AValue: boolean ); virtual;
    procedure SetControlPreventsInserting( AValue: boolean ); virtual;
    procedure SetControlPreventsSearching( AValue: boolean ); virtual;
  { System Methods }
    procedure SysPrepareSQL; virtual;
    procedure SysBeforeUpdateDescriptors; virtual;
    procedure SysAfterUpdateDescriptors; virtual;
    procedure SysPreparedChanged; virtual;
    procedure SysBeforeExecute; virtual;
    procedure SysAfterExecute; virtual;
    procedure SysModifyingChanged; virtual;
    procedure SysActiveChanged; virtual;
    procedure SysStateChanged; virtual;
    procedure SysLayoutChanged; virtual;
    procedure SysDataChange( AField: TIB_Column ); virtual;
    procedure SysUpdateData( AField: TIB_Column ); virtual;
  { Focusing stuff }
    procedure DoReceiveFocus( DS: TIB_DataSource ); virtual;
  { Event Dispatch methods }
    procedure ProcessStatementEvent( AEvent: TIB_StatementEventType;
                                     Info: longint ); virtual;
    procedure DoBeforeAssignment; virtual;
    procedure DoAfterAssignment; virtual;
    procedure DoPrepareSQL; virtual;
    procedure DoDataChange( AField: TIB_Column ); virtual;
    procedure DoUpdateData( AField: TIB_Column ); virtual;
  public
  { Event Hooks }
    AfterAssignment: TIB_DataLinkEvent;
    AfterExecute: TIB_DataLinkEvent;
    BeforeAssignment: TIB_DataLinkEvent;
    BeforeExecute: TIB_DataLinkEvent;
    OnPreparedChanged: TIB_DataLinkEvent;
    OnPrepareSQL: TIB_DataLinkEvent;
    OnReceiveFocus: TIB_DataLinkEvent;
    OnModifyingChanged: TIB_DataLinkEvent;
    OnActiveChanged: TIB_DataLinkEvent;
    OnStateChanged: TIB_DataLinkEvent;
    OnLayoutChanged: TIB_DataLinkEvent;
    OnDataChange: TIB_ColumnDataLinkEvent;
    OnUpdateData: TIB_ColumnDataLinkEvent;
  { Inherited Methods }
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  { New methods }
    procedure Edit; dynamic;
    procedure Insert; dynamic;
    function Modify: boolean; dynamic;
    property Modifying: boolean read GetModifying;
    procedure SaveSearch; dynamic;
    procedure ClearSearch; dynamic;
    procedure RecallSearch; dynamic;
    procedure UpdateData; dynamic;
    procedure SetFocus; dynamic;
    procedure Reset; dynamic;
    procedure FocusControl( AField: TIB_ColumnRef ); dynamic;
  { Properties }
    property DataSource: TIB_DataSource read FDataSource write SetDataSource;
    property DataSet: TIB_DataSet read GetDataSet;
    property Prepared: boolean read GetPrepared;
    property Active: boolean read GetActive;
    property State: TIB_DataSetState read GetState;
    property RowState: TIB_RowState read GetRowState;
    property CanModify: boolean read GetCanModify;
    property Control: TWinControl read FControl write FControl;
    property Color: TColor read GetColor;
    property ReadOnly: boolean read GetReadOnly;
    property NeedToPost: boolean read GetNeedToPost;
    property PreventEditing: boolean read GetPreventEditing;
    property PreventInserting: boolean read GetPreventInserting;
    property PreventSearching: boolean read GetPreventSearching;
    property SearchBuffer: string read GetSearchBuffer write SetSearchBuffer;
    property ReceiveFocus: boolean read FReceiveFocus write SetReceiveFocus;
    property ColorScheme: boolean read GetColorScheme;
    property RowNum: longint read GetRowNum;
  { Control Property Storage }
    property ControlIsModified: boolean read FControlIsModified
                                        write SetControlIsModified;
    property ControlIsReadOnly: boolean read FControlIsReadOnly
                                        write SetControlIsReadOnly;
    property ControlPreventsEditing: boolean read FControlPreventsEditing
                                             write SetControlPreventsEditing;
    property ControlPreventsInserting: boolean read FControlPreventsInserting
                                              write SetControlPreventsInserting;
    property ControlPreventsSearching: boolean read FControlPreventsSearching
                                              write SetControlPreventsSearching;
  end;

// IMP_IB_DataLink
