
{                                                                              }
{ TIB_Statement                                                                }
{                                                                              }

  EIB_StatementError = class( EIB_Error );

  TIB_StatementType = ( stUnknown,
                        stSelect,
                        stInsert,
                        stUpdate,
                        stDelete,
                        stDDL,
                        stGetSegment,
                        stPutSegment,
                        stExecProcedure,
                        stStartTrans,
                        stCommit,
                        stRollback,
                        stSelectForUpdate,
                        stSetGenerator );

  TIB_StatementEventType = ( setBeforeAssignment,
                             setAfterAssignment,
                             setBeforeAllocate,
                             setAfterAllocate,
                             setBeforePrepare,
                             setAfterPrepare,
                             setBeforeUnprepare,
                             setAfterUnprepare,
                             setBeforeExecute,
                             setAfterExecute,
                             setBeforeExecDML,
                             setAfterExecDML,
                             setBeforeExecDDL,
                             setAfterExecDDL,
                             setPreparedChanged,
                             setActiveChanged,
                             setLayoutChanged,
                             setParamsStateChanged,
                             setParamsDataChange,
                             setParamsUpdateData,
                             setFieldsStateChanged,
                             setFieldsDataChange,
                             setFieldsUpdateData,
                             setInvalidateRows,
                             setFocusControl,
                             setOrderingChanged,
                             setOrderingLinkChanged,
                             setSearchingLinkChanged );

  TIB_StatementEvent = procedure( Sender: TIB_Statement ) of object;

  TIB_RowStateEvent = procedure( Sender: TIB_Statement;
                                 Row: TIB_Row ) of object;

  TIB_RowType  = ( rtParam, rtField );
  TIB_RowState = ( rsNone, rsUnmodified, rsModified );

  TIB_SQLStrings = class( TIB_StringList )
  private
    FStatement: TIB_Statement;
  protected
    procedure Assign( APersistent: TPersistent ); override;
    procedure AssignTo( APersistent: TPersistent ); override;
  end;

  TIB_Statement = class(TIB_Component)
  private
  { Property storage fields }
    FstHandle: isc_stmt_handle;
    FIB_Transaction: TIB_TransactionSingle; // For an internal instance.
    FIB_ConnectionLink: TIB_ConnectionStatementLink;
    FIB_TransactionLink: TIB_TransactionStatementLink;
    FIB_StatementLinkList: TList;
    FEnableLinkLevel: integer;
    FSQL: TStrings;
    FSQLKeyRelation: string;
    FSysKeyRelation: string;
    FPreparedSQL: string;
    FSysParamNames: TStrings;
    FOldParamValues: TStrings;
    FParamCheck: boolean;
    FSQLHistoryList: TStrings;
    FKeepSQLHistory: boolean;
    FSQLHistoryRef: integer;
    FStatementType: TIB_StatementType;
    FStatementPlan: string;
    FPrepared: boolean;
    FSQLValid: boolean;
    FParams: TIB_Row;
    FParamCount: smallint;
    FCursorFields: TIB_Row;
    FCursorFieldCount: smallint;
    FActive: boolean;
    FParamChar: char;
    FPreparingSQL: boolean;
    FRepreparingSQL: boolean;
  { Hints that appear in the TIB_NavigatorBar and other controls }
    FHints: TStrings;
  { Field attributes }
    FColumnAttributes: TStrings;
    FFieldsAlignment: TStrings;
    FFieldsDisplayFormat: TStrings;
    FFieldsDisplayLabel: TStrings;
    FFieldsDisplayWidth: TStrings;
    FFieldsEditMask: TStrings;
    FFieldsIndex: TStrings;
    FFieldsReadOnly: TStrings;
    FFieldsVisible: TStrings;
  { Optimization/System Flags }
    flag_statement_type_invalid: boolean;
    flag_statement_plan_invalid: boolean;
    flag_prepare_after_load: boolean;
    flag_junk_value: integer;
  { Event storage fields }
    FBeforeAllocate: TIB_StatementEvent;
    FAfterAllocate: TIB_StatementEvent;
    FBeforeDeallocate: TIB_StatementEvent;
    FAfterDeallocate: TIB_StatementEvent;
    FBeforePrepare: TIB_StatementEvent;
    FAfterPrepare: TIB_StatementEvent;
    FBeforeUnprepare: TIB_StatementEvent;
    FAfterUnprepare: TIB_StatementEvent;
    FBeforeExecute: TIB_StatementEvent;
    FAfterExecute: TIB_StatementEvent;
    FBeforeExecDDL: TIB_StatementEvent;
    FAfterExecDDL: TIB_StatementEvent;
    FBeforeExecDML: TIB_StatementEvent;
    FAfterExecDML: TIB_StatementEvent;
    FOnAllocatedChanged: TIB_StatementEvent;
    FOnPrepareSQL: TIB_StatementEvent;
    FOnPreparedChanged: TIB_StatementEvent;
    FOnActiveChanged: TIB_StatementEvent;
    FOnLayoutChanged: TIB_StatementEvent;
    FOnParamsStateChanged: TIB_RowStateEvent;
    FOnFieldsStateChanged: TIB_RowStateEvent;
  { Property Access Methods }
    function GetPdbHandle: pisc_db_handle;
    function GetPtrHandle: pisc_tr_handle;
    function GetPstHandle: pisc_stmt_handle;
    function GetStatementPlan: string;
    function GetStatementLinkCount: integer;
//    function GetStatementLinks( Index: integer ):TIB_StatementLinkBase;
    procedure SetSQLKeyRelation( AValue: string );
    function GetSQLHistoryCount: integer;
    function GetSQLHistory( Index: integer ): TStrings;
    procedure SetSQLHistoryRef( AValue: integer );
    function GetSQLHistoryRef: integer;
    function IsIB_TransactionStored: boolean;
    procedure UpdateAlignment;
    procedure UpdateDisplayFormat;
    procedure UpdateDisplayLabel;
    procedure UpdateDisplayWidth;
    procedure UpdateEditMask;
    procedure UpdateIndex;
    procedure UpdateReadOnly;
    procedure UpdateVisible;
    procedure OnSQLChange( Sender: TObject );
    procedure OnLayoutChange( Sender: TObject );
  protected
  { Used to flag an internally owned cursor for buffered datasets }
    FIgnoreLayoutChange: integer;
    FBindingCursor: TIB_Dataset;
  { Methods }
    procedure Notification( AComponent: TComponent;
                            Operation: TOperation); override;
    procedure SysUpdateLayout; virtual;
    procedure SetColumnAttributes( Value: TStrings ); virtual;
    procedure SetFieldsAlignment( Value: TStrings ); virtual;
    procedure SetFieldsDisplayFormat( Value: TStrings ); virtual;
    procedure SetFieldsDisplayLabel( Value: TStrings ); virtual;
    procedure SetFieldsDisplayWidth( Value: TStrings ); virtual;
    procedure SetFieldsEditMask( Value: TStrings ); virtual;
    procedure SetFieldsIndex( Value: TStrings ); virtual;
    procedure SetFieldsReadOnly( Value: TStrings ); virtual;
    procedure SetFieldsVisible( Value: TStrings ); virtual;
  { Hints for NavBar and other Bars }
    procedure SetHints( Value: TStrings ); virtual;
    property Hints: TStrings read FHints write SetHints;
  { Property Access Methods }
    function GetAllocated: boolean; virtual;
    procedure SetAllocated( Value: boolean ); virtual;
    procedure SetPrepared( Value: boolean ); virtual;
    procedure SetActive( Value: boolean ); virtual;
    function GetActive: boolean; virtual;
    procedure SetIB_Connection( AValue: TIB_Connection ); virtual;
    function GetIB_Connection: TIB_Connection; virtual;
    procedure SetIB_Transaction( AValue: TIB_Transaction ); virtual;
    function GetIB_Transaction: TIB_Transaction; virtual;
    procedure SetSQL( AValue: TStrings ); virtual;
    procedure SysSQLChange( Sender: TObject ); virtual;
    procedure SysLayoutChange( Sender: TObject ); virtual;
    function GetStatementType: TIB_StatementType; virtual;
    function GetFields: TIB_Row; virtual;
    function GetParams( Index: integer ): TIB_Column;
    function GetFieldCount: integer;
    function GetParamCount: integer;
  { Special Record data access methods }
    procedure SetParams( AValue: TIB_Row ); virtual;
    procedure SetCursorFields( AValue: TIB_Row ); virtual;
    procedure SetFields( AValue: TIB_Row ); virtual;
  { System Methods }
    function SysCheckConnection( RequestConnect: boolean ): boolean; virtual;
    function SysCheckTransaction( RequestStart: boolean ): boolean; virtual;
    procedure SysPrepareAfterLoad; virtual;
    function SysAllocate: boolean; virtual;
    procedure SysDeallocate; virtual;
    function SysPrepare: boolean; virtual;
    procedure SysExecPrepare; virtual;
    procedure SysClose; virtual;
    procedure SysUnprepare; virtual;
    procedure SysBeforePrepare; virtual;
    procedure SysAfterPrepare; virtual;
    procedure SysBeforeUnprepare; virtual;
    procedure SysAfterUnprepare; virtual;
    procedure SysExecute; virtual;
    procedure SysBeforeExecute; virtual;
    procedure SysAfterExecute; virtual;
    procedure SysBeforeExecuteForOutput; virtual;
    procedure SysAfterExecuteForOutput; virtual;
    procedure SysBeforeExecuteWithInput; virtual;
    procedure SysAfterExecuteWithInput; virtual;
    procedure SysExecuteImmediate( Statement: string; Param: PXSQLDA ); virtual;
    procedure SysExecImmed2( Statement: string; Param,
                                                Field: PXSQLDA ); virtual;
    procedure SysAllocatedChanged; virtual;
    procedure SysPreparedChanged; virtual;
    procedure SysActiveChanged; virtual;
    procedure SysLayoutChanged; virtual;
    procedure SysUpdateStatementType; virtual;
    procedure SysUpdateStatementPlan; virtual;
    procedure SysPrepareSQL; virtual;
    procedure SysBeforeUpdateDescriptors; virtual;
    procedure SysAfterUpdateDescriptors; virtual;
    procedure SysUpdateDescriptors; virtual;
    procedure SysExecUnknown; virtual;
    procedure SysExecSetGenerator; virtual;
    procedure SysExecDML; virtual;
    procedure SysExecStartTrans; virtual;
    procedure SysExecCommit; virtual;
    procedure SysExecRollback; virtual;
    procedure SysExecDDL; virtual;
    procedure SysExecSelect; virtual;
    procedure SysExecGetSegment; virtual;
    procedure SysExecPutSegment; virtual;
    procedure SysExecExecProcedure; virtual;
    procedure SysClearSQLHistory; virtual;
    procedure SysAddSQLHistory; virtual;
    procedure SysPrepareFailed; virtual;
    function SysGetRawSQL: string; virtual;
  { Row alteration event handlers }
    procedure SysParamsStateChanged( Sender: TIB_Row ); virtual;
    procedure SysBeforeParamsDataChange( Sender: TIB_Row;
                                         AField: TIB_Column ); virtual;
    procedure SysAfterParamsDataChange( Sender: TIB_Row;
                                        AField: TIB_Column ); virtual;
    procedure SysCursorFieldsStateChanged( Sender: TIB_Row ); virtual;
    procedure SysBeforeCursorFieldDataChange( Sender: TIB_Row;
                                              AField: TIB_Column ); virtual;
    procedure SysAfterCursorFieldDataChange( Sender: TIB_Row;
                                             AField: TIB_Column ); virtual;
    procedure SysFieldsStateChanged( Sender: TIB_Row ); virtual;
    procedure SysBeforeFieldDataChange( Sender: TIB_Row;
                                        AField: TIB_Column ); virtual;
    procedure SysAfterFieldDataChange( Sender: TIB_Row;
                                       AField: TIB_Column ); virtual;
  { Connection Link event handlers }
    procedure ProcessConnectionEvent( AEvent: TIB_ConnectionEventType );
                                                                        virtual;
  { Transaction Link event handlers }
    procedure ProcessTransactionEvent( AEvent: TIB_TransactionEventType );
                                                                        virtual;
  { System Methods }
    procedure API_Allocate; 
    procedure API_Deallocate; 
    function API_Prepare(     Text: PChar;
                          var InVar,
                              OutVar: smallint ): longint; 
    procedure API_Execute; 
    procedure API_Execute2; 
    procedure API_ExecuteImmediate( Statement:string; Param: PXSQLDA ); 
    procedure API_ExecImmed2( Statement: string; Param,
                                                 Field: PXSQLDA ); 
    procedure API_DSQL_SQL_INFO( var Items: array of Char;
                                 var Buffer: array of Char); 
  { Inherited methods }
    procedure Loaded; override;
  { Stored Internal Property Methods }
    procedure DefineProperties( Filer: TFiler ); override;
    procedure ReadOldParamData( Reader: TReader );
    procedure WriteOldParamData( Writer: TWriter );
  { Properties }
    property StatementLinkCount: integer read GetStatementLinkCount;
    property SQLValid: boolean read FSQLValid;
  { Properties to be published }
    property SQLKeyRelation: string read FSQLKeyRelation
                                    write SetSQLKeyRelation;
    property SQLHistoryCount: integer read GetSQLHistoryCount;
    property SQLHistory[ Index: integer ]: TStrings read GetSQLHistory;
    property SQLHistoryRef: integer read GetSQLHistoryRef
                                    write SetSQLHistoryRef;
    property KeepSQLHistory: boolean read FKeepSQLHistory
                                     write FKeepSQLHistory
                                     default false;
    property CursorFields: TIB_Row read FCursorFields;
    property CursorFieldCount: smallint read FCursorFieldCount;
  { Events to be published }
    property OnAllocatedChanged: TIB_StatementEvent read FOnAllocatedChanged
                                                    write FOnAllocatedChanged;
    property OnPrepareSQL: TIB_StatementEvent read FOnPrepareSQL
                                              write FOnPrepareSQL;
    property OnPreparedChanged: TIB_StatementEvent read FOnPreparedChanged
                                                   write FOnPreparedChanged;
    property OnActiveChanged: TIB_StatementEvent read FOnActiveChanged
                                                 write FOnActiveChanged;
    property OnLayoutChanged: TIB_StatementEvent read FOnLayoutChanged
                                                 write FOnLayoutChanged;
    property OnParamsStateChanged: TIB_RowStateEvent
        read FOnParamsStateChanged                                                 
       write FOnParamsStateChanged;                                                 
    property OnFieldsStateChanged: TIB_RowStateEvent
        read FOnFieldsStateChanged
       write FOnFieldsStateChanged;
  { Events }
    property BeforeAllocate:   TIB_StatementEvent read FBeforeAllocate
                                                  write FBeforeAllocate;
    property AfterAllocate:    TIB_StatementEvent read FAfterAllocate
                                                  write FAfterAllocate;
    property BeforeDeallocate: TIB_StatementEvent read FBeforeDeallocate
                                                  write FBeforeDeallocate;
    property AfterDeallocate:  TIB_StatementEvent read FAfterDeallocate
                                                  write FAfterDeallocate;
    property BeforePrepare:    TIB_StatementEvent read FBeforePrepare
                                                  write FBeforePrepare;
    property AfterPrepare:     TIB_StatementEvent read FAfterPrepare
                                                  write FAfterPrepare;
    property BeforeUnprepare:  TIB_StatementEvent read FBeforeUnprepare
                                                  write FBeforeUnprepare;
    property AfterUnprepare:   TIB_StatementEvent read FAfterUnprepare
                                                  write FAfterUnprepare;
    property BeforeExecute:    TIB_StatementEvent read FBeforeExecute
                                                  write FBeforeExecute;
    property AfterExecute:     TIB_StatementEvent read FAfterExecute
                                                  write FAfterExecute;
    property BeforeExecDDL:    TIB_StatementEvent read FBeforeExecDDL
                                                  write FBeforeExecDDL;
    property AfterExecDDL:     TIB_StatementEvent read FAfterExecDDL
                                                  write FAfterExecDDL;
    property BeforeExecDML:    TIB_StatementEvent read FBeforeExecDML
                                                  write FBeforeExecDML;
    property AfterExecDML:     TIB_StatementEvent read FAfterExecDML
                                                  write FAfterExecDML;
  public
  { Inherited Methods }
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
  { Procedures }
    procedure ProcessEvent( AEvent: TIB_StatementEvent ); virtual;
    procedure ProcessLinkEvent( AEvent: TIB_StatementEventType;
                                Info: longint ); virtual;
  { System Methods }
    procedure Allocate; virtual;
    procedure Deallocate; virtual;
    procedure Prepare; virtual;
    procedure Unprepare; virtual;
    procedure Execute; virtual;
    procedure ExecuteImmediate( Statement: string; Param: PXSQLDA ); virtual;
    procedure ExecImmed2( Statement: string; Param, Field: PXSQLDA ); virtual;
    procedure InvalidateSQL; dynamic;
    procedure DisableLinks; dynamic;
    procedure EnableLinks; dynamic;
    procedure UpdateLayout; virtual;
    function FieldByName( AFieldName: string ): TIB_Column;
    function ParamByName( AFieldName: string ): TIB_Column;
    function FindField( const FieldName: string ): TIB_Column;
    function FindParam( const FieldName: string ): TIB_Column;
    procedure GetFieldList( AStrings: TStrings );
    procedure GetParamList( AStrings: TStrings );
    procedure BeginLayout;
    procedure EndLayout;
  { New Properties }
    property Allocated: boolean read GetAllocated write SetAllocated;
    property Prepared: boolean read FPrepared write SetPrepared default false;
    property Active: boolean read GetActive write SetActive default false;
    property StatementType: TIB_StatementType read GetStatementType;
    property StatementPlan: string read GetStatementPlan;
    property stHandle: isc_stmt_handle read FstHandle;
    property PstHandle: pisc_stmt_handle read GetPstHandle;
    property PdbHandle: pisc_db_handle read GetPdbHandle;
    property PtrHandle: pisc_tr_handle read GetPtrHandle;
    property Fields: TIB_Row read GetFields;
    property FieldCount: integer read GetFieldCount;
    property ParamCount: integer read GetParamCount;
    property ParamChar: char read FParamChar write FParamChar default ':';
    property ParamCheck: boolean read FParamCheck
                                 write FParamCheck
                                 default true;
    property PreparingSQL: boolean read FPreparingSQL;
    property PreparedSQL: string read FPreparedSQL;
    property RepreparingSQL: boolean read FRepreparingSQL;
  { More properties }
    property FieldsAlignment: TStrings read FFieldsAlignment
                                       write SetFieldsAlignment;
    property FieldsDisplayFormat: TStrings read FFieldsDisplayFormat
                                           write SetFieldsDisplayFormat;
    property FieldsDisplayLabel: TStrings read FFieldsDisplayLabel
                                          write SetFieldsDisplayLabel;
    property FieldsDisplayWidth: TStrings read FFieldsDisplayWidth
                                          write SetFieldsDisplayWidth;
    property FieldsEditMask: TStrings read FFieldsEditMask
                                      write SetFieldsEditMask;
    property FieldsIndex: TStrings read FFieldsIndex
                                   write SetFieldsIndex;
    property FieldsReadOnly: TStrings read FFieldsReadOnly
                                      write SetFieldsReadOnly;
    property FieldsVisible: TStrings read FFieldsVisible
                                     write SetFieldsVisible;
  published
  { Published properties }
    property IB_Connection: TIB_Connection read GetIB_Connection
                                           write SetIB_Connection;
    property IB_Transaction: TIB_Transaction read GetIB_Transaction
                                             write SetIB_Transaction
                                             stored IsIB_TransactionStored;
    property ColumnAttributes: TStrings read FColumnAttributes
                                        write SetColumnAttributes;
    property Params: TIB_Row read FParams write SetParams;
    property SQL: TStrings read FSQL write SetSQL;
{------------------------------------------------------------------------------}
    property _StatementLinkCount: integer read GetStatementLinkCount
                                          write flag_junk_value
                                          stored false;
{------------------------------------------------------------------------------}
  end; { TIB_Statement }

{                                                                              }
{ TIB_DSQL                                                                     }
{                                                                              }

  TIB_DSQL = class( TIB_Statement )
  public
    property SQLHistoryRef;
    property SQLHistoryCount;
    property SQLHistory;
  published
  { Properties }
    property KeepSQLHistory;
    property ParamChar;
    property ParamCheck;
  { Events }
    property OnAllocatedChanged;
    property OnPreparedChanged;
    property OnActiveChanged;
    property OnLayoutChanged;
    property BeforeAllocate;
    property AfterAllocate;
    property BeforeDeallocate;
    property AfterDeallocate;
    property BeforePrepare;
    property AfterPrepare;
    property BeforeUnprepare;
    property AfterUnprepare;
    property BeforeExecute;
    property AfterExecute;
    property BeforeExecDDL;
    property AfterExecDDL;
    property BeforeExecDML;
    property AfterExecDML;
  end;

// IMP_IB_Statement
