
{                                                                              }
{ TIB_Transaction                                                              }
{                                                                              }

  TIB_TransactionState = ( tsNone,
                           tsInactivePending,
                           tsInactive,
                           tsActivePending,
                           tsActive,
                           tsSavePointPending,
                           tsCommitPending,
                           tsRollbackPending );

  TIB_TransactionEventType = ( tetBeforeAssignment,
                               tetAfterAssignment,
                               tetBeforeStart,
                               tetAfterStart,
                               tetBeforeCommit,
                               tetAfterCommit,
                               tetBeforeSavepoint,
                               tetAfterSavepoint,
                               tetBeforeRollback,
                               tetAfterRollback,
                               tetBeforeEnd,
                               tetAfterEnd,
                               tetOnStatusChanged );

  TIB_Isolation = ( tiConcurrency,  //  Read-Repeatable   .
                    tiConsistency,  //  Forced-Repeatable .
                    tiCommitted );  //  Read-Committed    .

  EIB_TransactionError = class(EIB_Error);

  TIB_TransactionEvent = procedure( Sender: TIB_Transaction ) of object;

  TIB_Transaction = class(TIB_Component)
  private
  { Utility Storage }
    flag_start_after_load: boolean;
    flag_req_start_after_load: boolean;
    flag_junk_value: integer;
  { Property storage fields }
    FtrHandle: isc_tr_handle;
    FConnectionLinkList: TList;
    FStatementList: TList;
    FDataSetList: TList;
    FTransactionLinkList: TList;
    FTransactionState: TIB_TransactionState;
    FReadOnly: boolean;
    FIsolation: TIB_Isolation;
    FRecVersion: boolean;
    FLockWait: boolean;
    FAutoCommit: boolean;
    FAutoStart: boolean;
    FAutoStartPending: boolean;
    FAnnounceFocus: boolean;
    FPostPendingCount: integer;
    FDDL_PostAll: boolean;
    FDDL_Commit: boolean;
  { Event storage fields }
    FBeforeStart:      TIB_TransactionEvent;
    FAfterStart:       TIB_TransactionEvent;
    FBeforeSavePoint:  TIB_TransactionEvent;
    FAfterSavePoint:   TIB_TransactionEvent;
    FBeforeCommit:     TIB_TransactionEvent;
    FAfterCommit:      TIB_TransactionEvent;
    FBeforeRollback:   TIB_TransactionEvent;
    FAfterRollback:    TIB_TransactionEvent;
    FBeforeEnd:        TIB_TransactionEvent;
    FAfterEnd:         TIB_TransactionEvent;
    FOnGainFocus:      TIB_TransactionEvent;
    FOnLoseFocus:      TIB_TransactionEvent;
  { Property Access Methods }
    procedure SetStarted( AValue: boolean );
    function GetStarted: boolean;
    function GetPtrHandle: pisc_tr_handle;
    function GetPdbHandles( Index: integer ): pisc_db_handle;
    function GetConnectionCount: integer;
    function GetConnection( Index: integer ): TIB_Connection;
    function GetConnectionIndex( Index: TIB_Connection ): integer;
    function GetStatementCount: integer;
    function GetStatement( Index: integer ): TIB_Statement;
    function GetDataSetCount: integer;
    function GetDataSet( Index: integer ): TIB_DataSet;
    function GetTransactionLinkCount: integer;
    function GetTransactionLink( Index: integer ): TObject;
  { Connection handling stuff }
    procedure SysAddConnection( NewConnection: TIB_Connection );
    procedure SysRemoveConnection( OldConnection: TIB_Connection );
    procedure SysRemoveAllConnections;
  protected
  { Property Access Methods }
    procedure SetTransactionState( AValue: TIB_TransactionState ); virtual;
    procedure SetAnnounceFocus( AValue: boolean ); virtual;
    procedure SetAutoStart( AValue: boolean ); virtual;
    procedure SetAutoCommit( AValue: boolean ); virtual;
    procedure SetReadOnly( AValue: boolean ); virtual;
    procedure SetIsolation( AValue: TIB_Isolation ); virtual;
    procedure SetRecVersion( AValue: boolean ); virtual;
    procedure SetLockWait( AValue: boolean ); virtual;
  { Inherited Methods }
    procedure Loaded; override;
  { System Methods }
    function SysStart: boolean; dynamic;
    procedure SysBeforeStart; dynamic;
    procedure SysFailedStart; dynamic;
    procedure SysAfterStart; dynamic;
    procedure SysReqAutoStart; dynamic;
    procedure SysExecAutoStart; dynamic;
    procedure SysStartAfterLoad; dynamic;
    procedure SysCheckDataSetsAfterLoad; dynamic;
    procedure SysCommit; dynamic;
    procedure SysExecCommit; dynamic;
    procedure SysSavePoint; dynamic;
    procedure SysRollback; dynamic;
    procedure SysExecRollback; dynamic;
    procedure SysBeforeEnd; dynamic;
    procedure SysAfterEnd; dynamic;
    procedure SysPostAll; dynamic;
    procedure SysCancelAll; dynamic;
    procedure SysClose; dynamic;
    procedure SysUnprepareStatements; dynamic;
    procedure SysAdjustPostPendingCount( AdjFactor: smallint ); dynamic;
    procedure SysBeforeExecDDL; dynamic;
    procedure SysAfterExecDDL; dynamic;
  { API Level methods }
    procedure API_Start; dynamic;
    procedure API_Commit;
    procedure API_SavePoint;
    procedure API_Rollback; 
  { Event Dispatch Methods }
    procedure DoBeforeStart; dynamic;
    procedure DoAfterStart; dynamic;
    procedure DoBeforeCommit; dynamic;
    procedure DoAfterCommit; dynamic;
    procedure DoBeforeSavePoint; dynamic;
    procedure DoAfterSavePoint; dynamic;
    procedure DoBeforeRollback; dynamic;
    procedure DoAfterRollback; dynamic;
    procedure DoBeforeEnd; dynamic;
    procedure DoAfterEnd; dynamic;
    procedure DoGainFocus; dynamic;
    procedure DoLoseFocus; dynamic;
  { Linked Event Dispatch Methods }
    procedure ProcessEvent( AEvent: TIB_TransactionEventType ); virtual;
    procedure ProcessConnectionEvent( Sender: TIB_Connection;
                                     AEvent: TIB_ConnectionEventType ); virtual;
  public
  { Inherited Methods }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  { Link management }
    procedure AddTransactionLink( ALink: TObject );
    procedure RemoveTransactionLink( ALink: TObject );
  { New methods }
    procedure Start; dynamic;
    procedure Activate; dynamic;
    procedure SavePoint; dynamic;
    procedure Commit; dynamic;
    procedure Rollback; dynamic;
    procedure Close; dynamic;
    function CloseWithConfirm: TModalResult; dynamic;
    procedure PostAll; dynamic;
    procedure CancelAll; dynamic;
    procedure SetFocus; dynamic;
    procedure UpdateStatus; dynamic;
  { Properties }
    property trHandle: isc_tr_handle read FtrHandle;
    property PtrHandle: pisc_tr_handle read GetPtrHandle;
    property TransactionState: TIB_TransactionState read FTransactionState;
    property AutoStartPending: boolean read FAutoStartPending;
    property ConnectionCount: integer read GetConnectionCount;
    property Connections[Index: integer]: TIB_Connection read GetConnection;
    property ConnectionIndex[Index: TIB_Connection]: integer
        read GetConnectionIndex;
    property StatementCount: integer read GetStatementCount;
    property Statements[Index: integer]: TIB_Statement read GetStatement;
    property DataSetCount: integer read GetDataSetCount;
    property DataSets[Index: integer]: TIB_DataSet read GetDataSet;
    property PdbHandles[Index: integer]: pisc_db_handle read GetPdbHandles;
    property PostPendingCount: integer read FPostPendingCount;
    property TransactionLinkCount: integer read GetTransactionLinkCount;
  published
  { Properties }
    property Started: boolean read GetStarted
                              write SetStarted
                              default false;
    property AutoCommit: boolean read FAutoCommit
                                 write SetAutoCommit
                                 default false;
    property AutoStart: boolean read FAutoStart
                                write SetAutoStart
                                default false;
    property DDL_PostAll: boolean read FDDL_PostAll
                                  write FDDL_PostAll
                                  default false;
    property DDL_Commit: boolean read FDDL_Commit
                                 write FDDL_Commit
                                 default false;
    property AnnounceFocus: boolean read FAnnounceFocus
                                    write SetAnnounceFocus
                                    default false;
    property ReadOnly: boolean read FReadOnly
                               write SetReadOnly
                               default false;
    property Isolation: TIB_Isolation read FIsolation
                                      write SetIsolation
                                      default tiConcurrency;
    property RecVersion: boolean read FRecVersion
                                 write SetRecVersion
                                 default false;
    property LockWait: boolean read FLockWait
                               write SetLockWait
                               default false;
{------------------------------------------------------------------------------}
    property _TransactionLinkCount: integer read GetTransactionLinkCount
                                            write flag_junk_value
                                            stored false;
    property _ConnectionCount: integer read GetConnectionCount
                                       write flag_junk_value
                                       stored false;
    property _StatementCount: integer read GetStatementCount
                                      write flag_junk_value
                                      stored false;
    property _DataSetCount: integer read GetDataSetCount
                                    write flag_junk_value
                                    stored false;
{------------------------------------------------------------------------------}
  { New Events }
    property BeforeStart:     TIB_TransactionEvent read FBeforeStart
                                                   write FBeforeStart;
    property AfterStart:      TIB_TransactionEvent read FAfterStart
                                                   write FAfterStart;
    property BeforeCommit:    TIB_TransactionEvent read FBeforeCommit
                                                   write FBeforeCommit;
    property AfterCommit:     TIB_TransactionEvent read FAfterCommit
                                                   write FAfterCommit;
    property BeforeSavePoint: TIB_TransactionEvent read FBeforeSavePoint
                                                   write FBeforeSavePoint;
    property AfterSavePoint:  TIB_TransactionEvent read FAfterSavePoint
                                                   write FAfterSavePoint;
    property BeforeRollback:  TIB_TransactionEvent read FBeforeRollback
                                                   write FBeforeRollback;
    property AfterRollback:   TIB_TransactionEvent read FAfterRollback
                                                   write FAfterRollback;
    property BeforeEnd:       TIB_TransactionEvent read FBeforeEnd
                                                   write FBeforeEnd;
    property AfterEnd:        TIB_TransactionEvent read FAfterEnd
                                                   write FAfterEnd;
    property OnGainFocus:     TIB_TransactionEvent read FOnGainFocus
                                                   write FOnGainFocus;
    property OnLoseFocus:     TIB_TransactionEvent read FOnLoseFocus
                                                   write FOnLoseFocus;
  end; { TIB_Transaction }

  TIB_TransactionSingle = class( TIB_Transaction )
  protected
    function GetIB_Connection: TIB_Connection; virtual;
    procedure SetIB_Connection( AValue: TIB_Connection ); virtual;
    procedure SysExecAutoStart; override;
    procedure API_Start; override;
  published
    property IB_Connection: TIB_Connection read GetIB_Connection
                                           write SetIB_Connection;
  end;

  TIB_TransactionDouble = class( TIB_Transaction )
  protected
    function GetIB_Connection1: TIB_Connection; virtual;
    procedure SetIB_Connection1( AValue: TIB_Connection ); virtual;
    function GetIB_Connection2: TIB_Connection; virtual;
    procedure SetIB_Connection2( AValue: TIB_Connection ); virtual;
    procedure SysExecAutoStart; override;
    procedure SysBeforeStart; override;
  published
    property IB_Connection1: TIB_Connection read GetIB_Connection1
                                            write SetIB_Connection1;
    property IB_Connection2: TIB_Connection read GetIB_Connection2
                                            write SetIB_Connection2;
  end;

  TIB_TransactionMultiple = class( TIB_Transaction )
  public
    procedure AddConnection( New: TIB_Connection ); virtual;
    procedure RemoveConnection( Old: TIB_Connection ); virtual;
  end;

// IMP_IB_Transaction
