unit savConf;

{this unit provieds the configuration-dialog for my screen saver sample
 you must change this whole unit to make your new saver work. Look out
 for comments in the file on special issues

 Version 1.0
 Author: Meik Weber

 IMPORTANT: See Copyright and disclaimer in README.TXT and Project Source}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Registry, savInit;

type
  TfrmConfig = class(TForm)
    Label1: TLabel;
    butOK: TButton;
    butCancel: TButton;
    SpeedBar: TTrackBar;
    Slow: TLabel;
    Fast: TLabel;
    Label2: TLabel;
    labSpeed: TLabel;
    Label3: TLabel;
    procedure butOKClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure butCancelClick(Sender: TObject);
    procedure SpeedBarChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    {it is very important that you leave this CreateParams intact, it is
     essential to making this window modal to the screen-properties-dialog}
    procedure CreateParams (var Params : TCreateParams); override;
  end;

var
  frmConfig: TfrmConfig;

implementation

{$R *.DFM}

{Click on OK saves my configuration to the registry.
 I checked the sample-savers from Microsoft (BEZIER.SCR) and found out
 that they still write lots of information to CONTROL.INI.
 I think it is better to write it to the Registry}
procedure TfrmConfig.butOKClick(Sender: TObject);
var
  Registry : TRegistry;
begin
  Registry := TRegistry.Create;
  try
    if Registry.OpenKey ('Software\Fade in fade out', true) then
      Registry.WriteInteger ('Speed', SpeedBar.Position);
  finally
    Registry.Free;
    Close {close this window and exit the config dialog}
  end;
end;

procedure TfrmConfig.CreateParams (var Params : TCreateParams);
begin {here comes the tricky part}
  inherited Createparams (Params);
  if ConfigParent <> 0 then begin
    {this is necessary to make myself a child window of the screen-properties-dialog,
     the effect is, when I click on my config-dialog, the screen-properties-dialog shows
     up behind my dialog}
    Params.WndParent := ConfigParent;
    EnableWindow (ConfigParent, false); {My dialog should be modal, so disable parent}
  end;
end;

procedure TfrmConfig.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if ConfigParent <> 0 then begin
    EnableWindow (ConfigParent, true); {as I made myself modal, I should undo so now}
    SetForegroundWindow (ConfigParent);
  end;
end;

procedure TfrmConfig.butCancelClick(Sender: TObject);
begin
  Close {close and exit without saving conig}
end;

procedure TfrmConfig.SpeedBarChange(Sender: TObject);
begin
  case SpeedBar.Position of
    0 : labSpeed.Caption := 'Extremely slow';
    1 : labSpeed.Caption := 'Very slow';
    2 : labSpeed.Caption := 'Slow';
    3 : labSpeed.Caption := 'Fast';
    4 : labSpeed.Caption := 'Very fast';
    5 : labSpeed.Caption := 'Extremely fast';
  end;
end;

procedure TfrmConfig.FormCreate(Sender: TObject);
var
  Registry : TRegistry;
  Speed : Integer;
begin
  {read config-values from registry and setup this SpeedBar}
  Registry := TRegistry.Create;
  try
    if Registry.OpenKey ('Software\Fade in fade out', false) then begin
      try
        Speed := Registry.ReadInteger ('Speed');
        if Speed < 0 then
          Speed := 0
        else if Speed > 5 then
          Speed := 5;
        SpeedBar.Position := Speed;
      except
        {ignore exceptions}
      end;
    end;
  finally
    Registry.Free;
    SpeedBarChange (Self) {call this once, to update the "labSpeed"}
  end;
end;

end.
