unit savRun;

{this unit implements the full-scree-mode of my screen saver
 look out for comments on special issues.

 BTW: I set the background color to this nice green, to ensure the
 background gets never painted. I thought it would be easy to detect
 the unwanted painting of this green.

 Version 1.0
 Author: Meik Weber

 IMPORTANT: See Copyright and disclaimer in README.TXT and Project Source}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry, savThr, ExtCtrls;

type
  TfrmFullScreen = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
  private
    {now comes my custom stuff}
    MyThread : TSaverThread;
    procedure CreateParams (var Params : TCreateParams); override;
    procedure WMSysCommand (var Msg : TWMSysCommand); message WM_SYSCOMMAND;
    procedure WMKeyDown (var Msg : TWMKeyDown); message WM_KEYDOWN;
    procedure WMSysKeyDown (var Msg : TWMSysKeyDown); message WM_SYSKEYDOWN;
    procedure WMEraseBkgnd (var Msg : TMessage); message WM_ERASEBKGND;
    procedure Closeifok;
    procedure CloseSaver;
    procedure MouseEvent;
    { Private declarations }
  public
    IgnoreMouse : Integer;
    { Public declarations }
  end;

var
  frmFullScreen: TfrmFullScreen;

implementation

{$R *.DFM}

{this shows how many mouse-events (Button-Up and Move) are ignored before the
 saver gets closed. It should be at least 2, or you may observe that your saver
 does not start (or closes immediately after activation)}
const TicksToIgnore = 6;

procedure TfrmFullScreen.WMKeyDown;
begin
  Closeifok;
end;

procedure TfrmFullScreen.WMSysKeyDown;
begin
  Closeifok;
end;

{this function checks for password if necessary and closes the screen-saver if
 pwd was correct}
procedure TfrmFullScreen.CloseIfOk;
var
  hLib : THandle;
  P : function (Parent : THandle) : Boolean; stdcall;
  SysDir : String;
  NewLength : Integer;
  Registry : TRegistry;
begin
  {check if password must be checked}
  Registry := TRegistry.Create;
  Registry.RootKey := HKEY_CURRENT_USER;
  if Registry.OpenKey ('Control Panel\desktop', false) then begin
    if Registry.ReadInteger ('ScreenSaveUsePassword') <> 0 then begin
      ShowCursor (True); {activate mouse}
      {load library PASSWORD.CPL from system-directory}
      SetLength (SysDir, MAX_PATH);
      NewLength := GetSystemDirectory (PChar (SysDir), MAX_PATH);
      SetLength (SysDir, NewLength);
      if (length (SysDir) > 0) and (SysDir [length (SysDir)] <> '\') then
        SysDir := SysDir + '\';
      hLib := LoadLibrary (PChar (SysDir + 'PASSWORD.CPL'));
      if hLib = 0 then
        CloseSaver {if dll not found, simply close the saver}
      else begin
        {show the password verification dialog}
        P := GetProcAddress (hLib, 'VerifyScreenSavePwd');
        if P (Handle) then
          CloseSaver;
        FreeLibrary (hLib); {unload PASSWORD.CPL}
      end;
      IgnoreMouse := TicksToIgnore;
      ShowCursor (False); {disable the mouse again}
    end else
      CloseSaver
  end else
    CloseSaver;
end;

procedure TfrmFullScreen.FormCreate(Sender: TObject);
var
  Dummy : Integer;
begin
  {make window full-screen}
  top := 0;
  Left := 0;
  Width := Screen.Width;
  Height := Screen.Height;
  SystemParametersInfo (97, Word (True), @Dummy, 0); {no alt-tab or ctrl-alt-del any more}
  IgnoreMouse := TicksToIgnore;
  ShowCursor (false); {hide mouse cursor}
end;

procedure TfrmFullScreen.CloseSaver;
var
  Dummy : Integer;
begin
  MyThread.Terminate;
  MyThread := nil; {make sure it does not get terminated twice or so, it frees itself}
  SystemParametersInfo (97, Word (False), @Dummy, 0); {enable every Win95 system key}
  ShowCursor (True);
  Close
end;

procedure TfrmFullScreen.FormShow(Sender: TObject);
begin
  {if the graphics is not yet running, create the thread}
  if not assigned (MyThread) then
    MyThread := TSaverThread.Create (Canvas, Rect (0, 0, Width, Height));
end;

procedure TfrmFullScreen.FormPaint (Sender: TObject);
var
  R : TRect;
begin
  {this gets called when the password change dialog is moved over the screen,
   so you should do something intelligent here and repaint the screen parts
   which must be repainted}
  MyThread.ForceRect := Canvas.ClipRect; {tell the saver to repaint this rectangle}
end;

procedure TfrmFullScreen.WMEraseBkGnd;
begin
  Msg.Result := 0; {oh no, never write the background}
end;

procedure TfrmFullScreen.WMSysCommand;
begin
  if Msg.cmdType = SC_SCREENSAVE then
    Msg.Result := 1 {do not activate the saver more than once}
  else
    inherited;
end;

procedure TfrmFullScreen.CreateParams (var Params : TCreateParams);
begin
  inherited CreateParams (Params);
  Params.ExStyle := Params.ExStyle or WS_EX_TOPMOST; {make the window topmost}
end;

procedure TfrmFullScreen.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Dummy : Integer;
begin
  if assigned (MyThread) then begin
    MyThread.Terminate;
    MyThread := nil;
    SystemParametersInfo (97, Word (False), @Dummy, 0); {enable every Win95 system key}
    ShowCursor (True);
  end;
end;

{this is the generic mouse event. It is best to ignore some mouse events,
 instead of terminating or showing the password dlg. I figured out that it
 is best to ignore 2 mouse events after startup or pwd-dialog, even if a lot
 of documentation I saw tells me I should only ignore 1 event}
procedure TfrmFullScreen.MouseEvent;
begin
  if IgnoreMouse > 0 then
    Dec (ignoreMouse)
  else
    CloseIfOK
end;

procedure TfrmFullScreen.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseEvent {you may recieve one after startup (think for the test button)}
end;

procedure TfrmFullScreen.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  MouseEvent {you will recieve at least one mosue move after startup}
end;

procedure TfrmFullScreen.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  CloseIfOk; {mousedown is not sent to me accidently, this always closes}
end;

{the timer serves as a Mouse-Reset. It is here to avoid the screen saver
being deactivated by bumps on the table if the user has a high-resultion-mouse.
This constantly resets the 'IgnoreMouse' variable every 2 seconds.
The value of 'TicksToIgnore' controls how nice your saver behaves to these
unwanted mouse movements. You may want to change the time-value or you may want
to disable or delete this timer as MS Screen Savers don't do stuff like this,
but I found it very useful}
procedure TfrmFullScreen.Timer1Timer(Sender: TObject);
begin
  IgnoreMouse := TicksToIgnore;
end;

end.
