unit savThr;

{this is the heart of my screen-saver. Look for comments.

 Version 1.0
 Author: Meik Weber

 IMPORTANT: See Copyright and disclaimer in README.TXT and Project Source}

interface

uses Windows, Classes, Graphics, Forms, SysUtils, Messages, Dialogs, Registry, savInit;

{this is the basic class, only change it if you want to change the logic of working}
type TSaverBasics = class (TThread)
                    private
                      FForceRect : TRect;
                      procedure SetForceRect (ARect : TRect);
                    protected
                      Canvas     : TCanvas; {the canvas to draw the saver action to}
                      Left,                 {dimensions of the display window      }
                      Top,                  {all drawing must be done inside       }
                      Width,                {the here specified dimensions         }
                      Height     : Integer;
                      Background : TBitmap; {bitmap holding the background         }
                      SleepTime  : Integer; {time to go to sleep for Sleep function}

                      procedure AStep; virtual;
                      {override this and implement a step in your screen saver, this gets
                       called again and again until the saver finishes}
                      procedure CleanUp; virtual;
                      {this makes is called immediately before ending the execute function}
                    public
                      constructor Create (ACanvas : TCanvas; DrawRect : TRect); virtual;
                      {ACanvas holds the canvas to draw the saver action to and the
                       DrawRect holds the wanted rectangle, usually this is the full
                       canvas}
                      procedure Execute; override;
                      {don't care about this}
                      procedure ClearForceRect;
                      {call this if you have redrawn the ForceRect to set it to 0,0,0,0}
                      property ForceRect : TRect read FForceRect write SetForceRect;
                      {this holds a ForceRect and gets called whenever a rectangle needs
                       painting. This is caused by a password dialog dragged over the window
                       or windows being dragged over the preview window. Make sure you draw
                       the ForceRect in your AStep function to make your saver look nice. But
                       you can also ignore this}
                    end;

{these functions are very essentiyl, study them, don't change them}
procedure ExecPreview;
procedure ExecSetPwd;

{this is the sample saver used in this project, you should change the implementation
 of this class to your own needs}
type TSaverThread = class (TSaverBasics)
                      Mode      : Integer;
                      Direction : Integer;
                      Position  : Integer;
                      constructor Create (ACanvas : TCanvas; DrawRect : TRect); override;
                      procedure AStep; override;
                    end;

implementation

{Part 1 abstract definition of the TSaverBasics thread}
{Part 2 definition of the TSaverThread for this sample\
{Part 3 implementation of the preview window for the TSaverThread clas}
{Part 4 implementation of the password change dialog}
{I think, the only thing you have to do is to change Part 2 to your
 new screen saver display options}


{Part 1 abstract definition of the TSaverBasics thread}

constructor TSaverBasics.Create (ACanvas : TCanvas; DrawRect : TRect);
var
  BackgroundCanvas : TCanvas;
  DC : hDC;
begin
  {set local variables}
  Canvas := ACanvas;
  Left   := DrawRect.Left;
  Top    := DrawRect.Top;
  Width  := DrawRect.Right  - DrawRect.Left;
  Height := DrawRect.Bottom - DrawRect.Left;
  FForceRect := DrawRect;
  SleepTime := 1;
  {get the background bitmap}
  Background := TBitmap.Create;
  Background.Width := Width;
  Background.Height := Height;
  DC := GetDC (0);
  BackgroundCanvas := TCanvas.Create;
  BackgroundCanvas.Handle := DC;
  {stretch the bitmap to the display size (it could be much smaller (preview))}
  Background.Canvas.CopyRect (Rect (0, 0, Width, Height), BackgroundCanvas,
                              Rect (0, 0, Screen.Width, Screen.Height));
  BackgroundCanvas.Free;
  ReleaseDC (0, DC);

  {initialize the thread as starting immediately}
  inherited Create (false);

  {and now initialize some thread properties to the appropriate values}
  FreeOnTerminate := True;
end;

procedure TSaverBasics.CleanUp;
begin {nothing here, as you see}
end;

procedure TSaverBasics.SetForceRect (ARect : TRect);
begin {only enlare this rectangle, never shrink it}
  if FForceRect.Left > ARect.Left then
    FForceRect.Left := ARect.Left;
  if FForceRect.Top > ARect.Top then
    FForceRect.Top := ARect.Top;
  if FForceRect.Right < ARect.Right then
    FForceRect.Right := ARect.Right;
  if FForceRect.Bottom < ARect.Bottom then
    FForceRect.Bottom := ARect.Bottom;
end;

procedure TSaverBasics.ClearForceRect;
begin
  FForceRect := Rect (0, 0, 0, 0)
end;

procedure TSaverBasics.Execute;
begin
  if assigned (Canvas) then begin {do nothing if there is no canvas to draw to}
    while not Terminated do begin
      Synchronize (AStep);
      if SleepTime > 0 then
        Sleep (SleepTime);
    end;
  end;
  CleanUp;
  Background.Free; {free the background bitmap}
end;

procedure TSaverBasics.AStep;
begin
  {this class implements nothing. It is implemented in descendant classes}
end;

{Part 2 definition of the TSaverThread for this sample}
{this is only commented to show what to override to TSaverBasics}

const ModeClearH = 0;
      ModeShowV  = 3;

constructor TSaverThread.Create (ACanvas : TCanvas; DrawRect : TRect);
var
  Registry : TRegistry;
  Speed : Integer;
begin
  inherited Create (ACanvas, DrawRect);
  Position  := 0;
  Mode      := ModeClearH;
  Direction := 1;
  SleepTime := 5;
  Registry  := TRegistry.Create;
  try
    if Registry.OpenKey ('Software\Fade in fade out', false) then begin
      Speed := Registry.ReadInteger ('Speed');
      if Speed < 0 then
        Speed := 0
      else if Speed > 5 then
        Speed := 5;
      case Speed of
        0 : SleepTime := 1000;
        1 : SleepTime := 500;
        2 : SleepTime := 100;
        3 : SleepTime := 20;
        4 : SleepTime := 5;
        5 : SleepTime := 0;
      end;
    end;
  except
    {ignore everything}
  end;
  Registry.Free; {as every exception is ignored, it should get here, even without a finally}
end;

{my screen saver is here:
 every step, clears one line from display, or shows one line
 it repaints certain rectangles given in ForceRect according to actual
 display status of my saver. move the password dialog around the running
 saver and you will see that it works}
procedure TSaverThread.AStep;
var
  R : TRect;
begin
  Position := Position + Direction;
  case Mode of
    modeClearH : begin
                   if ForceRect.Bottom > 0 then begin
                     if Direction < 0 then begin
                       R := Rect (Left, Top, Left + Width, Position);
                       Canvas.CopyRect (R, Background.Canvas, R);
                       Canvas.Brush.Color := clBlack;
                       R.Top  := Position;
                       R.Bottom := Top + Height;
                       Canvas.FillRect (R);
                     end else begin
                       R := Rect (Left, Position, Left + Width, Top + Height);
                       Canvas.CopyRect (R, Background.Canvas, R);
                       Canvas.Brush.Color := clBlack;
                       R.Top := Top;
                       R.Bottom := Position;
                       Canvas.FillRect (R);
                     end;
                     {after repainting the required parts of screen,
                      set the ForceRect to 0}
                     ClearForceRect;
                   end;
                   Canvas.Pen.Color := clBlack;
                   Canvas.MoveTo (Left, Position-Direction);
                   Canvas.LineTo (Left + Width, Position-Direction);
                   if (Position = Height) or (Position = 0) then begin
                     Canvas.MoveTo (Left, Position);
                     Canvas.LineTo (Left + Width, Position);
                     Direction := - Direction;
                     if Direction < 0 then
                       Position := Width
                     else
                       Position := 0;
                     mode := modeShowV;
                   end else begin
                     Canvas.Pen.Color := clRed;
                     Canvas.MoveTo (Left, Position);
                     Canvas.LineTo (Left + Width, Position);
                   end;
                 end;
    modeShowV  : begin
                   if ForceRect.Bottom > 0 then begin
                     if Direction < 0 then begin
                       R := Rect (Position, Top, Left + Width, Top + Height);
                       Canvas.CopyRect (R, Background.Canvas, R);
                       Canvas.Brush.Color := clBlack;
                       R.Left := Left;
                       R.Right := Position;
                       Canvas.FillRect (R);
                     end else begin
                       R := Rect (Left, Top, Position, Top + Height);
                       Canvas.CopyRect (R, Background.Canvas, R);
                       Canvas.Brush.Color := clBlack;
                       R.Left := Position;
                       R.Right := Left + Width;
                       Canvas.FillRect (R);
                     end;
                     ClearForceRect;
                   end;
                   R := Rect (Position-Direction, Top, Position-Direction+1, Top+Height);
                   Canvas.CopyRect (R, Background.Canvas, R);
                   if (Position = 0) or (Position = Width) then begin
                     if Direction < 0 then
                       Position := Height
                     else
                       Position := 0;
                     mode := modeClearH;
                   end else begin
                     Canvas.Pen.Color := clRed;
                     Canvas.MoveTo (Position, Top);
                     Canvas.LineTo (Position, Top + Height);
                   end;
                 end;
  end;
  ClearForceRect;
end;

{Part 3 implementation of the preview window for the TSaverThread clas}

var
  MyThread : TSaverThread;
  PrevRect : TRect;

{this is the window-proc for the preview window, it only implements 3 messages}
function MyWndProc (Wnd : HWnd; Msg : Integer; wParam : Word; lParam : Integer) : Integer; far; stdcall;
begin {I don't know if it is ok to quit on WM_CLOSE, but I had no problems doing so}
  if (Msg = WM_DESTROY) or (Msg = WM_CLOSE) then
    PostMessage (Wnd, WM_QUIT, 0, 0)
  else if Msg = WM_PAINT then {on paint is always the complete background now}
    MyThread.ForceRect := PrevRect;
  DefWindowProc (Wnd, Msg, wParam, lParam);
end;

{This executes the preview window,
 and goes in a message loop to keep it running}
procedure ExecPreview;
var
  PreviewCanvas : TCanvas;
  WndClass : TWndClass;
  Atom     : TAtom;
  DC       : hDC;
  MyWnd    : hWnd;
  Msg      : TMsg;
begin
  {create a new window class}
  with WndClass do begin
    style := CS_PARENTDC;
    lpfnWndProc := @MyWndProc;
    cbClsExtra := 0;
    cbWndExtra := 0;
    hIcon := 0;
    hCursor := 0;
    hbrBackground := 0;
    lpszMenuName := nil;
    lpszClassName := 'Delphi2ScreenSaverPreview';
  end;
  WndClass.hInstance := hInstance;
  Atom := Windows.RegisterClass (WndClass);
  {get some info on parent window}
  GetWindowRect (ParamHandle, PrevRect);
  PrevRect.Right := PrevRect.Right - PrevRect.Left;
  PrevRect.Bottom := PrevRect.Bottom - PrevREct.Top;
  PrevRect.Left := 0;
  PrevRect.Top := 0;
  {and now create the window as child of the window given in ParamHandle}
  MyWnd := CreateWindow ('Delphi2ScreenSaverPreview', 'FunSaver', WS_CHILD or WS_DISABLED or WS_VISIBLE, 0, 0, PrevRect.Right, PrevRect.Bottom,
                         ParamHandle, 0, hInstance, nil);
  {get the DC for the new created window}
  DC := GetDC (MyWnd);
  PreviewCanvas := TCanvas.Create;
  PreviewCanvas.Handle := DC;
  MyThread := TSaverThread.Create (PreviewCanvas, PrevRect);
  {and now do the message loop, I think I cannot use any predifined loop here}
  while GetMessage(Msg, 0, 0, 0) do begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
  MyThread.Terminate;
  PreviewCanvas.Free;
  {I don't care about other resources here, as my app terminates immediately and
   Win95 cleans up after me}
end;

{Part 4 implementation of the password change dialog}

procedure ExecSetPwd;
var
  hLib : THandle;
  P : function (a : PChar; ParentHandle : THandle; b, c : Integer) : Integer; stdcall;
  SysDir : String;
  NewLength : Integer;
  i : Integer;
begin
  SetLength (SysDir, MAX_PATH);
  NewLength := GetSystemDirectory (PChar (SysDir), MAX_PATH);
  SetLength (SysDir, NewLength);
  if (length (SysDir) > 0) and (SysDir [length (SysDir)] <> '\') then
    SysDir := SysDir + '\';
  hLib := LoadLibrary (PChar (SysDir + 'MPR.DLL'));
  if hLib <> 0 then begin
    P := GetProcAddress (hLib, 'PwdChangePasswordA');
    if assigned (P) then
      P ('SCRSAVE', ParamHandle, 0, 0);
    FreeLibrary (hLib);
  end;
end;

end.
